@@ -17,7 +17,6 @@ use std::error::Error as _;
 use std::io;
 use std::io::Write as _;
 use std::iter;
-use std::process::ExitCode;
 use std::str;
 use std::sync::Arc;
 
@@ -29,6 +28,7 @@ use jj_lib::config::ConfigGetError;
 use jj_lib::config::ConfigLoadError;
 use jj_lib::config::ConfigMigrateError;
 use jj_lib::dsl_util::Diagnostics;
+use jj_lib::evolution::WalkPredecessorsError;
 use jj_lib::fileset::FilePatternParseError;
 use jj_lib::fileset::FilesetParseError;
 use jj_lib::fileset::FilesetParseErrorKind;
@@ -417,6 +417,16 @@ impl From<TransactionCommitError> for CommandError {
     }
 }
 
+impl From<WalkPredecessorsError> for CommandError {
+    fn from(err: WalkPredecessorsError) -> Self {
+        match err {
+            WalkPredecessorsError::Backend(err) => err.into(),
+            WalkPredecessorsError::OpStore(err) => err.into(),
+            WalkPredecessorsError::CycleDetected(_) => internal_error(err),
+        }
+    }
+}
+
 impl From<DiffEditError> for CommandError {
     fn from(err: DiffEditError) -> Self {
         user_error_with_message("Failed to edit diff", err)
@@ -442,6 +452,9 @@ impl From<ConflictResolveError> for CommandError {
             ConflictResolveError::Io(err) => err.into(),
             _ => {
                 let hint = match &err {
+                    ConflictResolveError::ConflictTooComplicated { .. } => {
+                        Some("Edit the conflict markers manually to resolve this.".to_owned())
+                    }
                     ConflictResolveError::ExecutableConflict { .. } => {
                         Some("Use `jj file chmod` to update the executable bit.".to_owned())
                     }
@@ -508,7 +521,6 @@ impl From<TrailerParseError> for CommandError {
 mod git {
     use jj_lib::git::GitExportError;
     use jj_lib::git::GitFetchError;
-    use jj_lib::git::GitFetchPrepareError;
     use jj_lib::git::GitImportError;
     use jj_lib::git::GitPushError;
     use jj_lib::git::GitRemoteManagementError;
@@ -562,23 +574,11 @@ jj currently does not support partial clones. To use jj with this repository, tr
                     "Run `jj git remote rename` to give a different name.",
                 ),
                 GitFetchError::InvalidBranchPattern(_) => user_error(err),
-                #[cfg(feature = "git2")]
-                GitFetchError::Git2(err) => map_git2_error(err),
                 GitFetchError::Subprocess(_) => user_error(err),
             }
         }
     }
 
-    impl From<GitFetchPrepareError> for CommandError {
-        fn from(err: GitFetchPrepareError) -> Self {
-            match err {
-                #[cfg(feature = "git2")]
-                GitFetchPrepareError::Git2(err) => map_git2_error(err),
-                GitFetchPrepareError::UnexpectedBackend(_) => user_error(err),
-            }
-        }
-    }
-
     impl From<GitPushError> for CommandError {
         fn from(err: GitPushError) -> Self {
             match err {
@@ -587,8 +587,6 @@ jj currently does not support partial clones. To use jj with this repository, tr
                     err,
                     "Run `jj git remote rename` to give a different name.",
                 ),
-                #[cfg(feature = "git2")]
-                GitPushError::Git2(err) => map_git2_error(err),
                 GitPushError::Subprocess(_) => user_error(err),
                 GitPushError::UnexpectedBackend(_) => user_error(err),
             }
@@ -612,26 +610,6 @@ jj currently does not support partial clones. To use jj with this repository, tr
             user_error(err)
         }
     }
-
-    #[cfg(feature = "git2")]
-    fn map_git2_error(err: git2::Error) -> CommandError {
-        if err.class() == git2::ErrorClass::Ssh {
-            let hint = if err.code() == git2::ErrorCode::Certificate
-                && std::env::var_os("HOME").is_none()
-            {
-                "The HOME environment variable is not set, and might be required for Git to \
-                 successfully load certificates. Try setting it to the path of a directory that \
-                 contains a `.ssh` directory."
-            } else {
-                "Jujutsu uses libssh2, which doesn't respect ~/.ssh/config. Does `ssh -F \
-                 /dev/null` to the host work?"
-            };
-
-            user_error_with_hint(err, hint)
-        } else {
-            user_error(err)
-        }
-    }
 }
 
 impl From<RevsetEvaluationError> for CommandError {
@@ -931,23 +909,20 @@ fn template_parse_error_hint(err: &TemplateParseError) -> Option<String> {
 
 const BROKEN_PIPE_EXIT_CODE: u8 = 3;
 
-pub(crate) fn handle_command_result(ui: &mut Ui, result: Result<(), CommandError>) -> ExitCode {
-    try_handle_command_result(ui, result).unwrap_or_else(|_| ExitCode::from(BROKEN_PIPE_EXIT_CODE))
+pub(crate) fn handle_command_result(ui: &mut Ui, result: Result<(), CommandError>) -> u8 {
+    try_handle_command_result(ui, result).unwrap_or(BROKEN_PIPE_EXIT_CODE)
 }
 
-fn try_handle_command_result(
-    ui: &mut Ui,
-    result: Result<(), CommandError>,
-) -> io::Result<ExitCode> {
+fn try_handle_command_result(ui: &mut Ui, result: Result<(), CommandError>) -> io::Result<u8> {
     let Err(cmd_err) = &result else {
-        return Ok(ExitCode::SUCCESS);
+        return Ok(0);
     };
     let err = &cmd_err.error;
     let hints = &cmd_err.hints;
     match cmd_err.kind {
         CommandErrorKind::User => {
             print_error(ui, "Error: ", err, hints)?;
-            Ok(ExitCode::from(1))
+            Ok(1)
         }
         CommandErrorKind::Config => {
             print_error(ui, "Config error: ", err, hints)?;
@@ -956,23 +931,23 @@ fn try_handle_command_result(
                 "For help, see https://jj-vcs.github.io/jj/latest/config/ or use `jj help -k \
                  config`."
             )?;
-            Ok(ExitCode::from(1))
+            Ok(1)
         }
         CommandErrorKind::Cli => {
             if let Some(err) = err.downcast_ref::<clap::Error>() {
                 handle_clap_error(ui, err, hints)
             } else {
                 print_error(ui, "Error: ", err, hints)?;
-                Ok(ExitCode::from(2))
+                Ok(2)
             }
         }
         CommandErrorKind::BrokenPipe => {
             // A broken pipe is not an error, but a signal to exit gracefully.
-            Ok(ExitCode::from(BROKEN_PIPE_EXIT_CODE))
+            Ok(BROKEN_PIPE_EXIT_CODE)
         }
         CommandErrorKind::Internal => {
             print_error(ui, "Internal error: ", err, hints)?;
-            Ok(ExitCode::from(255))
+            Ok(255)
         }
     }
 }
@@ -1033,7 +1008,7 @@ fn print_error_hints(ui: &Ui, hints: &[ErrorHint]) -> io::Result<()> {
     Ok(())
 }
 
-fn handle_clap_error(ui: &mut Ui, err: &clap::Error, hints: &[ErrorHint]) -> io::Result<ExitCode> {
+fn handle_clap_error(ui: &mut Ui, err: &clap::Error, hints: &[ErrorHint]) -> io::Result<u8> {
     let clap_str = if ui.color() {
         err.render().ansi().to_string()
     } else {
@@ -1050,7 +1025,7 @@ fn handle_clap_error(ui: &mut Ui, err: &clap::Error, hints: &[ErrorHint]) -> io:
     match err.kind() {
         clap::error::ErrorKind::DisplayHelp | clap::error::ErrorKind::DisplayVersion => {
             write!(ui.stdout(), "{clap_str}")?;
-            return Ok(ExitCode::SUCCESS);
+            return Ok(0);
         }
         _ => {}
     }
@@ -1058,7 +1033,7 @@ fn handle_clap_error(ui: &mut Ui, err: &clap::Error, hints: &[ErrorHint]) -> io:
     // Skip the first source error, which should be printed inline.
     print_error_sources(ui, err.source().and_then(|err| err.source()))?;
     print_error_hints(ui, hints)?;
-    Ok(ExitCode::from(2))
+    Ok(2)
 }
 
 /// Prints diagnostic messages emitted during parsing.
