@@ -19,10 +19,9 @@ use criterion::measurement::Measurement;
 use criterion::BatchSize;
 use criterion::BenchmarkGroup;
 use criterion::BenchmarkId;
-use jj_lib::revset;
 use jj_lib::revset::DefaultSymbolResolver;
-use jj_lib::revset::RevsetExpression;
 use jj_lib::revset::SymbolResolverExtension;
+use jj_lib::revset::UserRevsetExpression;
 
 use super::new_criterion;
 use super::CriterionArgs;
@@ -80,14 +79,13 @@ fn bench_revset<M: Measurement>(
     revset: &RevisionArg,
 ) -> Result<(), CommandError> {
     writeln!(ui.status(), "----------Testing revset: {revset}----------")?;
-    let expression = revset::optimize(
-        workspace_command
-            .parse_revset(ui, revset)?
-            .expression()
-            .clone(),
-    );
+    let expression = workspace_command
+        .parse_revset(ui, revset)?
+        .expression()
+        .clone();
     // Time both evaluation and iteration.
-    let routine = |workspace_command: &WorkspaceCommandHelper, expression: Rc<RevsetExpression>| {
+    let routine = |workspace_command: &WorkspaceCommandHelper,
+                   expression: Rc<UserRevsetExpression>| {
         // Evaluate the expression without parsing/evaluating short-prefixes.
         let repo = workspace_command.repo().as_ref();
         let symbol_resolver =
