@@ -17,7 +17,7 @@ use std::io::Write as _;
 use std::path::Path;
 use std::process::Stdio;
 
-use clap_complete::ArgValueCandidates;
+use clap_complete::ArgValueCompleter;
 use itertools::Itertools as _;
 use jj_lib::backend::CommitId;
 use jj_lib::backend::FileId;
@@ -33,6 +33,7 @@ use jj_lib::repo_path::RepoPathUiConverter;
 use jj_lib::settings::UserSettings;
 use jj_lib::store::Store;
 use pollster::FutureExt as _;
+use tokio::io::AsyncReadExt as _;
 use tracing::instrument;
 
 use crate::cli_util::CommandHelper;
@@ -108,7 +109,7 @@ pub(crate) struct FixArgs {
         long,
         short,
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::mutable_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_mutable),
     )]
     source: Vec<RevisionArg>,
     /// Fix only these paths
@@ -143,15 +144,15 @@ pub(crate) fn cmd_fix(
         .to_matcher();
 
     let mut tx = workspace_command.start_transaction();
-    let parallel_fixer = ParallelFileFixer::new(|store, file_to_fix| {
-        fix_one_file(&workspace_root, &tools_config, store, file_to_fix)
+    let mut parallel_fixer = ParallelFileFixer::new(|store, file_to_fix| {
+        fix_one_file(&workspace_root, &tools_config, store, file_to_fix).block_on()
     });
     let summary = fix_files(
         root_commits,
         &matcher,
         args.include_unchanged_files,
         tx.repo_mut(),
-        &parallel_fixer,
+        &mut parallel_fixer,
     )?;
     writeln!(
         ui.status(),
@@ -174,7 +175,7 @@ pub(crate) fn cmd_fix(
 ///
 /// TODO: Better error handling so we can tell the user what went wrong with
 /// each failed input.
-fn fix_one_file(
+async fn fix_one_file(
     workspace_root: &Path,
     tools_config: &ToolsConfig,
     store: &Store,
@@ -190,8 +191,10 @@ fn fix_one_file(
         // subsequent matching tool gets its input from the previous matching tool's
         // output.
         let mut old_content = vec![];
-        let mut read = store.read_file(&file_to_fix.repo_path, &file_to_fix.file_id)?;
-        read.read_to_end(&mut old_content)?;
+        let mut read = store
+            .read_file(&file_to_fix.repo_path, &file_to_fix.file_id)
+            .await?;
+        read.read_to_end(&mut old_content).await?;
         let new_content = matching_tools.fold(old_content.clone(), |prev_content, tool_config| {
             match run_tool(
                 workspace_root,
@@ -210,7 +213,7 @@ fn fix_one_file(
             // TODO: send futures back over channel
             let new_file_id = store
                 .write_file(&file_to_fix.repo_path, &mut new_content.as_slice())
-                .block_on()?;
+                .await?;
             return Ok(Some(new_file_id));
         }
     }
