@@ -14,13 +14,13 @@
 
 use clap_complete::ArgValueCandidates;
 use jj_lib::git;
-use jj_lib::repo::Repo;
+use jj_lib::ref_name::RemoteNameBuf;
+use jj_lib::repo::Repo as _;
 
 use crate::cli_util::CommandHelper;
 use crate::command_error::CommandError;
 use crate::complete;
 use crate::git_util::absolute_git_url;
-use crate::git_util::get_git_repo;
 use crate::ui::Ui;
 
 /// Set the URL of a Git remote
@@ -28,7 +28,7 @@ use crate::ui::Ui;
 pub struct GitRemoteSetUrlArgs {
     /// The remote's name
     #[arg(add = ArgValueCandidates::new(complete::git_remotes))]
-    remote: String,
+    remote: RemoteNameBuf,
     /// The desired URL or path for `remote`
     ///
     /// Local path will be resolved to absolute form.
@@ -42,9 +42,7 @@ pub fn cmd_git_remote_set_url(
     args: &GitRemoteSetUrlArgs,
 ) -> Result<(), CommandError> {
     let workspace_command = command.workspace_helper(ui)?;
-    let repo = workspace_command.repo();
-    let git_repo = get_git_repo(repo.store())?;
     let url = absolute_git_url(command.cwd(), &args.url)?;
-    git::set_remote_url(&git_repo, &args.remote, &url)?;
+    git::set_remote_url(workspace_command.repo().store(), &args.remote, &url)?;
     Ok(())
 }
