@@ -14,7 +14,7 @@
 
 use std::collections::HashMap;
 
-use clap_complete::ArgValueCandidates;
+use clap_complete::ArgValueCompleter;
 use indexmap::IndexSet;
 use itertools::Itertools as _;
 use jj_lib::backend::CommitId;
@@ -59,7 +59,7 @@ pub(crate) struct ParallelizeArgs {
     /// Revisions to parallelize
     #[arg(
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::mutable_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_mutable),
     )]
     revisions: Vec<RevisionArg>,
 }
@@ -77,19 +77,18 @@ pub(crate) fn cmd_parallelize(
         .parse_union_revsets(ui, &args.revisions)?
         .evaluate_to_commits()?
         .try_collect()?;
-    workspace_command.check_rewritable(target_commits.iter().ids())?;
-
-    let mut tx = workspace_command.start_transaction();
 
     // New parents for commits in the target set. Since commits in the set are now
     // supposed to be independent, they inherit the parent's non-target parents,
     // recursively.
     let mut new_target_parents: HashMap<CommitId, Vec<CommitId>> = HashMap::new();
+    let mut needs_rewrite = Vec::new();
     for commit in target_commits.iter().rev() {
         let mut new_parents = vec![];
         for old_parent in commit.parent_ids() {
             if let Some(grand_parents) = new_target_parents.get(old_parent) {
                 new_parents.extend_from_slice(grand_parents);
+                needs_rewrite.push(commit.id());
             } else {
                 new_parents.push(old_parent.clone());
             }
@@ -97,6 +96,9 @@ pub(crate) fn cmd_parallelize(
         new_target_parents.insert(commit.id().clone(), new_parents);
     }
 
+    workspace_command.check_rewritable(needs_rewrite)?;
+    let mut tx = workspace_command.start_transaction();
+
     // If a commit outside the target set has a commit in the target set as parent,
     // then - after the transformation - it should also have that commit's
     // parents as direct parents, if those commits are also in the target set.
