@@ -12,7 +12,6 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use clap_complete::ArgValueCandidates;
 use clap_complete::ArgValueCompleter;
 use indoc::formatdoc;
 use itertools::Itertools as _;
@@ -33,6 +32,7 @@ use crate::command_error::user_error;
 use crate::command_error::user_error_with_hint;
 use crate::command_error::CommandError;
 use crate::complete;
+use crate::description_util::add_trailers;
 use crate::description_util::combine_messages_for_editing;
 use crate::description_util::description_template;
 use crate::description_util::edit_description;
@@ -68,7 +68,7 @@ pub(crate) struct SquashArgs {
         long,
         short,
         value_name = "REVSET",
-        add = ArgValueCandidates::new(complete::mutable_revisions)
+        add = ArgValueCompleter::new(complete::revset_expression_mutable),
     )]
     revision: Option<RevisionArg>,
     /// Revision(s) to squash from (default: @)
@@ -76,7 +76,7 @@ pub(crate) struct SquashArgs {
         long, short,
         conflicts_with = "revision",
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::mutable_revisions),
+        add = ArgValueCompleter::new(complete::revset_expression_mutable),
     )]
     from: Vec<RevisionArg>,
     /// Revision to squash into (default: @)
@@ -85,7 +85,7 @@ pub(crate) struct SquashArgs {
         conflicts_with = "revision",
         visible_alias = "to",
         value_name = "REVSET",
-        add = ArgValueCandidates::new(complete::mutable_revisions),
+        add = ArgValueCompleter::new(complete::revset_expression_mutable),
     )]
     into: Option<RevisionArg>,
     /// The description to use for squashed revision (don't open editor)
@@ -176,18 +176,43 @@ pub(crate) fn cmd_squash(
     )? {
         let mut commit_builder = squashed.commit_builder.detach();
         let new_description = match description {
-            SquashedDescription::Exact(description) => description,
-            SquashedDescription::UseDestination => destination.description().to_owned(),
+            SquashedDescription::Exact(description) => {
+                if description.is_empty() {
+                    description
+                } else {
+                    commit_builder.set_description(description);
+                    add_trailers(ui, &tx, &commit_builder)?
+                }
+            }
+            SquashedDescription::UseDestination => {
+                if destination.description().is_empty() {
+                    destination.description().to_owned()
+                } else {
+                    commit_builder.set_description(destination.description());
+                    add_trailers(ui, &tx, &commit_builder)?
+                }
+            }
             SquashedDescription::Combine => {
                 let abandoned_commits = &squashed.abandoned_commits;
                 if let Some(description) = try_combine_messages(abandoned_commits, &destination) {
-                    description
+                    if description.is_empty() {
+                        description
+                    } else {
+                        commit_builder.set_description(description);
+                        add_trailers(ui, &tx, &commit_builder)?
+                    }
                 } else {
-                    let intro = "Enter a description for the combined commit.";
-                    let combined = combine_messages_for_editing(abandoned_commits, &destination);
+                    let combined = combine_messages_for_editing(
+                        ui,
+                        &tx,
+                        abandoned_commits,
+                        &destination,
+                        &commit_builder,
+                    )?;
                     // It's weird that commit.description() contains "JJ: " lines, but works.
                     commit_builder.set_description(combined);
                     let temp_commit = commit_builder.write_hidden()?;
+                    let intro = "Enter a description for the combined commit.";
                     let template = description_template(ui, &tx, intro, &temp_commit)?;
                     edit_description(&text_editor, &template)?
                 }
