@@ -33,8 +33,9 @@ use crate::ui::Ui;
 ///
 /// This includes:
 ///
-///  * The working copy commit and its (first) parent, and a summary of the
-///    changes between them
+///  * The working copy commit and its parents, and a summary of the changes in
+///    the working copy (compared to the merged parents)
+///  * Conflicts in the working copy
 ///  * [Conflicted bookmarks]
 ///
 /// [Conflicted bookmarks]:
@@ -114,7 +115,7 @@ pub(crate) fn cmd_status(
 
         let template = workspace_command.commit_summary_template();
         write!(formatter, "Working copy  (@) : ")?;
-        formatter.with_label("working_copy", |fmt| template.format(wc_commit, fmt))?;
+        template.format(wc_commit, formatter)?;
         writeln!(formatter)?;
         for parent in wc_commit.parents() {
             let parent = parent?;
