@@ -287,8 +287,8 @@ impl UnresolvedConfigEnv {
                 Self::warn_for_deprecated_path(
                     ui,
                     path.as_path(),
-                    "~/Library/Application Support",
-                    "~/.config",
+                    "~/Library/Application Support/jj",
+                    "~/.config/jj",
                 );
                 paths.push(path);
             }
@@ -298,8 +298,8 @@ impl UnresolvedConfigEnv {
                 Self::warn_for_deprecated_path(
                     ui,
                     path.as_path(),
-                    "~/Library/Application Support",
-                    "~/.config",
+                    "~/Library/Application Support/jj",
+                    "~/.config/jj",
                 );
                 paths.push(path);
             }
@@ -347,6 +347,10 @@ impl ConfigEnv {
                     // Library/Preferences is supposed to be exclusively plists
                     s.data_dir()
                 })
+                .filter(|data_dir| {
+                    // User might've purposefully set their config dir to the deprecated one
+                    Some(data_dir) != config_dir.as_ref()
+                })
         } else {
             None
         };
@@ -714,27 +718,6 @@ pub fn default_config_migrations() -> Vec<ConfigMigrationRule> {
         ),
         // TODO: Delete in jj 0.34+
         ConfigMigrationRule::rename_value("diff.format", "ui.diff.format"),
-        // TODO: Delete with the `git.subprocess` setting.
-        #[cfg(not(feature = "git2"))]
-        ConfigMigrationRule::custom(
-            |layer| {
-                let Ok(Some(subprocess)) = layer.look_up_item("git.subprocess") else {
-                    return false;
-                };
-                subprocess.as_bool() == Some(false)
-            },
-            |_| Ok("jj was compiled without `git.subprocess = false` support".into()),
-        ),
-        // TODO: Delete with the `git.subprocess` setting.
-        ConfigMigrationRule::custom(
-            |layer| {
-                let Ok(Some(subprocess)) = layer.look_up_item("git.subprocess") else {
-                    return false;
-                };
-                subprocess.as_bool() == Some(true) && layer.source != ConfigSource::Default
-            },
-            |_| Ok("`git.subprocess = true` is now the default".into()),
-        ),
         // TODO: Delete in jj 0.35.0+
         ConfigMigrationRule::rename_update_value(
             "ui.default-description",
@@ -747,6 +730,17 @@ pub fn default_config_migrations() -> Vec<ConfigMigrationRule> {
                 Ok(format!(r#""{escaped}""#).into())
             },
         ),
+        // TODO: Delete in jj 0.36+
+        ConfigMigrationRule::rename_value("ui.diff.tool", "ui.diff-formatter"),
+        // TODO: Delete in jj 0.36+
+        ConfigMigrationRule::rename_update_value(
+            "ui.diff.format",
+            "ui.diff-formatter",
+            |old_value| {
+                let value = old_value.as_str().ok_or("expected a string")?;
+                Ok(format!(":{value}").into())
+            },
+        ),
     ]
 }
 
@@ -789,6 +783,17 @@ impl CommandNameAndArgs {
         }
     }
 
+    /// Returns command string only if the underlying type is a string.
+    ///
+    /// Use this to parse enum strings such as `":builtin"`, which can be
+    /// escaped as `[":builtin"]`.
+    pub fn as_str(&self) -> Option<&str> {
+        match self {
+            CommandNameAndArgs::String(s) => Some(s),
+            CommandNameAndArgs::Vec(_) | CommandNameAndArgs::Structured { .. } => None,
+        }
+    }
+
     /// Returns process builder configured with this.
     pub fn to_command(&self) -> Command {
         let empty: HashMap<&str, &str> = HashMap::new();
