@@ -20,6 +20,7 @@ use std::sync::Arc;
 
 use itertools::Itertools as _;
 use jj_lib::backend::BackendError;
+use jj_lib::backend::CopyId;
 use jj_lib::backend::MergedTreeId;
 use jj_lib::backend::TreeValue;
 use jj_lib::config::ConfigGetError;
@@ -253,7 +254,7 @@ impl DiffEditor {
         conflict_marker_style: ConflictMarkerStyle,
     ) -> Result<Self, MergeToolConfigError> {
         let args = editor_args_from_settings(ui, settings, "ui.diff-editor")?;
-        let tool = if let CommandNameAndArgs::String(name) = &args {
+        let tool = if let Some(name) = args.as_str() {
             DiffTool::get_tool_config(settings, name)?
         } else {
             None
@@ -388,7 +389,7 @@ impl MergeEditor {
         conflict_marker_style: ConflictMarkerStyle,
     ) -> Result<Self, MergeToolConfigError> {
         let args = editor_args_from_settings(ui, settings, "ui.merge-editor")?;
-        let tool = if let CommandNameAndArgs::String(name) = &args {
+        let tool = if let Some(name) = args.as_str() {
             MergeTool::get_tool_config(settings, name)?
         } else {
             None
@@ -464,8 +465,11 @@ fn pick_conflict_side(
         let file = &merge_tool_file.file;
         let file_id = file.ids.get_add(add_index).unwrap();
         let executable = file.executable.expect("should have been resolved");
-        let new_tree_value =
-            Merge::resolved(file_id.clone().map(|id| TreeValue::File { id, executable }));
+        let new_tree_value = Merge::resolved(file_id.clone().map(|id| TreeValue::File {
+            id,
+            executable,
+            copy_id: CopyId::placeholder(),
+        }));
         tree_builder.set_or_remove(merge_tool_file.repo_path.clone(), new_tree_value);
     }
     tree_builder.write_tree(tree.store())
@@ -516,6 +520,7 @@ mod tests {
                     0,
                 ],
                 diff_invocation_mode: Dir,
+                diff_do_chdir: true,
                 edit_args: [
                     "$left",
                     "$right",
@@ -546,6 +551,7 @@ mod tests {
                     0,
                 ],
                 diff_invocation_mode: Dir,
+                diff_do_chdir: true,
                 edit_args: [
                     "--edit",
                     "args",
@@ -592,6 +598,7 @@ mod tests {
                     0,
                 ],
                 diff_invocation_mode: Dir,
+                diff_do_chdir: true,
                 edit_args: [
                     "$left",
                     "$right",
@@ -618,6 +625,7 @@ mod tests {
                     0,
                 ],
                 diff_invocation_mode: Dir,
+                diff_do_chdir: true,
                 edit_args: [
                     "-l",
                     "$left",
@@ -646,6 +654,7 @@ mod tests {
                     0,
                 ],
                 diff_invocation_mode: Dir,
+                diff_do_chdir: true,
                 edit_args: [
                     "--diff",
                     "$left",
@@ -678,6 +687,7 @@ mod tests {
                     0,
                 ],
                 diff_invocation_mode: Dir,
+                diff_do_chdir: true,
                 edit_args: [
                     "--edit",
                     "args",
@@ -711,6 +721,7 @@ mod tests {
                     0,
                 ],
                 diff_invocation_mode: Dir,
+                diff_do_chdir: true,
                 edit_args: [
                     "$left",
                     "$right",
@@ -736,6 +747,7 @@ mod tests {
                     0,
                 ],
                 diff_invocation_mode: Dir,
+                diff_do_chdir: true,
                 edit_args: [
                     "$left",
                     "$right",
@@ -792,6 +804,7 @@ mod tests {
                     0,
                 ],
                 diff_invocation_mode: Dir,
+                diff_do_chdir: true,
                 edit_args: [
                     "$left",
                     "$right",
@@ -848,6 +861,7 @@ mod tests {
                     0,
                 ],
                 diff_invocation_mode: Dir,
+                diff_do_chdir: true,
                 edit_args: [
                     "$left",
                     "$right",
@@ -881,6 +895,7 @@ mod tests {
                     0,
                 ],
                 diff_invocation_mode: Dir,
+                diff_do_chdir: true,
                 edit_args: [
                     "$left",
                     "$right",
@@ -917,6 +932,7 @@ mod tests {
                     0,
                 ],
                 diff_invocation_mode: Dir,
+                diff_do_chdir: true,
                 edit_args: [
                     "$left",
                     "$right",
