@@ -97,7 +97,7 @@ impl Template for Signature {
         }
         if !self.email.is_empty() {
             write!(formatter, "<")?;
-            let email: Email = self.email.clone().into();
+            let email = Email(self.email.clone());
             email.format(formatter)?;
             write!(formatter, ">")?;
         }
@@ -108,12 +108,6 @@ impl Template for Signature {
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub struct Email(pub String);
 
-impl From<String> for Email {
-    fn from(value: String) -> Self {
-        Self(value)
-    }
-}
-
 impl Template for Email {
     fn format(&self, formatter: &mut TemplateFormatter) -> io::Result<()> {
         let (local, domain) = text_util::split_email(&self.0);
@@ -445,10 +439,32 @@ pub trait TemplatePropertyExt: TemplateProperty {
     {
         Box::new(self)
     }
+
+    /// Converts this property into wrapped (or tagged) type.
+    ///
+    /// Use `W::wrap_property()` if the self type is known to be boxed.
+    fn into_dyn_wrapped<'a, W>(self) -> W
+    where
+        Self: Sized + 'a,
+        W: WrapTemplateProperty<'a, Self::Output>,
+    {
+        W::wrap_property(self.into_dyn())
+    }
 }
 
 impl<P: TemplateProperty + ?Sized> TemplatePropertyExt for P {}
 
+/// Wraps template property of type `O` in tagged type.
+///
+/// This is basically `From<BoxedTemplateProperty<'a, O>>`, but is restricted to
+/// property types.
+#[diagnostic::on_unimplemented(
+    message = "the template property of type `{O}` cannot be wrapped in `{Self}`"
+)]
+pub trait WrapTemplateProperty<'a, O>: Sized {
+    fn wrap_property(property: BoxedTemplateProperty<'a, O>) -> Self;
+}
+
 /// Adapter that wraps literal value in `TemplateProperty`.
 pub struct Literal<O>(pub O);
 
@@ -720,13 +736,14 @@ impl<'a, C: Clone> TemplateRenderer<'a, C> {
         }
     }
 
-    /// Returns renderer that will format template with the given `label`.
+    /// Returns renderer that will format template with the given `labels`.
     ///
     /// This is equivalent to wrapping the content template with `label()`
-    /// function. For example, `content.labeled("foo").labeled("bar")` can be
-    /// expressed as `label("bar", label("foo", content))` in template.
-    pub fn labeled(mut self, label: impl Into<String>) -> Self {
-        self.labels.insert(0, label.into());
+    /// function. For example,
+    /// `content.labeled(["foo", "bar"]).labeled(["baz"])` can be expressed as
+    /// `label("baz", label("foo bar", content))` in template.
+    pub fn labeled<S: Into<String>>(mut self, labels: impl IntoIterator<Item = S>) -> Self {
+        self.labels.splice(0..0, labels.into_iter().map(Into::into));
         self
     }
 
