@@ -27,8 +27,8 @@ fn test_commit_with_description_from_cli() {
     // Description applies to the current working-copy (not the new one)
     work_dir.run_jj(["commit", "-m=first"]).success();
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
-    @  e8ea92a8b6b3
-    ○  fa15625b4a98 first
+    @  eb9fd2ab82e7
+    ○  68a505386f93 first
     ◆  000000000000
     [EOF]
     ");
@@ -47,8 +47,8 @@ fn test_commit_with_editor() {
     std::fs::write(&edit_script, ["dump editor0", "write\nmodified"].join("\0")).unwrap();
     work_dir.run_jj(["commit"]).success();
     insta::assert_snapshot!(get_log_output(&work_dir), @r"
-    @  a57b2c95fb75
-    ○  159271101e05 modified
+    @  2094c8f2e360
+    ○  a7ba1eb73836 modified
     ◆  000000000000
     [EOF]
     ");
@@ -95,6 +95,58 @@ fn test_commit_with_editor_avoids_unc() {
     assert_eq!(edited_path, dunce::simplified(&edited_path));
 }
 
+#[test]
+fn test_commit_with_empty_description_from_cli() {
+    let test_env = TestEnvironment::default();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    let output = work_dir.run_jj(["commit", "-m="]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
+    @  51b556e22ca0
+    ○  cc8ff2284a8c
+    ◆  000000000000
+    [EOF]
+    ");
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Working copy  (@) now at: rlvkpnrz 51b556e2 (empty) (no description set)
+    Parent commit (@-)      : qpvuntsm cc8ff228 (empty) (no description set)
+    [EOF]
+    ");
+}
+
+#[test]
+fn test_commit_with_empty_description_from_editor() {
+    let mut test_env = TestEnvironment::default();
+    let edit_script = test_env.set_up_fake_editor();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    // Check that the text file gets initialized and leave it untouched.
+    std::fs::write(&edit_script, ["dump editor0"].join("\0")).unwrap();
+    let output = work_dir.run_jj(["commit"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
+    @  51b556e22ca0
+    ○  cc8ff2284a8c
+    ◆  000000000000
+    [EOF]
+    ");
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor0")).unwrap(),
+        @r#"JJ: Lines starting with "JJ:" (like this one) will be removed."#
+    );
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Hint: The commit message was left empty.
+    If this was not intentional, run `jj undo` to restore the previous state.
+    Or run `jj desc @-` to add a description to the parent commit.
+    Working copy  (@) now at: rlvkpnrz 51b556e2 (empty) (no description set)
+    Parent commit (@-)      : qpvuntsm cc8ff228 (empty) (no description set)
+    [EOF]
+    ");
+}
+
 #[test]
 fn test_commit_interactive() {
     let mut test_env = TestEnvironment::default();
@@ -116,7 +168,7 @@ fn test_commit_interactive() {
 
     insta::assert_snapshot!(
         std::fs::read_to_string(test_env.env_root().join("instrs")).unwrap(), @r"
-    You are splitting the working-copy commit: qpvuntsm 4219467e add files
+    You are splitting the working-copy commit: qpvuntsm d849dc34 add files
 
     The diff initially shows all changes. Adjust the right side until it shows the
     contents you want for the first commit. The remainder will be included in the
@@ -155,10 +207,10 @@ fn test_commit_interactive() {
 
     let output = work_dir.run_jj(["log", "--summary"]);
     insta::assert_snapshot!(output, @r"
-    @  mzvwutvl test.user@example.com 2001-02-03 08:05:11 21b846a6
+    @  mzvwutvl test.user@example.com 2001-02-03 08:05:11 9b0176ab
     │  (no description set)
     │  A file2
-    ○  qpvuntsm test.user@example.com 2001-02-03 08:05:11 7d156390
+    ○  qpvuntsm test.user@example.com 2001-02-03 08:05:11 6e6fa925
     │  add files
     │  A file1
     ◆  zzzzzzzz root() 00000000
@@ -194,8 +246,8 @@ fn test_commit_interactive_with_paths() {
     let output = work_dir.run_jj(["commit", "-i", "file1", "file2"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy  (@) now at: kkmpptxz f3e6062e (no description set)
-    Parent commit (@-)      : rlvkpnrz 9453cb28 edit
+    Working copy  (@) now at: kkmpptxz 50f426df (no description set)
+    Parent commit (@-)      : rlvkpnrz eb640375 edit
     [EOF]
     ");
 
@@ -211,14 +263,14 @@ fn test_commit_interactive_with_paths() {
 
     let output = work_dir.run_jj(["log", "--summary"]);
     insta::assert_snapshot!(output, @r"
-    @  kkmpptxz test.user@example.com 2001-02-03 08:05:09 f3e6062e
+    @  kkmpptxz test.user@example.com 2001-02-03 08:05:09 50f426df
     │  (no description set)
     │  M file2
     │  M file3
-    ○  rlvkpnrz test.user@example.com 2001-02-03 08:05:09 9453cb28
+    ○  rlvkpnrz test.user@example.com 2001-02-03 08:05:09 eb640375
     │  edit
     │  A file1
-    ○  qpvuntsm test.user@example.com 2001-02-03 08:05:08 497ed465
+    ○  qpvuntsm test.user@example.com 2001-02-03 08:05:08 ff687a2f
     │  (no description set)
     │  A file2
     │  A file3
@@ -241,12 +293,12 @@ fn test_commit_with_default_description() {
     work_dir.run_jj(["commit"]).success();
 
     insta::assert_snapshot!(get_log_output(&work_dir), @r#"
-    @  c65242099289
-    ○  573b6df51aea TESTED=TODO
+    @  cba559ac1a48
+    ○  7276dfff8027 TESTED=TODO
     ◆  000000000000
     [EOF]
     ------- stderr -------
-    Warning: Deprecated config: ui.default-description is updated to template-aliases.default_commit_description = '"\n\nTESTED=TODO\n"'
+    Warning: Deprecated user-level config: ui.default-description is updated to template-aliases.default_commit_description = '"\n\nTESTED=TODO\n"'
     [EOF]
     "#);
     insta::assert_snapshot!(
@@ -395,8 +447,8 @@ fn test_commit_paths_warning() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Warning: The given paths do not match any file: file3
-    Working copy  (@) now at: rlvkpnrz d1872100 (no description set)
-    Parent commit (@-)      : qpvuntsm fa15625b (empty) first
+    Working copy  (@) now at: rlvkpnrz 4c6f0146 (no description set)
+    Parent commit (@-)      : qpvuntsm 68a50538 (empty) first
     [EOF]
     ");
 
@@ -464,8 +516,8 @@ fn test_commit_trailers() {
     let output = work_dir.run_jj(["commit", "-m=first"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy  (@) now at: rlvkpnrz e5635290 (empty) (no description set)
-    Parent commit (@-)      : qpvuntsm 78139a3f first
+    Working copy  (@) now at: rlvkpnrz 0c0495f3 (empty) (no description set)
+    Parent commit (@-)      : qpvuntsm ae86ffd4 first
     [EOF]
     ");
 
@@ -481,8 +533,8 @@ fn test_commit_trailers() {
     let output = work_dir.run_jj(["commit", "--config=user.email=foo@bar.org", "-m=second"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy  (@) now at: zsuskuln 3b39fdb9 (empty) (no description set)
-    Parent commit (@-)      : rlvkpnrz 1e30b4ab (empty) second
+    Working copy  (@) now at: zsuskuln fd73eac2 (empty) (no description set)
+    Parent commit (@-)      : rlvkpnrz 6e69e833 (empty) second
     [EOF]
     ");
 
@@ -499,8 +551,8 @@ fn test_commit_trailers() {
     let output = work_dir.run_jj(["commit", "--config=user.email=foo@bar.org"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy  (@) now at: royxmykx 1dd4f5c9 (empty) (no description set)
-    Parent commit (@-)      : zsuskuln 4ca072aa (empty) Reviewed-by: foo@bar.org
+    Working copy  (@) now at: royxmykx dac9709c (empty) (no description set)
+    Parent commit (@-)      : zsuskuln d9ced309 (empty) Reviewed-by: foo@bar.org
     [EOF]
     ");
 
@@ -509,7 +561,7 @@ fn test_commit_trailers() {
         format!("-----\n{editor0}-----\n"), @r#"
     -----
 
-    
+
     Reviewed-by: foo@bar.org
 
     JJ: Lines starting with "JJ:" (like this one) will be removed.
