@@ -34,25 +34,25 @@ fn test_concurrent_operation_divergence() {
     insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: The "@" expression resolved to more than one operation
-    Hint: Try specifying one of the operations by ID: 0162305507cc, d74dff64472e
+    Hint: Try specifying one of the operations by ID: b2cffe4f3026, d8ced2ea64a8
     [EOF]
     [exit status: 1]
     "#);
 
     // "op log --at-op" should work without merging the head operations
-    let output = work_dir.run_jj(["op", "log", "--at-op=d74dff64472e"]);
+    let output = work_dir.run_jj(["op", "log", "--at-op=d8ced2ea64a8"]);
     insta::assert_snapshot!(output, @r"
-    @  d74dff64472e test-username@host.example.com 2001-02-03 04:05:09.000 +07:00 - 2001-02-03 04:05:09.000 +07:00
-    │  describe commit 230dd059e1b059aefc0da06a2e5a7dbf22362f22
+    @  d8ced2ea64a8 test-username@host.example.com 2001-02-03 04:05:09.000 +07:00 - 2001-02-03 04:05:09.000 +07:00
+    │  describe commit e8849ae12c709f2321908879bc724fdb2ab8a781
     │  args: jj describe -m 'message 2' --at-op @-
-    ○  eac759b9ab75 test-username@host.example.com 2001-02-03 04:05:07.000 +07:00 - 2001-02-03 04:05:07.000 +07:00
+    ○  8f47435a3990 test-username@host.example.com 2001-02-03 04:05:07.000 +07:00 - 2001-02-03 04:05:07.000 +07:00
     │  add workspace 'default'
     ○  000000000000 root()
     [EOF]
     ");
 
     // We should be informed about the concurrent modification
-    let output = work_dir.run_jj(["log", "-T", "description"]);
+    let output = get_log_output(&work_dir);
     insta::assert_snapshot!(output, @r"
     @  message 1
     │ ○  message 2
@@ -73,32 +73,17 @@ fn test_concurrent_operations_auto_rebase() {
 
     work_dir.write_file("file", "contents");
     work_dir.run_jj(["describe", "-m", "initial"]).success();
-    let output = work_dir.run_jj(["op", "log"]);
-    insta::assert_snapshot!(output, @r"
-    @  c62ace5c0522 test-username@host.example.com 2001-02-03 04:05:08.000 +07:00 - 2001-02-03 04:05:08.000 +07:00
-    │  describe commit 4e8f9d2be039994f589b4e57ac5e9488703e604d
-    │  args: jj describe -m initial
-    ○  82d32fc68fc3 test-username@host.example.com 2001-02-03 04:05:08.000 +07:00 - 2001-02-03 04:05:08.000 +07:00
-    │  snapshot working copy
-    │  args: jj describe -m initial
-    ○  eac759b9ab75 test-username@host.example.com 2001-02-03 04:05:07.000 +07:00 - 2001-02-03 04:05:07.000 +07:00
-    │  add workspace 'default'
-    ○  000000000000 root()
-    [EOF]
-    ");
-    let op_id_hex = output.stdout.raw()[3..15].to_string();
-
     work_dir.run_jj(["describe", "-m", "rewritten"]).success();
     work_dir
-        .run_jj(["new", "--at-op", &op_id_hex, "-m", "new child"])
+        .run_jj(["new", "--at-op=@-", "-m", "new child"])
         .success();
 
     // We should be informed about the concurrent modification
     let output = get_log_output(&work_dir);
     insta::assert_snapshot!(output, @r"
-    ○  db141860e12c2d5591c56fde4fc99caf71cec418 new child
-    @  07c3641e495cce57ea4ca789123b52f421c57aa2 rewritten
-    ◆  0000000000000000000000000000000000000000
+    ○  new child
+    @  rewritten
+    ◆
     [EOF]
     ------- stderr -------
     Concurrent modification detected, resolving automatically.
@@ -115,25 +100,20 @@ fn test_concurrent_operations_wc_modified() {
 
     work_dir.write_file("file", "contents\n");
     work_dir.run_jj(["describe", "-m", "initial"]).success();
-    let output = work_dir.run_jj(["op", "log"]).success();
-    let op_id_hex = output.stdout.raw()[3..15].to_string();
-
+    work_dir.run_jj(["new", "-m", "new child1"]).success();
     work_dir
-        .run_jj(["new", "--at-op", &op_id_hex, "-m", "new child1"])
-        .success();
-    work_dir
-        .run_jj(["new", "--at-op", &op_id_hex, "-m", "new child2"])
+        .run_jj(["new", "--at-op=@-", "-m", "new child2"])
         .success();
     work_dir.write_file("file", "modified\n");
 
     // We should be informed about the concurrent modification
     let output = get_log_output(&work_dir);
     insta::assert_snapshot!(output, @r"
-    @  4eadcf3df11f46ef3d825c776496221cc8303053 new child1
-    │ ○  68119f1643b7e3c301c5f7c2b6c9bf4ccba87379 new child2
+    @  new child1
+    │ ○  new child2
     ├─╯
-    ○  2ff7ae858a3a11837fdf9d1a76be295ef53f1bb3 initial
-    ◆  0000000000000000000000000000000000000000
+    ○  initial
+    ◆
     [EOF]
     ------- stderr -------
     Concurrent modification detected, resolving automatically.
@@ -160,7 +140,7 @@ fn test_concurrent_operations_wc_modified() {
     ○ │  new empty commit
     │ ○  new empty commit
     ├─╯
-    ○  describe commit 506f4ec3c2c62befa15fabc34ca9d4e6d7bef254
+    ○  describe commit 9a462e35578a347e6a3951bf7a58ad7146959a8b
     ○  snapshot working copy
     ○  add workspace 'default'
     ○
@@ -187,65 +167,67 @@ fn test_concurrent_snapshot_wc_reloadable() {
     work_dir.write_file("child1", "");
     work_dir.run_jj(["commit", "-m", "new child1"]).success();
 
-    let template = r#"id ++ "\n" ++ description ++ "\n" ++ tags"#;
+    let template = r#"id.short() ++ "\n" ++ description ++ "\n" ++ tags"#;
     let output = work_dir.run_jj(["op", "log", "-T", template]);
     insta::assert_snapshot!(output, @r"
-    @  ec6bf266624bbaed55833a34ae62fa95c0e9efa651b94eb28846972da645845052dcdc8580332a5628849f23f48b9e99fc728dc3fb13106df8d0666d746f8b85
-    │  commit 554d22b2c43c1c47e279430197363e8daabe2fd6
+    @  a631dcf37fea
+    │  commit c91a0909a9d3f3d8392ba9fab88f4b40fc0810ee
     │  args: jj commit -m 'new child1'
-    ○  23858df860b789e8176a73c0eb21804e3f1848f26d68b70d234c004d08980c41499b6669042bca20fbc2543c437222a084c7cd473e91c7a9a095a02bf38544ab
+    ○  2b8e6f8683dc
     │  snapshot working copy
     │  args: jj commit -m 'new child1'
-    ○  e1db5fa988fc66e5cc0491b00c53fb93e25e730341c850cb42e1e0db0c76d2b4065005787563301b1d292c104f381918897f7deabeb92d2532f42ce75d3fe588
-    │  commit de71e09289762a65f80bb1c3dae2a949df6bcde7
+    ○  2e1c4ffb74ca
+    │  commit 9af4c151edead0304de97ce3a0b414552921a425
     │  args: jj commit -m initial
-    ○  7de878155a459b7751097222132c935f9dcbb8f69a72b0f3a9036345a963010a553dc7c92964220128679ead72b087ca3aaf4ab9e20a221d1ffa4f9e92a32193
+    ○  cfe73d1664ae
     │  snapshot working copy
     │  args: jj commit -m initial
-    ○  eac759b9ab75793fd3da96e60939fb48f2cd2b2a9c1f13ffe723cf620f3005b8d3e7e923634a07ea39513e4f2f360c87b9ad5d331cf90d7a844864b83b72eba1
+    ○  8f47435a3990
     │  add workspace 'default'
-    ○  00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000
+    ○  000000000000
 
     [EOF]
     ");
-    let op_log_lines = output.stdout.raw().lines().collect_vec();
-    let current_op_id = op_log_lines[0].split_once("  ").unwrap().1;
-    let previous_op_id = op_log_lines[6].split_once("  ").unwrap().1;
+    let template = r#"id ++ "\n""#;
+    let output = work_dir.run_jj(["op", "log", "--no-graph", "-T", template]);
+    let [op_id_after_snapshot, _, op_id_before_snapshot] =
+        output.stdout.raw().lines().next_array().unwrap();
+    insta::assert_snapshot!(op_id_after_snapshot[..12], @"a631dcf37fea");
+    insta::assert_snapshot!(op_id_before_snapshot[..12], @"2e1c4ffb74ca");
 
-    // Another process started from the "initial" operation, but snapshots after
-    // the "child1" checkout has been completed.
+    // Simulate a concurrent operation that began from the "initial" operation
+    // (before the "child1" snapshot) but finished after the "child1"
+    // snapshot and commit.
     std::fs::rename(
-        op_heads_dir.join(current_op_id),
-        op_heads_dir.join(previous_op_id),
+        op_heads_dir.join(op_id_after_snapshot),
+        op_heads_dir.join(op_id_before_snapshot),
     )
     .unwrap();
     work_dir.write_file("child2", "");
     let output = work_dir.run_jj(["describe", "-m", "new child2"]);
-    insta::assert_snapshot!(output, @r"
+    insta::assert_snapshot!(output, @r###"
     ------- stderr -------
-    Working copy  (@) now at: kkmpptxz 1795621b new child2
-    Parent commit (@-)      : rlvkpnrz 86f54245 new child1
+    Working copy  (@) now at: kkmpptxz 493da83e new child2
+    Parent commit (@-)      : rlvkpnrz 15bd889d new child1
     [EOF]
-    ");
+    "###);
 
     // Since the repo can be reloaded before snapshotting, "child2" should be
     // a child of "child1", not of "initial".
-    let template = r#"commit_id ++ " " ++ description"#;
-    let output = work_dir.run_jj(["log", "-T", template, "-s"]);
+    let output = work_dir.run_jj(["log", "-T", "description", "-s"]);
     insta::assert_snapshot!(output, @r"
-    @  1795621b54f4ebb435978b65d66bc0f90d8f20b6 new child2
+    @  new child2
     │  A child2
-    ○  86f54245e13f850f8275b5541e56da996b6a47b7 new child1
+    ○  new child1
     │  A child1
-    ○  84f07f6bca2ffeddac84a8b09f60c6b81112375c initial
+    ○  initial
     │  A base
-    ◆  0000000000000000000000000000000000000000
+    ◆
     [EOF]
     ");
 }
 
 #[must_use]
 fn get_log_output(work_dir: &TestWorkDir) -> CommandOutput {
-    let template = r#"commit_id ++ " " ++ description"#;
-    work_dir.run_jj(["log", "-T", template])
+    work_dir.run_jj(["log", "-T", "description"])
 }
