@@ -32,28 +32,36 @@ fn test_evolog_with_or_without_diff() {
 
     let output = work_dir.run_jj(["evolog"]);
     insta::assert_snapshot!(output, @r"
-    @  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 66b42ad3
+    @  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 33c10ace
     │  my description
-    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 07b18245 conflict
+    │  -- operation 3499115d3831 (2001-02-03 08:05:10) snapshot working copy
+    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 7f56b2a0 conflict
     │  my description
-    ○  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 068224a7
+    │  -- operation eb87ec366530 (2001-02-03 08:05:09) rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    ○  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 51e08f95
     │  my description
-    ○  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 2b023b5f
+    │  -- operation 18a971ce330a (2001-02-03 08:05:09) snapshot working copy
+    ○  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 b955b72e
        (empty) my description
+       -- operation e0f8e58b3800 (2001-02-03 08:05:08) new empty commit
     [EOF]
     ");
 
     // Color
     let output = work_dir.run_jj(["--color=always", "evolog"]);
     insta::assert_snapshot!(output, @r"
-    [1m[38;5;2m@[0m  [1m[38;5;13mr[38;5;8mlvkpnrz[39m [38;5;3mtest.user@example.com[39m [38;5;14m2001-02-03 08:05:10[39m [38;5;12m6[38;5;8m6b42ad3[39m[0m
+    [1m[38;5;2m@[0m  [1m[38;5;13mr[38;5;8mlvkpnrz[39m [38;5;3mtest.user@example.com[39m [38;5;14m2001-02-03 08:05:10[39m [38;5;12m3[38;5;8m3c10ace[39m[0m
     │  [1mmy description[0m
-    [1m[38;5;1m×[0m  [1m[39mr[0m[38;5;8mlvkpnrz[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:09[39m [1m[38;5;4m07[0m[38;5;8mb18245[39m [38;5;1mconflict[39m
+    │  [38;5;8m--[39m operation [38;5;4m3499115d3831[39m ([38;5;6m2001-02-03 08:05:10[39m) snapshot working copy
+    [1m[38;5;1m×[0m  [1m[39mr[0m[38;5;8mlvkpnrz[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:09[39m [1m[38;5;4m7[0m[38;5;8mf56b2a0[39m [38;5;1mconflict[39m
     │  my description
-    ○  [1m[39mr[0m[38;5;8mlvkpnrz[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:09[39m [1m[38;5;4m06[0m[38;5;8m8224a7[39m
+    │  [38;5;8m--[39m operation [38;5;4meb87ec366530[39m ([38;5;6m2001-02-03 08:05:09[39m) rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    ○  [1m[39mr[0m[38;5;8mlvkpnrz[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:09[39m [1m[38;5;4m5[0m[38;5;8m1e08f95[39m
     │  my description
-    ○  [1m[39mr[0m[38;5;8mlvkpnrz[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:08[39m [1m[38;5;4m2b[0m[38;5;8m023b5f[39m
+    │  [38;5;8m--[39m operation [38;5;4m18a971ce330a[39m ([38;5;6m2001-02-03 08:05:09[39m) snapshot working copy
+    ○  [1m[39mr[0m[38;5;8mlvkpnrz[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:08[39m [1m[38;5;4mb[0m[38;5;8m955b72e[39m
        [38;5;2m(empty)[39m my description
+       [38;5;8m--[39m operation [38;5;4me0f8e58b3800[39m ([38;5;6m2001-02-03 08:05:08[39m) new empty commit
     [EOF]
     ");
 
@@ -61,8 +69,9 @@ fn test_evolog_with_or_without_diff() {
     // (even even though it resulted in a conflict).
     let output = work_dir.run_jj(["evolog", "-p"]);
     insta::assert_snapshot!(output, @r"
-    @  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 66b42ad3
+    @  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 33c10ace
     │  my description
+    │  -- operation 3499115d3831 (2001-02-03 08:05:10) snapshot working copy
     │  Resolved conflict in file1:
     │     1     : <<<<<<< Conflict 1 of 1
     │     2     : %%%%%%% Changes from base to side #1
@@ -71,49 +80,59 @@ fn test_evolog_with_or_without_diff() {
     │     5     : foo
     │     6     : bar
     │     7    1: >>>>>>> Conflict 1 of 1 endsresolved
-    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 07b18245 conflict
+    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 7f56b2a0 conflict
     │  my description
-    ○  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 068224a7
+    │  -- operation eb87ec366530 (2001-02-03 08:05:09) rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    ○  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 51e08f95
     │  my description
+    │  -- operation 18a971ce330a (2001-02-03 08:05:09) snapshot working copy
     │  Modified regular file file1:
     │     1    1: foo
     │          2: bar
     │  Added regular file file2:
     │          1: foo
-    ○  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 2b023b5f
+    ○  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 b955b72e
        (empty) my description
+       -- operation e0f8e58b3800 (2001-02-03 08:05:08) new empty commit
     [EOF]
     ");
 
     // Test `--limit`
     let output = work_dir.run_jj(["evolog", "--limit=2"]);
     insta::assert_snapshot!(output, @r"
-    @  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 66b42ad3
+    @  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 33c10ace
     │  my description
-    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 07b18245 conflict
+    │  -- operation 3499115d3831 (2001-02-03 08:05:10) snapshot working copy
+    ×  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 7f56b2a0 conflict
     │  my description
+    │  -- operation eb87ec366530 (2001-02-03 08:05:09) rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
     [EOF]
     ");
 
     // Test `--no-graph`
     let output = work_dir.run_jj(["evolog", "--no-graph"]);
     insta::assert_snapshot!(output, @r"
-    rlvkpnrz test.user@example.com 2001-02-03 08:05:10 66b42ad3
+    rlvkpnrz test.user@example.com 2001-02-03 08:05:10 33c10ace
     my description
-    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 07b18245 conflict
+    -- operation 3499115d3831 (2001-02-03 08:05:10) snapshot working copy
+    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 7f56b2a0 conflict
     my description
-    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 068224a7
+    -- operation eb87ec366530 (2001-02-03 08:05:09) rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 51e08f95
     my description
-    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 2b023b5f
+    -- operation 18a971ce330a (2001-02-03 08:05:09) snapshot working copy
+    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 b955b72e
     (empty) my description
+    -- operation e0f8e58b3800 (2001-02-03 08:05:08) new empty commit
     [EOF]
     ");
 
     // Test `--git` format, and that it implies `-p`
     let output = work_dir.run_jj(["evolog", "--no-graph", "--git"]);
     insta::assert_snapshot!(output, @r"
-    rlvkpnrz test.user@example.com 2001-02-03 08:05:10 66b42ad3
+    rlvkpnrz test.user@example.com 2001-02-03 08:05:10 33c10ace
     my description
+    -- operation 3499115d3831 (2001-02-03 08:05:10) snapshot working copy
     diff --git a/file1 b/file1
     index 0000000000..2ab19ae607 100644
     --- a/file1
@@ -127,10 +146,12 @@ fn test_evolog_with_or_without_diff() {
     -bar
     ->>>>>>> Conflict 1 of 1 ends
     +resolved
-    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 07b18245 conflict
+    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 7f56b2a0 conflict
     my description
-    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 068224a7
+    -- operation eb87ec366530 (2001-02-03 08:05:09) rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 51e08f95
     my description
+    -- operation 18a971ce330a (2001-02-03 08:05:09) snapshot working copy
     diff --git a/file1 b/file1
     index 257cc5642c..3bd1f0e297 100644
     --- a/file1
@@ -145,8 +166,9 @@ fn test_evolog_with_or_without_diff() {
     +++ b/file2
     @@ -0,0 +1,1 @@
     +foo
-    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 2b023b5f
+    rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 b955b72e
     (empty) my description
+    -- operation e0f8e58b3800 (2001-02-03 08:05:08) new empty commit
     [EOF]
     ");
 }
@@ -170,14 +192,18 @@ fn test_evolog_with_custom_symbols() {
     let output = work_dir.run_jj(["evolog", "--config", config]);
 
     insta::assert_snapshot!(output, @r"
-    $  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 66b42ad3
+    $  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 33c10ace
     │  my description
-    ┝  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 07b18245 conflict
+    │  -- operation 3622beb20303 (2001-02-03 08:05:10) snapshot working copy
+    ┝  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 7f56b2a0 conflict
     │  my description
-    ┝  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 068224a7
+    │  -- operation eb87ec366530 (2001-02-03 08:05:09) rebase commit 51e08f95160c897080d035d330aead3ee6ed5588
+    ┝  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:09 51e08f95
     │  my description
-    ┝  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 2b023b5f
+    │  -- operation 18a971ce330a (2001-02-03 08:05:09) snapshot working copy
+    ┝  rlvkpnrz hidden test.user@example.com 2001-02-03 08:05:08 b955b72e
        (empty) my description
+       -- operation e0f8e58b3800 (2001-02-03 08:05:08) new empty commit
     [EOF]
     ");
 }
@@ -200,35 +226,49 @@ fn test_evolog_word_wrap() {
 
     // ui.log-word-wrap option applies to both graph/no-graph outputs
     insta::assert_snapshot!(render(&["evolog"], 40, false), @r"
-    @  qpvuntsm test.user@example.com 2001-02-03 08:05:08 fa15625b
+    @  qpvuntsm test.user@example.com 2001-02-03 08:05:08 68a50538
     │  (empty) first
-    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 230dd059
+    │  -- operation 75545f7ff2df (2001-02-03 08:05:08) describe commit e8849ae12c709f2321908879bc724fdb2ab8a781
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 e8849ae1
        (empty) (no description set)
+       -- operation 8f47435a3990 (2001-02-03 08:05:07) add workspace 'default'
     [EOF]
     ");
     insta::assert_snapshot!(render(&["evolog"], 40, true), @r"
     @  qpvuntsm test.user@example.com
-    │  2001-02-03 08:05:08 fa15625b
+    │  2001-02-03 08:05:08 68a50538
     │  (empty) first
+    │  -- operation 75545f7ff2df (2001-02-03
+    │  08:05:08) describe commit
+    │  e8849ae12c709f2321908879bc724fdb2ab8a781
     ○  qpvuntsm hidden test.user@example.com
-       2001-02-03 08:05:07 230dd059
+       2001-02-03 08:05:07 e8849ae1
        (empty) (no description set)
+       -- operation 8f47435a3990 (2001-02-03
+       08:05:07) add workspace 'default'
     [EOF]
     ");
     insta::assert_snapshot!(render(&["evolog", "--no-graph"], 40, false), @r"
-    qpvuntsm test.user@example.com 2001-02-03 08:05:08 fa15625b
+    qpvuntsm test.user@example.com 2001-02-03 08:05:08 68a50538
     (empty) first
-    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 230dd059
+    -- operation 75545f7ff2df (2001-02-03 08:05:08) describe commit e8849ae12c709f2321908879bc724fdb2ab8a781
+    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 e8849ae1
     (empty) (no description set)
+    -- operation 8f47435a3990 (2001-02-03 08:05:07) add workspace 'default'
     [EOF]
     ");
     insta::assert_snapshot!(render(&["evolog", "--no-graph"], 40, true), @r"
     qpvuntsm test.user@example.com
-    2001-02-03 08:05:08 fa15625b
+    2001-02-03 08:05:08 68a50538
     (empty) first
+    -- operation 75545f7ff2df (2001-02-03
+    08:05:08) describe commit
+    e8849ae12c709f2321908879bc724fdb2ab8a781
     qpvuntsm hidden test.user@example.com
-    2001-02-03 08:05:07 230dd059
+    2001-02-03 08:05:07 e8849ae1
     (empty) (no description set)
+    -- operation 8f47435a3990 (2001-02-03
+    08:05:07) add workspace 'default'
     [EOF]
     ");
 }
@@ -275,28 +315,35 @@ fn test_evolog_squash() {
 
     let output = work_dir.run_jj(["evolog", "-p", "-r", "description('squash')"]);
     insta::assert_snapshot!(output, @r"
-    ○      qpvuntsm test.user@example.com 2001-02-03 08:05:15 d49749bf
+    ○      qpvuntsm test.user@example.com 2001-02-03 08:05:15 5f3281c6
     ├─┬─╮  squashed 3
-    │ │ ○  vruxwmqv hidden test.user@example.com 2001-02-03 08:05:15 8f2ae2b5
+    │ │ │  -- operation 838e6d867fda (2001-02-03 08:05:15) squash commits into 5ec0619af5cb4f7707a556a71a6f96af0bc294d2
+    │ │ ○  vruxwmqv hidden test.user@example.com 2001-02-03 08:05:15 770795d0
     │ │ │  fifth
+    │ │ │  -- operation 1d38c000b52d (2001-02-03 08:05:15) snapshot working copy
     │ │ │  Added regular file file5:
     │ │ │          1: foo5
-    │ │ ○  vruxwmqv hidden test.user@example.com 2001-02-03 08:05:14 04d28ca9
+    │ │ ○  vruxwmqv hidden test.user@example.com 2001-02-03 08:05:14 2e0123d1
     │ │    (empty) fifth
-    │ ○  yqosqzyt hidden test.user@example.com 2001-02-03 08:05:14 c5801e10
+    │ │    -- operation fc852ed87801 (2001-02-03 08:05:14) new empty commit
+    │ ○  yqosqzyt hidden test.user@example.com 2001-02-03 08:05:14 ea8161b6
     │ │  fourth
+    │ │  -- operation 3b09d55dfa6e (2001-02-03 08:05:14) snapshot working copy
     │ │  Added regular file file4:
     │ │          1: foo4
-    │ ○  yqosqzyt hidden test.user@example.com 2001-02-03 08:05:13 bb54a199
+    │ ○  yqosqzyt hidden test.user@example.com 2001-02-03 08:05:13 1de5fdb6
     │    (empty) fourth
-    ○    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:12 1408a0a7
+    │    -- operation 9404a551035a (2001-02-03 08:05:13) new empty commit
+    ○    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:12 5ec0619a
     ├─╮  squashed 2
+    │ │  -- operation fa9796d12627 (2001-02-03 08:05:12) squash commits into 690858846504af0e42fde980fdacf9851559ebb8
     │ │  Removed regular file file2:
     │ │     1     : foo2
     │ │  Removed regular file file3:
     │ │     1     : foo3
-    │ ○  zsuskuln hidden test.user@example.com 2001-02-03 08:05:12 c9460789
+    │ ○  zsuskuln hidden test.user@example.com 2001-02-03 08:05:12 cce957f1
     │ │  third
+    │ │  -- operation de96267cd621 (2001-02-03 08:05:12) snapshot working copy
     │ │  Modified regular file file1:
     │ │     1    1: foo
     │ │     2    2: bar
@@ -305,27 +352,35 @@ fn test_evolog_squash() {
     │ │          1: foo2
     │ │  Added regular file file3:
     │ │          1: foo3
-    │ ○  zsuskuln hidden test.user@example.com 2001-02-03 08:05:11 66645763
+    │ ○  zsuskuln hidden test.user@example.com 2001-02-03 08:05:11 3a2a4253
     │ │  (empty) third
-    │ ○  zsuskuln hidden test.user@example.com 2001-02-03 08:05:10 1c7afcb4
+    │ │  -- operation 4611a6121e8a (2001-02-03 08:05:11) describe commit ebec10f449ad7ab92c7293efab5e3db2d8e9fea1
+    │ ○  zsuskuln hidden test.user@example.com 2001-02-03 08:05:10 ebec10f4
     │    (empty) (no description set)
-    ○    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:10 e3c2a446
+    │    -- operation 65c81703100d (2001-02-03 08:05:10) squash commits into 5878cbe03cdf599c9353e5a1a52a01f4c5e0e0fa
+    ○    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:10 69085884
     ├─╮  squashed 1
-    │ ○  kkmpptxz hidden test.user@example.com 2001-02-03 08:05:10 46acd22a
+    │ │  -- operation 65c81703100d (2001-02-03 08:05:10) squash commits into 5878cbe03cdf599c9353e5a1a52a01f4c5e0e0fa
+    │ ○  kkmpptxz hidden test.user@example.com 2001-02-03 08:05:10 a3759c9d
     │ │  second
+    │ │  -- operation a7b202f56742 (2001-02-03 08:05:10) snapshot working copy
     │ │  Modified regular file file1:
     │ │     1    1: foo
     │ │          2: bar
-    │ ○  kkmpptxz hidden test.user@example.com 2001-02-03 08:05:09 cba41deb
+    │ ○  kkmpptxz hidden test.user@example.com 2001-02-03 08:05:09 a5b2f625
     │    (empty) second
-    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:09 766420db
+    │    -- operation 26f649a0cdfa (2001-02-03 08:05:09) new empty commit
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:09 5878cbe0
     │  first
+    │  -- operation af15122a5868 (2001-02-03 08:05:09) snapshot working copy
     │  Added regular file file1:
     │          1: foo
-    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:08 fa15625b
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:08 68a50538
     │  (empty) first
-    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 230dd059
+    │  -- operation 75545f7ff2df (2001-02-03 08:05:08) describe commit e8849ae12c709f2321908879bc724fdb2ab8a781
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 e8849ae1
        (empty) (no description set)
+       -- operation 8f47435a3990 (2001-02-03 08:05:07) add workspace 'default'
     [EOF]
     ");
 }
@@ -380,23 +435,29 @@ fn test_evolog_reversed_no_graph() {
     work_dir.run_jj(["describe", "-m", "c"]).success();
     let output = work_dir.run_jj(["evolog", "--reversed", "--no-graph"]);
     insta::assert_snapshot!(output, @r"
-    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 230dd059
+    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 e8849ae1
     (empty) (no description set)
-    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:08 d8d5f980
+    -- operation 8f47435a3990 (2001-02-03 08:05:07) add workspace 'default'
+    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:08 b86e28cd
     (empty) a
-    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:09 b4584f54
+    -- operation ab34d1de4875 (2001-02-03 08:05:08) describe commit e8849ae12c709f2321908879bc724fdb2ab8a781
+    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:09 9f43967b
     (empty) b
-    qpvuntsm test.user@example.com 2001-02-03 08:05:10 5cb22a87
+    -- operation 3851e9877d51 (2001-02-03 08:05:09) describe commit b86e28cd6862624ad77e1aaf31e34b2c7545bebd
+    qpvuntsm test.user@example.com 2001-02-03 08:05:10 b28cda4b
     (empty) c
+    -- operation 5f4c7b5cb177 (2001-02-03 08:05:10) describe commit 9f43967b1cdbce4ab322cb7b4636fc0362c38373
     [EOF]
     ");
 
     let output = work_dir.run_jj(["evolog", "--limit=2", "--reversed", "--no-graph"]);
     insta::assert_snapshot!(output, @r"
-    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:09 b4584f54
+    qpvuntsm hidden test.user@example.com 2001-02-03 08:05:09 9f43967b
     (empty) b
-    qpvuntsm test.user@example.com 2001-02-03 08:05:10 5cb22a87
+    -- operation 3851e9877d51 (2001-02-03 08:05:09) describe commit b86e28cd6862624ad77e1aaf31e34b2c7545bebd
+    qpvuntsm test.user@example.com 2001-02-03 08:05:10 b28cda4b
     (empty) c
+    -- operation 5f4c7b5cb177 (2001-02-03 08:05:10) describe commit 9f43967b1cdbce4ab322cb7b4636fc0362c38373
     [EOF]
     ");
 }
@@ -429,31 +490,41 @@ fn test_evolog_reverse_with_graph() {
         .success();
     let output = work_dir.run_jj(["evolog", "-r", "description(c+d+e)", "--reversed"]);
     insta::assert_snapshot!(output, @r"
-    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 230dd059
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 e8849ae1
     │  (empty) (no description set)
-    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:08 d8d5f980
+    │  -- operation 8f47435a3990 (2001-02-03 08:05:07) add workspace 'default'
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:08 b86e28cd
     │  (empty) a
-    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:09 b4584f54
+    │  -- operation ab34d1de4875 (2001-02-03 08:05:08) describe commit e8849ae12c709f2321908879bc724fdb2ab8a781
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:09 9f43967b
     │  (empty) b
-    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:10 5cb22a87
+    │  -- operation 3851e9877d51 (2001-02-03 08:05:09) describe commit b86e28cd6862624ad77e1aaf31e34b2c7545bebd
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:10 b28cda4b
     │  (empty) c
-    │ ○  mzvwutvl hidden test.user@example.com 2001-02-03 08:05:11 280cbb6e
+    │  -- operation 5f4c7b5cb177 (2001-02-03 08:05:10) describe commit 9f43967b1cdbce4ab322cb7b4636fc0362c38373
+    │ ○  mzvwutvl hidden test.user@example.com 2001-02-03 08:05:11 6a4ff8aa
     ├─╯  (empty) d
-    │ ○  royxmykx hidden test.user@example.com 2001-02-03 08:05:12 031df638
+    │    -- operation 774accf68695 (2001-02-03 08:05:11) new empty commit
+    │ ○  royxmykx hidden test.user@example.com 2001-02-03 08:05:12 7dea2d1d
     ├─╯  (empty) e
-    ○  qpvuntsm test.user@example.com 2001-02-03 08:05:13 a177c2f2
+    │    -- operation 4c2c3012e2c3 (2001-02-03 08:05:12) new empty commit
+    ○  qpvuntsm test.user@example.com 2001-02-03 08:05:13 78fdd026
        (empty) c+d+e
+       -- operation 2c736b66cd16 (2001-02-03 08:05:13) squash commits into b28cda4b118fc50495ca34a24f030abc078d032e
     [EOF]
     ");
 
     let output = work_dir.run_jj(["evolog", "-rdescription(c+d+e)", "--limit=3", "--reversed"]);
     insta::assert_snapshot!(output, @r"
-    ○  mzvwutvl hidden test.user@example.com 2001-02-03 08:05:11 280cbb6e
+    ○  mzvwutvl hidden test.user@example.com 2001-02-03 08:05:11 6a4ff8aa
     │  (empty) d
-    │ ○  royxmykx hidden test.user@example.com 2001-02-03 08:05:12 031df638
+    │  -- operation 774accf68695 (2001-02-03 08:05:11) new empty commit
+    │ ○  royxmykx hidden test.user@example.com 2001-02-03 08:05:12 7dea2d1d
     ├─╯  (empty) e
-    ○  qpvuntsm test.user@example.com 2001-02-03 08:05:13 a177c2f2
+    │    -- operation 4c2c3012e2c3 (2001-02-03 08:05:12) new empty commit
+    ○  qpvuntsm test.user@example.com 2001-02-03 08:05:13 78fdd026
        (empty) c+d+e
+       -- operation 2c736b66cd16 (2001-02-03 08:05:13) squash commits into b28cda4b118fc50495ca34a24f030abc078d032e
     [EOF]
     ");
 }
