@@ -72,9 +72,9 @@ fn test_resolution() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Resolving conflicts in: file
-    Working copy  (@) now at: vruxwmqv e069f073 conflict | conflict
-    Parent commit (@-)      : zsuskuln aa493daf a | a
-    Parent commit (@-)      : royxmykx db6a4daf b | b
+    Working copy  (@) now at: vruxwmqv 741263c9 conflict | conflict
+    Parent commit (@-)      : zsuskuln 45537d53 a | a
+    Parent commit (@-)      : royxmykx 89d1b299 b | b
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
@@ -114,9 +114,9 @@ fn test_resolution() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Resolving conflicts in: file
-    Working copy  (@) now at: vruxwmqv 1a70c7c6 conflict | conflict
-    Parent commit (@-)      : zsuskuln aa493daf a | a
-    Parent commit (@-)      : royxmykx db6a4daf b | b
+    Working copy  (@) now at: vruxwmqv 1f8a36f7 conflict | conflict
+    Parent commit (@-)      : zsuskuln 45537d53 a | a
+    Parent commit (@-)      : royxmykx 89d1b299 b | b
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
@@ -211,24 +211,25 @@ fn test_resolution() {
         "resolve",
         "--config=merge-tools.fake-editor.merge-tool-edits-conflict-markers=true",
     ]);
-    insta::assert_snapshot!(output, @r"
+    insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Resolving conflicts in: file
-    Working copy  (@) now at: vruxwmqv 608a2310 conflict | (conflict) conflict
-    Parent commit (@-)      : zsuskuln aa493daf a | a
-    Parent commit (@-)      : royxmykx db6a4daf b | b
+    Working copy  (@) now at: vruxwmqv 0d40d2b8 conflict | (conflict) conflict
+    Parent commit (@-)      : zsuskuln 45537d53 a | a
+    Parent commit (@-)      : royxmykx 89d1b299 b | b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict
     New conflicts appeared in 1 commits:
-      vruxwmqv 608a2310 conflict | (conflict) conflict
-    Hint: To resolve the conflicts, start by updating to it:
+      vruxwmqv 0d40d2b8 conflict | (conflict) conflict
+    Hint: To resolve the conflicts, start by creating a commit on top of
+    the conflicted commit:
       jj new vruxwmqv
     Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want to inspect the result with `jj diff`.
+    Once the conflicts are resolved, you can inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
     [EOF]
-    ");
+    "###);
     insta::assert_snapshot!(
         std::fs::read_to_string(test_env.env_root().join("editor2")).unwrap(), @r"
     <<<<<<< Conflict 1 of 1
@@ -289,9 +290,9 @@ fn test_resolution() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Resolving conflicts in: file
-    Working copy  (@) now at: vruxwmqv 3166dfd2 conflict | conflict
-    Parent commit (@-)      : zsuskuln aa493daf a | a
-    Parent commit (@-)      : royxmykx db6a4daf b | b
+    Working copy  (@) now at: vruxwmqv 2cc7f5e3 conflict | conflict
+    Parent commit (@-)      : zsuskuln 45537d53 a | a
+    Parent commit (@-)      : royxmykx 89d1b299 b | b
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
@@ -354,24 +355,25 @@ fn test_resolution() {
         "--config=merge-tools.fake-editor.merge-tool-edits-conflict-markers=true",
         "--config=merge-tools.fake-editor.conflict-marker-style=git",
     ]);
-    insta::assert_snapshot!(output, @r"
+    insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Resolving conflicts in: file
-    Working copy  (@) now at: vruxwmqv 8e03fefa conflict | (conflict) conflict
-    Parent commit (@-)      : zsuskuln aa493daf a | a
-    Parent commit (@-)      : royxmykx db6a4daf b | b
+    Working copy  (@) now at: vruxwmqv d5f058ec conflict | (conflict) conflict
+    Parent commit (@-)      : zsuskuln 45537d53 a | a
+    Parent commit (@-)      : royxmykx 89d1b299 b | b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict
     New conflicts appeared in 1 commits:
-      vruxwmqv 8e03fefa conflict | (conflict) conflict
-    Hint: To resolve the conflicts, start by updating to it:
+      vruxwmqv d5f058ec conflict | (conflict) conflict
+    Hint: To resolve the conflicts, start by creating a commit on top of
+    the conflicted commit:
       jj new vruxwmqv
     Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want to inspect the result with `jj diff`.
+    Once the conflicts are resolved, you can inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
     [EOF]
-    ");
+    "###);
     insta::assert_snapshot!(
         std::fs::read_to_string(test_env.env_root().join("editor4")).unwrap(), @r"
     <<<<<<< Side #1 (Conflict 1 of 1)
@@ -432,24 +434,25 @@ fn test_resolution() {
         "resolve",
         "--config=merge-tools.fake-editor.merge-conflict-exit-codes=[1]",
     ]);
-    insta::assert_snapshot!(output, @r"
+    insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Resolving conflicts in: file
-    Working copy  (@) now at: vruxwmqv a786ac2f conflict | (conflict) conflict
-    Parent commit (@-)      : zsuskuln aa493daf a | a
-    Parent commit (@-)      : royxmykx db6a4daf b | b
+    Working copy  (@) now at: vruxwmqv 6c205356 conflict | (conflict) conflict
+    Parent commit (@-)      : zsuskuln 45537d53 a | a
+    Parent commit (@-)      : royxmykx 89d1b299 b | b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict
     New conflicts appeared in 1 commits:
-      vruxwmqv a786ac2f conflict | (conflict) conflict
-    Hint: To resolve the conflicts, start by updating to it:
+      vruxwmqv 6c205356 conflict | (conflict) conflict
+    Hint: To resolve the conflicts, start by creating a commit on top of
+    the conflicted commit:
       jj new vruxwmqv
     Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want to inspect the result with `jj diff`.
+    Once the conflicts are resolved, you can inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
     [EOF]
-    ");
+    "###);
     insta::assert_snapshot!(
         std::fs::read_to_string(test_env.env_root().join("editor5")).unwrap(), @"");
     insta::assert_snapshot!(work_dir.run_jj(["diff", "--git"]), @r"
@@ -645,6 +648,7 @@ fn test_too_many_parents() {
     Hint: Using default editor ':builtin'; run `jj config set --user ui.merge-editor :builtin` to disable this message.
     Error: Failed to resolve conflicts
     Caused by: The conflict at "file" has 3 sides. At most 2 sides are supported.
+    Hint: Edit the conflict markers manually to resolve this.
     [EOF]
     [exit status: 1]
     "#);
@@ -676,8 +680,8 @@ fn test_simplify_conflict_sides() {
     // Even though the tree-level conflict is a 4-sided conflict, each file is
     // materialized as a 2-sided conflict.
     insta::assert_snapshot!(work_dir.run_jj(["debug", "tree"]), @r#"
-    fileA: Ok(Conflicted([Some(File { id: FileId("d00491fd7e5bb6fa28c517a0bb32b8b506539d4d"), executable: false }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false }), Some(File { id: FileId("0cfbf08886fca9a91cb753ec8734c84fcbe52c9f"), executable: false }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false })]))
-    fileB: Ok(Conflicted([Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false }), Some(File { id: FileId("d00491fd7e5bb6fa28c517a0bb32b8b506539d4d"), executable: false }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false }), Some(File { id: FileId("0cfbf08886fca9a91cb753ec8734c84fcbe52c9f"), executable: false })]))
+    fileA: Ok(Conflicted([Some(File { id: FileId("d00491fd7e5bb6fa28c517a0bb32b8b506539d4d"), executable: false, copy_id: CopyId("") }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false, copy_id: CopyId("") }), Some(File { id: FileId("0cfbf08886fca9a91cb753ec8734c84fcbe52c9f"), executable: false, copy_id: CopyId("") }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false, copy_id: CopyId("") }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false, copy_id: CopyId("") }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false, copy_id: CopyId("") }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false, copy_id: CopyId("") })]))
+    fileB: Ok(Conflicted([Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false, copy_id: CopyId("") }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false, copy_id: CopyId("") }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false, copy_id: CopyId("") }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false, copy_id: CopyId("") }), Some(File { id: FileId("d00491fd7e5bb6fa28c517a0bb32b8b506539d4d"), executable: false, copy_id: CopyId("") }), Some(File { id: FileId("df967b96a579e45a18b8251732d16804b2e56a55"), executable: false, copy_id: CopyId("") }), Some(File { id: FileId("0cfbf08886fca9a91cb753ec8734c84fcbe52c9f"), executable: false, copy_id: CopyId("") })]))
     [EOF]
     "#);
     insta::assert_snapshot!(work_dir.run_jj(["resolve", "--list"]), @r"
@@ -735,25 +739,26 @@ fn test_simplify_conflict_sides() {
         "--config=merge-tools.fake-editor.merge-tool-edits-conflict-markers=true",
         "fileB",
     ]);
-    insta::assert_snapshot!(output, @r"
+    insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Resolving conflicts in: fileB
-    Working copy  (@) now at: nkmrtpmo 69cc0c2d conflict | (conflict) conflict
-    Parent commit (@-)      : kmkuslsw 4601566f conflictA | (conflict) (empty) conflictA
-    Parent commit (@-)      : lylxulpl 6f8d8381 conflictB | (conflict) (empty) conflictB
+    Working copy  (@) now at: nkmrtpmo 25c5dd0b conflict | (conflict) conflict
+    Parent commit (@-)      : kmkuslsw ccb05364 conflictA | (conflict) (empty) conflictA
+    Parent commit (@-)      : lylxulpl d9bc60cb conflictB | (conflict) (empty) conflictB
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     fileA    2-sided conflict
     fileB    2-sided conflict
     New conflicts appeared in 1 commits:
-      nkmrtpmo 69cc0c2d conflict | (conflict) conflict
-    Hint: To resolve the conflicts, start by updating to it:
+      nkmrtpmo 25c5dd0b conflict | (conflict) conflict
+    Hint: To resolve the conflicts, start by creating a commit on top of
+    the conflicted commit:
       jj new nkmrtpmo
     Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want to inspect the result with `jj diff`.
+    Once the conflicts are resolved, you can inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
     [EOF]
-    ");
+    "###);
     insta::assert_snapshot!(work_dir.read_file("fileB"), @r"
     <<<<<<< Conflict 1 of 1
     %%%%%%% Changes from base to side #1
@@ -967,24 +972,25 @@ fn test_resolve_conflicts_with_executable() {
     // Test resolving the conflict in "file1", which should produce an executable
     std::fs::write(&editor_script, b"write\nresolution1\n").unwrap();
     let output = work_dir.run_jj(["resolve", "file1"]);
-    insta::assert_snapshot!(output, @r"
+    insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Resolving conflicts in: file1
-    Working copy  (@) now at: znkkpsqq eb159d56 conflict | (conflict) conflict
-    Parent commit (@-)      : mzvwutvl 08932848 a | a
-    Parent commit (@-)      : yqosqzyt b69b3de6 b | b
+    Working copy  (@) now at: znkkpsqq 8ab9c54e conflict | (conflict) conflict
+    Parent commit (@-)      : mzvwutvl 86f7f0e3 a | a
+    Parent commit (@-)      : yqosqzyt 36361412 b | b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file2    2-sided conflict including an executable
     New conflicts appeared in 1 commits:
-      znkkpsqq eb159d56 conflict | (conflict) conflict
-    Hint: To resolve the conflicts, start by updating to it:
+      znkkpsqq 8ab9c54e conflict | (conflict) conflict
+    Hint: To resolve the conflicts, start by creating a commit on top of
+    the conflicted commit:
       jj new znkkpsqq
     Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want to inspect the result with `jj diff`.
+    Once the conflicts are resolved, you can inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
     [EOF]
-    ");
+    "###);
     insta::assert_snapshot!(work_dir.run_jj(["diff", "--git"]), @r"
     diff --git a/file1 b/file1
     index 0000000000..95cc18629d 100755
@@ -1010,24 +1016,25 @@ fn test_resolve_conflicts_with_executable() {
     work_dir.run_jj(["undo"]).success();
     std::fs::write(&editor_script, b"write\nresolution2\n").unwrap();
     let output = work_dir.run_jj(["resolve", "file2"]);
-    insta::assert_snapshot!(output, @r"
+    insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Resolving conflicts in: file2
-    Working copy  (@) now at: znkkpsqq 4dccbb3c conflict | (conflict) conflict
-    Parent commit (@-)      : mzvwutvl 08932848 a | a
-    Parent commit (@-)      : yqosqzyt b69b3de6 b | b
+    Working copy  (@) now at: znkkpsqq d47830a6 conflict | (conflict) conflict
+    Parent commit (@-)      : mzvwutvl 86f7f0e3 a | a
+    Parent commit (@-)      : yqosqzyt 36361412 b | b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file1    2-sided conflict including an executable
     New conflicts appeared in 1 commits:
-      znkkpsqq 4dccbb3c conflict | (conflict) conflict
-    Hint: To resolve the conflicts, start by updating to it:
+      znkkpsqq d47830a6 conflict | (conflict) conflict
+    Hint: To resolve the conflicts, start by creating a commit on top of
+    the conflicted commit:
       jj new znkkpsqq
     Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want to inspect the result with `jj diff`.
+    Once the conflicts are resolved, you can inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
     [EOF]
-    ");
+    "###);
     insta::assert_snapshot!(work_dir.run_jj(["diff", "--git"]), @r"
     diff --git a/file2 b/file2
     index 0000000000..775f078581 100755
@@ -1054,9 +1061,9 @@ fn test_resolve_conflicts_with_executable() {
     let output = work_dir.run_jj(["resolve", "--tool=:ours"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy  (@) now at: znkkpsqq 5b59d7f0 conflict | conflict
-    Parent commit (@-)      : mzvwutvl 08932848 a | a
-    Parent commit (@-)      : yqosqzyt b69b3de6 b | b
+    Working copy  (@) now at: znkkpsqq d902c14b conflict | conflict
+    Parent commit (@-)      : mzvwutvl 86f7f0e3 a | a
+    Parent commit (@-)      : yqosqzyt 36361412 b | b
     Added 0 files, modified 2 files, removed 0 files
     [EOF]
     ");
@@ -1095,9 +1102,9 @@ fn test_resolve_conflicts_with_executable() {
     let output = work_dir.run_jj(["resolve", "--tool=:theirs"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy  (@) now at: znkkpsqq 087b8637 conflict | conflict
-    Parent commit (@-)      : mzvwutvl 08932848 a | a
-    Parent commit (@-)      : yqosqzyt b69b3de6 b | b
+    Working copy  (@) now at: znkkpsqq a340ca5f conflict | conflict
+    Parent commit (@-)      : mzvwutvl 86f7f0e3 a | a
+    Parent commit (@-)      : yqosqzyt 36361412 b | b
     Added 0 files, modified 2 files, removed 0 files
     [EOF]
     ");
@@ -1195,9 +1202,9 @@ fn test_resolve_change_delete_executable() {
     [EOF]
     ");
     insta::assert_snapshot!(work_dir.run_jj(["log", "--git"]), @r"
-    @    kmkuslsw test.user@example.com 2001-02-03 08:05:18 conflict 5db5dbdf conflict
+    @    kmkuslsw test.user@example.com 2001-02-03 08:05:18 conflict 7a7ac759 conflict
     ├─╮  (empty) conflict
-    │ ○  vruxwmqv test.user@example.com 2001-02-03 08:05:17 b 19e7d2ff
+    │ ○  vruxwmqv test.user@example.com 2001-02-03 08:05:17 b 888b6cc3
     │ │  b
     │ │  diff --git a/file1 b/file1
     │ │  deleted file mode 100644
@@ -1222,7 +1229,7 @@ fn test_resolve_change_delete_executable() {
     │ │  +++ b/file5
     │ │  @@ -0,0 +1,1 @@
     │ │  +b5
-    ○ │  mzvwutvl test.user@example.com 2001-02-03 08:05:13 a 4a44e1a9
+    ○ │  mzvwutvl test.user@example.com 2001-02-03 08:05:13 a e2d3924b
     ├─╯  a
     │    diff --git a/file1 b/file1
     │    old mode 100644
@@ -1242,7 +1249,7 @@ fn test_resolve_change_delete_executable() {
     │    diff --git a/file5 b/file5
     │    deleted file mode 100755
     │    index e69de29bb2..0000000000
-    ○  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 base dd88cb4a
+    ○  rlvkpnrz test.user@example.com 2001-02-03 08:05:10 base f747aa1f
     │  base
     │  diff --git a/file1 b/file1
     │  new file mode 100644
@@ -1296,9 +1303,9 @@ fn test_resolve_change_delete_executable() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Resolving conflicts in: file2
-    Working copy  (@) now at: kmkuslsw cab8be9b conflict | (conflict) conflict
-    Parent commit (@-)      : mzvwutvl 4a44e1a9 a | a
-    Parent commit (@-)      : vruxwmqv 19e7d2ff b | b
+    Working copy  (@) now at: kmkuslsw 1323520e conflict | (conflict) conflict
+    Parent commit (@-)      : mzvwutvl e2d3924b a | a
+    Parent commit (@-)      : vruxwmqv 888b6cc3 b | b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file3    2-sided conflict including an executable
@@ -1327,9 +1334,9 @@ fn test_resolve_change_delete_executable() {
     let output = work_dir.run_jj(["resolve", "file4", "--tool=:ours"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy  (@) now at: kmkuslsw 14b5c3d2 conflict | (conflict) conflict
-    Parent commit (@-)      : mzvwutvl 4a44e1a9 a | a
-    Parent commit (@-)      : vruxwmqv 19e7d2ff b | b
+    Working copy  (@) now at: kmkuslsw 630e8689 conflict | (conflict) conflict
+    Parent commit (@-)      : mzvwutvl e2d3924b a | a
+    Parent commit (@-)      : vruxwmqv 888b6cc3 b | b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file5    2-sided conflict including 1 deletion and an executable
@@ -1340,9 +1347,9 @@ fn test_resolve_change_delete_executable() {
     let output = work_dir.run_jj(["resolve", "file5", "--tool=:theirs"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy  (@) now at: kmkuslsw 5de68577 conflict | conflict
-    Parent commit (@-)      : mzvwutvl 4a44e1a9 a | a
-    Parent commit (@-)      : vruxwmqv 19e7d2ff b | b
+    Working copy  (@) now at: kmkuslsw 7337267a conflict | conflict
+    Parent commit (@-)      : mzvwutvl e2d3924b a | a
+    Parent commit (@-)      : vruxwmqv 888b6cc3 b | b
     Added 0 files, modified 1 files, removed 0 files
     Existing conflicts were resolved or abandoned from 1 commits.
     [EOF]
@@ -1409,24 +1416,25 @@ fn test_resolve_long_conflict_markers() {
     )
     .unwrap();
     let output = work_dir.run_jj(["resolve"]);
-    insta::assert_snapshot!(output, @r"
+    insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Resolving conflicts in: file
-    Working copy  (@) now at: vruxwmqv 2b985546 conflict | (conflict) conflict
-    Parent commit (@-)      : zsuskuln 64177fd4 a | a
-    Parent commit (@-)      : royxmykx db442c1e b | b
+    Working copy  (@) now at: vruxwmqv 1e254ee3 conflict | (conflict) conflict
+    Parent commit (@-)      : zsuskuln 10d994ef a | a
+    Parent commit (@-)      : royxmykx 7f215575 b | b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict
     New conflicts appeared in 1 commits:
-      vruxwmqv 2b985546 conflict | (conflict) conflict
-    Hint: To resolve the conflicts, start by updating to it:
+      vruxwmqv 1e254ee3 conflict | (conflict) conflict
+    Hint: To resolve the conflicts, start by creating a commit on top of
+    the conflicted commit:
       jj new vruxwmqv
     Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want to inspect the result with `jj diff`.
+    Once the conflicts are resolved, you can inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
     [EOF]
-    ");
+    "###);
     insta::assert_snapshot!(work_dir.run_jj(["diff", "--git"]), @r"
     diff --git a/file b/file
     --- a/file
@@ -1478,24 +1486,25 @@ fn test_resolve_long_conflict_markers() {
         "resolve",
         "--config=merge-tools.fake-editor.merge-tool-edits-conflict-markers=true",
     ]);
-    insta::assert_snapshot!(output, @r"
+    insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Resolving conflicts in: file
-    Working copy  (@) now at: vruxwmqv fac9406d conflict | (conflict) conflict
-    Parent commit (@-)      : zsuskuln 64177fd4 a | a
-    Parent commit (@-)      : royxmykx db442c1e b | b
+    Working copy  (@) now at: vruxwmqv 2481a401 conflict | (conflict) conflict
+    Parent commit (@-)      : zsuskuln 10d994ef a | a
+    Parent commit (@-)      : royxmykx 7f215575 b | b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict
     New conflicts appeared in 1 commits:
-      vruxwmqv fac9406d conflict | (conflict) conflict
-    Hint: To resolve the conflicts, start by updating to it:
+      vruxwmqv 2481a401 conflict | (conflict) conflict
+    Hint: To resolve the conflicts, start by creating a commit on top of
+    the conflicted commit:
       jj new vruxwmqv
     Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want to inspect the result with `jj diff`.
+    Once the conflicts are resolved, you can inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
     [EOF]
-    ");
+    "###);
     insta::assert_snapshot!(
         std::fs::read_to_string(test_env.env_root().join("editor")).unwrap(), @r"
     <<<<<<<<<<< Conflict 1 of 1
@@ -1553,24 +1562,25 @@ fn test_resolve_long_conflict_markers() {
         "resolve",
         r#"--config=merge-tools.fake-editor.merge-args=["$output", "$marker_length"]"#,
     ]);
-    insta::assert_snapshot!(output, @r"
+    insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Resolving conflicts in: file
-    Working copy  (@) now at: vruxwmqv 1b29631a conflict | (conflict) conflict
-    Parent commit (@-)      : zsuskuln 64177fd4 a | a
-    Parent commit (@-)      : royxmykx db442c1e b | b
+    Working copy  (@) now at: vruxwmqv 2cf0bfd3 conflict | (conflict) conflict
+    Parent commit (@-)      : zsuskuln 10d994ef a | a
+    Parent commit (@-)      : royxmykx 7f215575 b | b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict
     New conflicts appeared in 1 commits:
-      vruxwmqv 1b29631a conflict | (conflict) conflict
-    Hint: To resolve the conflicts, start by updating to it:
+      vruxwmqv 2cf0bfd3 conflict | (conflict) conflict
+    Hint: To resolve the conflicts, start by creating a commit on top of
+    the conflicted commit:
       jj new vruxwmqv
     Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want to inspect the result with `jj diff`.
+    Once the conflicts are resolved, you can inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
     [EOF]
-    ");
+    "###);
     insta::assert_snapshot!(work_dir.run_jj(["diff", "--git"]), @r"
     diff --git a/file b/file
     --- a/file
@@ -1684,24 +1694,25 @@ fn test_multiple_conflicts() {
     // Check that we can manually pick which of the conflicts to resolve first
     std::fs::write(&editor_script, "expect\n\0write\nresolution another_file\n").unwrap();
     let output = work_dir.run_jj(["resolve", "another_file"]);
-    insta::assert_snapshot!(output, @r"
+    insta::assert_snapshot!(output, @r###"
     ------- stderr -------
     Resolving conflicts in: another_file
-    Working copy  (@) now at: vruxwmqv 309e981c conflict | (conflict) conflict
-    Parent commit (@-)      : zsuskuln de7553ef a | a
-    Parent commit (@-)      : royxmykx f68bc2f0 b | b
+    Working copy  (@) now at: vruxwmqv d3584f6e conflict | (conflict) conflict
+    Parent commit (@-)      : zsuskuln 2c821f70 a | a
+    Parent commit (@-)      : royxmykx 4c2029de b | b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     this_file_has_a_very_long_name_to_test_padding 2-sided conflict
     New conflicts appeared in 1 commits:
-      vruxwmqv 309e981c conflict | (conflict) conflict
-    Hint: To resolve the conflicts, start by updating to it:
+      vruxwmqv d3584f6e conflict | (conflict) conflict
+    Hint: To resolve the conflicts, start by creating a commit on top of
+    the conflicted commit:
       jj new vruxwmqv
     Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want to inspect the result with `jj diff`.
+    Once the conflicts are resolved, you can inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
     [EOF]
-    ");
+    "###);
     insta::assert_snapshot!(work_dir.run_jj(["diff", "--git"]), @r"
     diff --git a/another_file b/another_file
     index 0000000000..a9fcc7d486 100644
@@ -1853,28 +1864,29 @@ fn test_multiple_conflicts_with_error() {
     )
     .unwrap();
     let output = work_dir.run_jj(["resolve"]);
-    insta::assert_snapshot!(output.normalize_stderr_exit_status(), @r"
+    insta::assert_snapshot!(output.normalize_stderr_exit_status(), @r###"
     ------- stderr -------
     Resolving conflicts in: file1
     Resolving conflicts in: file2
-    Working copy  (@) now at: vruxwmqv d2f3f858 conflict | (conflict) conflict
-    Parent commit (@-)      : zsuskuln 9db7fdfb a | a
-    Parent commit (@-)      : royxmykx d67e26e4 b | b
+    Working copy  (@) now at: vruxwmqv 98296abe conflict | (conflict) conflict
+    Parent commit (@-)      : zsuskuln 6c31698c a | a
+    Parent commit (@-)      : royxmykx ba0a5538 b | b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file2    2-sided conflict
     New conflicts appeared in 1 commits:
-      vruxwmqv d2f3f858 conflict | (conflict) conflict
-    Hint: To resolve the conflicts, start by updating to it:
+      vruxwmqv 98296abe conflict | (conflict) conflict
+    Hint: To resolve the conflicts, start by creating a commit on top of
+    the conflicted commit:
       jj new vruxwmqv
     Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want to inspect the result with `jj diff`.
+    Once the conflicts are resolved, you can inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
     Error: Stopped due to error after resolving 1 conflicts
     Caused by: The output file is either unchanged or empty after the editor quit (run with --debug to see the exact invocation).
     [EOF]
     [exit status: 1]
-    ");
+    "###);
     insta::assert_snapshot!(work_dir.run_jj(["diff", "--git"]), @r"
     diff --git a/file1 b/file1
     index 0000000000..95cc18629d 100644
@@ -1904,28 +1916,29 @@ fn test_multiple_conflicts_with_error() {
     )
     .unwrap();
     let output = work_dir.run_jj(["resolve"]);
-    insta::assert_snapshot!(output.normalize_stderr_exit_status(), @r"
+    insta::assert_snapshot!(output.normalize_stderr_exit_status(), @r###"
     ------- stderr -------
     Resolving conflicts in: file1
     Resolving conflicts in: file2
-    Working copy  (@) now at: vruxwmqv 0a54e8ed conflict | (conflict) conflict
-    Parent commit (@-)      : zsuskuln 9db7fdfb a | a
-    Parent commit (@-)      : royxmykx d67e26e4 b | b
+    Working copy  (@) now at: vruxwmqv 7daa6406 conflict | (conflict) conflict
+    Parent commit (@-)      : zsuskuln 6c31698c a | a
+    Parent commit (@-)      : royxmykx ba0a5538 b | b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file2    2-sided conflict
     New conflicts appeared in 1 commits:
-      vruxwmqv 0a54e8ed conflict | (conflict) conflict
-    Hint: To resolve the conflicts, start by updating to it:
+      vruxwmqv 7daa6406 conflict | (conflict) conflict
+    Hint: To resolve the conflicts, start by creating a commit on top of
+    the conflicted commit:
       jj new vruxwmqv
     Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want to inspect the result with `jj diff`.
+    Once the conflicts are resolved, you can inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
     Error: Stopped due to error after resolving 1 conflicts
     Caused by: Tool exited with exit status: 1 (run with --debug to see the exact invocation)
     [EOF]
     [exit status: 1]
-    ");
+    "###);
     insta::assert_snapshot!(work_dir.run_jj(["diff", "--git"]), @r"
     diff --git a/file1 b/file1
     index 0000000000..95cc18629d 100644
@@ -2040,10 +2053,10 @@ fn test_resolve_with_contents_of_side() {
     let output = work_dir.run_jj(["resolve", "--tool", ":ours"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy  (@) now at: znkkpsqq c83edc52 conflict | conflict
-    Parent commit (@-)      : zsuskuln 83b81681 a | a
-    Parent commit (@-)      : royxmykx 4f6eff49 b | b
-    Parent commit (@-)      : vruxwmqv 1e360aff c | c
+    Working copy  (@) now at: znkkpsqq 5410a03a conflict | conflict
+    Parent commit (@-)      : zsuskuln 72dced6e a | a
+    Parent commit (@-)      : royxmykx e5747f42 b | b
+    Parent commit (@-)      : vruxwmqv dd35236a c | c
     Added 0 files, modified 2 files, removed 0 files
     [EOF]
     ");
@@ -2062,10 +2075,10 @@ fn test_resolve_with_contents_of_side() {
     let output = work_dir.run_jj(["resolve", "--tool", ":theirs"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy  (@) now at: znkkpsqq baf3a71d conflict | conflict
-    Parent commit (@-)      : zsuskuln 83b81681 a | a
-    Parent commit (@-)      : royxmykx 4f6eff49 b | b
-    Parent commit (@-)      : vruxwmqv 1e360aff c | c
+    Working copy  (@) now at: znkkpsqq c07b2e9e conflict | conflict
+    Parent commit (@-)      : zsuskuln 72dced6e a | a
+    Parent commit (@-)      : royxmykx e5747f42 b | b
+    Parent commit (@-)      : vruxwmqv dd35236a c | c
     Added 0 files, modified 2 files, removed 0 files
     [EOF]
     ");
