@@ -17,19 +17,20 @@
 use std::any::Any;
 use std::collections::BTreeMap;
 use std::fmt::Debug;
-use std::io::Read;
+use std::pin::Pin;
 use std::time::SystemTime;
 
 use async_trait::async_trait;
 use futures::stream::BoxStream;
 use thiserror::Error;
+use tokio::io::AsyncRead;
 
 use crate::content_hash::ContentHash;
 use crate::hex_util;
 use crate::index::Index;
 use crate::merge::Merge;
 use crate::object_id::id_type;
-use crate::object_id::ObjectId;
+use crate::object_id::ObjectId as _;
 use crate::repo_path::RepoPath;
 use crate::repo_path::RepoPathBuf;
 use crate::repo_path::RepoPathComponent;
@@ -50,6 +51,7 @@ id_type!(pub TreeId { hex() });
 id_type!(pub FileId { hex() });
 id_type!(pub SymlinkId { hex() });
 id_type!(pub ConflictId { hex() });
+id_type!(pub CopyId { hex() });
 
 impl ChangeId {
     /// Returns the hex string representation of this ID, which uses `z-k`
@@ -59,6 +61,15 @@ impl ChangeId {
     }
 }
 
+impl CopyId {
+    /// Returns a placeholder copy id to be used when we don't have a real copy
+    /// id yet.
+    // TODO: Delete this
+    pub fn placeholder() -> Self {
+        Self::new(vec![])
+    }
+}
+
 #[derive(ContentHash, Debug, PartialEq, Eq, Clone, Copy, PartialOrd, Ord)]
 pub struct MillisSinceEpoch(pub i64);
 
@@ -142,6 +153,8 @@ impl MergedTreeId {
 #[derive(ContentHash, Debug, PartialEq, Eq, Clone)]
 pub struct Commit {
     pub parents: Vec<CommitId>,
+    // TODO: delete commit.predecessors when we can assume that most commits are
+    // tracked by op.commit_predecessors. (in jj 0.42 or so?)
     pub predecessors: Vec<CommitId>,
     pub root_tree: MergedTreeId,
     pub change_id: ChangeId,
@@ -193,6 +206,24 @@ pub struct CopyRecord {
     pub source_commit: CommitId,
 }
 
+/// Describes the copy history of a file. The copy object is unchanged when a
+/// file is modified.
+#[derive(ContentHash, Debug, PartialEq, Eq, Clone, PartialOrd, Ord)]
+pub struct CopyHistory {
+    /// The file's current path.
+    pub current_path: RepoPathBuf,
+    /// IDs of the files that became the current incarnation of this file.
+    ///
+    /// A newly created file has no parents. A regular copy or rename has one
+    /// parent. A merge of multiple files has multiple parents.
+    pub parents: Vec<CopyId>,
+    /// An optional piece of data to give the Copy object a different ID. May be
+    /// randomly generated. This allows a commit to say that a file was replaced
+    /// by a new incarnation of it, indicating a logically distinct file
+    /// taking the place of the previous file at the path.
+    pub salt: Vec<u8>,
+}
+
 /// Error that may occur during backend initialization.
 #[derive(Debug, Error)]
 #[error(transparent)]
@@ -266,7 +297,13 @@ pub type BackendResult<T> = Result<T, BackendError>;
 
 #[derive(ContentHash, Debug, PartialEq, Eq, Clone, Hash)]
 pub enum TreeValue {
-    File { id: FileId, executable: bool },
+    // TODO: When there's a CopyId here, the copy object's path must match
+    // the path identified by the tree.
+    File {
+        id: FileId,
+        executable: bool,
+        copy_id: CopyId,
+    },
     Symlink(SymlinkId),
     Tree(TreeId),
     GitSubmodule(CommitId),
@@ -421,18 +458,45 @@ pub trait Backend: Send + Sync + Debug {
     /// sent.
     fn concurrency(&self) -> usize;
 
-    async fn read_file(&self, path: &RepoPath, id: &FileId) -> BackendResult<Box<dyn Read>>;
+    async fn read_file(
+        &self,
+        path: &RepoPath,
+        id: &FileId,
+    ) -> BackendResult<Pin<Box<dyn AsyncRead>>>;
 
     async fn write_file(
         &self,
         path: &RepoPath,
-        contents: &mut (dyn Read + Send),
+        contents: &mut (dyn AsyncRead + Send + Unpin),
     ) -> BackendResult<FileId>;
 
     async fn read_symlink(&self, path: &RepoPath, id: &SymlinkId) -> BackendResult<String>;
 
     async fn write_symlink(&self, path: &RepoPath, target: &str) -> BackendResult<SymlinkId>;
 
+    /// Read the specified `CopyHistory` object.
+    ///
+    /// Backends that don't support copy tracking may return
+    /// `BackendError::Unsupported`.
+    async fn read_copy(&self, id: &CopyId) -> BackendResult<CopyHistory>;
+
+    /// Write the `CopyHistory` object and return its ID.
+    ///
+    /// Backends that don't support copy tracking may return
+    /// `BackendError::Unsupported`.
+    async fn write_copy(&self, copy: &CopyHistory) -> BackendResult<CopyId>;
+
+    /// Find all copy histories that are related to the specified one. This is
+    /// defined as those that are ancestors of the given specified one, plus
+    /// their descendants. Children must be returned before parents.
+    ///
+    /// It is valid (but wasteful) to include other copy histories, such as
+    /// siblings, or even completely unrelated copy histories.
+    ///
+    /// Backends that don't support copy tracking may return
+    /// `BackendError::Unsupported`.
+    async fn get_related_copies(&self, copy_id: &CopyId) -> BackendResult<Vec<CopyHistory>>;
+
     async fn read_tree(&self, path: &RepoPath, id: &TreeId) -> BackendResult<Tree>;
 
     async fn write_tree(&self, path: &RepoPath, contents: &Tree) -> BackendResult<TreeId>;
