@@ -166,7 +166,7 @@ pub struct DetachedCommitBuilder {
 }
 
 impl DetachedCommitBuilder {
-    /// Only called from [`MutRepo::new_commit`]. Use that function instead.
+    /// Only called from [`MutableRepo::new_commit`]. Use that function instead.
     pub(crate) fn for_new_commit(
         repo: &dyn Repo,
         settings: &UserSettings,
@@ -197,7 +197,8 @@ impl DetachedCommitBuilder {
         }
     }
 
-    /// Only called from [`MutRepo::rewrite_commit`]. Use that function instead.
+    /// Only called from [`MutableRepo::rewrite_commit`]. Use that function
+    /// instead.
     pub(crate) fn for_rewrite_from(
         repo: &dyn Repo,
         settings: &UserSettings,
@@ -348,8 +349,10 @@ impl DetachedCommitBuilder {
 
     /// Writes new commit and makes it visible in the `mut_repo`.
     pub fn write(self, mut_repo: &mut MutableRepo) -> BackendResult<Commit> {
+        let predecessors = self.commit.predecessors.clone();
         let commit = write_to_store(&self.store, self.commit, &self.sign_settings)?;
         mut_repo.add_head(&commit)?;
+        mut_repo.set_predecessors(commit.id().clone(), predecessors);
         if let Some(rewrite_source) = self.rewrite_source {
             if rewrite_source.change_id() == commit.change_id() {
                 mut_repo.set_rewritten_commit(rewrite_source.id().clone(), commit.id().clone());
