@@ -25,7 +25,7 @@ use super::composite::CompositeIndex;
 use super::composite::DynIndexSegment;
 use crate::backend::ChangeId;
 use crate::backend::CommitId;
-use crate::object_id::ObjectId;
+use crate::object_id::ObjectId as _;
 
 /// Global index position.
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Clone, Copy, Hash)]
@@ -114,37 +114,10 @@ impl<'a> IndexEntry<'a> {
         self.source.parent_positions(self.local_pos)
     }
 
-    pub fn parents(&self) -> impl ExactSizeIterator<Item = IndexEntry<'a>> {
+    pub fn parents(&self) -> impl ExactSizeIterator<Item = IndexEntry<'a>> + use<'a> {
         let composite = CompositeIndex::new(self.source);
         self.parent_positions()
             .into_iter()
             .map(move |pos| composite.entry_by_pos(pos))
     }
 }
-
-/// Wrapper to sort `IndexPosition` by its generation number.
-///
-/// This is similar to `IndexEntry` newtypes, but optimized for size and cache
-/// locality. The original `IndexEntry` will have to be looked up when needed.
-#[derive(Clone, Copy, Debug, Ord, PartialOrd)]
-pub(super) struct IndexPositionByGeneration {
-    pub generation: u32,    // order by generation number
-    pub pos: IndexPosition, // tie breaker
-}
-
-impl Eq for IndexPositionByGeneration {}
-
-impl PartialEq for IndexPositionByGeneration {
-    fn eq(&self, other: &Self) -> bool {
-        self.pos == other.pos
-    }
-}
-
-impl From<&IndexEntry<'_>> for IndexPositionByGeneration {
-    fn from(entry: &IndexEntry<'_>) -> Self {
-        IndexPositionByGeneration {
-            generation: entry.generation_number(),
-            pos: entry.position(),
-        }
-    }
-}
