@@ -26,6 +26,7 @@ mod entry;
 mod mutable;
 mod readonly;
 mod rev_walk;
+mod rev_walk_queue;
 pub mod revset_engine;
 mod revset_graph_iterator;
 mod store;
@@ -175,9 +176,6 @@ mod tests {
         let id_1 = CommitId::from_hex("111111");
         let change_id1 = new_change_id();
         let id_2 = CommitId::from_hex("222222");
-        #[expect(clippy::redundant_clone)] // Work around nightly clippy false positive
-        // TODO: Remove the exception after https://github.com/rust-lang/rust-clippy/issues/10577
-        // is fixed or file a new bug.
         let change_id2 = change_id1.clone();
         mutable_segment.add_commit_data(id_0.clone(), change_id0, &[]);
         mutable_segment.add_commit_data(id_1.clone(), change_id1.clone(), &[id_0.clone()]);
@@ -1099,7 +1097,7 @@ mod tests {
         // to (1,2) and matches the (1,2) of the first input set.
         assert_eq!(
             index.common_ancestors(&[id_1.clone(), id_2.clone()], &[id_5]),
-            vec![id_1.clone(), id_2.clone()]
+            vec![id_2.clone(), id_1.clone()]
         );
         assert_eq!(index.common_ancestors(&[id_1, id_2], &[id_3]), vec![id_0]);
     }
@@ -1200,9 +1198,16 @@ mod tests {
         // Multiple heads
         assert_eq!(
             index
-                .heads(&mut [id_4.clone(), id_3.clone()].iter())
+                .heads(&mut [id_3.clone(), id_4.clone()].iter())
                 .unwrap(),
-            vec![id_3.clone(), id_4]
+            vec![id_4.clone(), id_3.clone()]
+        );
+        // Duplicated inputs
+        assert_eq!(
+            index
+                .heads(&mut [id_4.clone(), id_3.clone(), id_4.clone()].iter())
+                .unwrap(),
+            vec![id_4.clone(), id_3.clone()]
         );
         // Merge commit and ancestors
         assert_eq!(
@@ -1214,7 +1219,7 @@ mod tests {
             index
                 .heads(&mut [id_5.clone(), id_3.clone()].iter())
                 .unwrap(),
-            vec![id_3.clone(), id_5.clone()]
+            vec![id_5.clone(), id_3.clone()]
         );
 
         assert_eq!(
