@@ -15,12 +15,13 @@
 #![allow(missing_docs)]
 
 use std::any::Any;
-use std::collections::HashSet;
+use std::collections::HashMap;
 use std::fs;
 use std::io;
 use std::io::Write as _;
 use std::path::Path;
 use std::path::PathBuf;
+use std::slice;
 use std::sync::Arc;
 
 use itertools::Itertools as _;
@@ -49,6 +50,7 @@ use crate::index::ReadonlyIndex;
 use crate::object_id::ObjectId as _;
 use crate::op_store::OpStoreError;
 use crate::op_store::OperationId;
+use crate::op_walk;
 use crate::operation::Operation;
 use crate::store::Store;
 
@@ -185,47 +187,59 @@ impl DefaultIndexStore {
         operation: &Operation,
         store: &Arc<Store>,
     ) -> Result<Arc<ReadonlyIndexSegment>, DefaultIndexStoreError> {
-        let view = operation.view()?;
+        tracing::info!("scanning operations to index");
         let operations_dir = self.operations_dir();
         let commit_id_length = store.commit_id_length();
         let change_id_length = store.change_id_length();
-        let mut visited_heads: HashSet<CommitId> =
-            view.all_referenced_commit_ids().cloned().collect();
-        let mut historical_heads: Vec<(CommitId, OperationId)> = visited_heads
+        let ops_to_visit: Vec<_> =
+            op_walk::walk_ancestors(slice::from_ref(operation)).try_collect()?;
+        // Pick the latest existing ancestor operation as the parent segment.
+        let parent_op = ops_to_visit
             .iter()
-            .map(|commit_id| (commit_id.clone(), operation.id().clone()))
-            .collect();
-        let mut parent_op_id: Option<OperationId> = None;
-        for op in dag_walk::dfs_ok(
-            [Ok(operation.clone())],
-            |op: &Operation| op.id().clone(),
-            |op: &Operation| op.parents().collect_vec(),
-        ) {
-            let op = op?;
-            // Pick the latest existing ancestor operation as the parent
-            // segment. Perhaps, breadth-first search is more appropriate here,
-            // but that wouldn't matter in practice as the operation log is
-            // mostly linear.
-            if parent_op_id.is_none() && operations_dir.join(op.id().hex()).is_file() {
-                parent_op_id = Some(op.id().clone());
+            .find(|op| operations_dir.join(op.id().hex()).is_file())
+            .cloned();
+        // Remove ancestors of the latest existing operation, which should have
+        // been indexed in the parent segment. This could be optimized for
+        // linear history, but parent_op is often None.
+        let ops_to_visit = if let Some(op) = &parent_op {
+            let mut wanted_ops: HashMap<&OperationId, &Operation> =
+                ops_to_visit.iter().map(|op| (op.id(), op)).collect();
+            let mut work = vec![op.id()];
+            while let Some(id) = work.pop() {
+                if let Some(op) = wanted_ops.remove(id) {
+                    work.extend(op.parent_ids());
+                }
             }
-            // TODO: no need to walk ancestors of the parent_op_id operation
+            ops_to_visit
+                .iter()
+                .filter(|op| wanted_ops.contains_key(op.id()))
+                .cloned()
+                .collect()
+        } else {
+            ops_to_visit
+        };
+        tracing::info!(
+            ops_count = ops_to_visit.len(),
+            "collecting head commits to index"
+        );
+        let mut historical_heads: HashMap<CommitId, OperationId> = HashMap::new();
+        for op in &ops_to_visit {
             for commit_id in op.view()?.all_referenced_commit_ids() {
-                if visited_heads.insert(commit_id.clone()) {
-                    historical_heads.push((commit_id.clone(), op.id().clone()));
+                if !historical_heads.contains_key(commit_id) {
+                    historical_heads.insert(commit_id.clone(), op.id().clone());
                 }
             }
         }
         let maybe_parent_file;
         let mut mutable_index;
-        match parent_op_id {
+        match &parent_op {
             None => {
                 maybe_parent_file = None;
                 mutable_index = DefaultMutableIndex::full(commit_id_length, change_id_length);
             }
-            Some(parent_op_id) => {
+            Some(op) => {
                 let parent_file = self.load_index_segments_at_operation(
-                    &parent_op_id,
+                    op.id(),
                     commit_id_length,
                     change_id_length,
                 )?;
@@ -263,7 +277,9 @@ impl DefaultIndexStore {
                 .map(|(commit_id, op_id)| get_commit_with_op(commit_id, op_id)),
             |(CommitByCommitterTimestamp(commit), _)| commit.id().clone(),
             |(CommitByCommitterTimestamp(commit), op_id)| {
-                itertools::chain(commit.parent_ids(), commit.predecessor_ids())
+                // TODO: preserve operation.commit_predecessors.keys() instead
+                // of commit.predecessors.
+                itertools::chain(commit.parent_ids(), &commit.store_commit().predecessors)
                     .filter(|&id| !parent_file_has_id(id))
                     .map(|commit_id| get_commit_with_op(commit_id, op_id))
                     .collect_vec()
