@@ -90,7 +90,7 @@ pub trait FileFixer {
     /// TODO: Better error handling so we can tell the user what went wrong with
     /// each failed input.
     fn fix_files<'a>(
-        &self,
+        &mut self,
         store: &Store,
         files_to_fix: &'a HashSet<FileToFix>,
     ) -> Result<HashMap<&'a FileToFix, FileId>, FixError>;
@@ -135,7 +135,7 @@ where
 {
     /// Applies `fix_fn()` to the inputs and stores the resulting file content.
     fn fix_files<'a>(
-        &self,
+        &mut self,
         store: &Store,
         files_to_fix: &'a HashSet<FileToFix>,
     ) -> Result<HashMap<&'a FileToFix, FileId>, FixError> {
@@ -178,7 +178,7 @@ pub fn fix_files(
     matcher: &dyn Matcher,
     include_unchanged_files: bool,
     repo_mut: &mut MutableRepo,
-    file_fixer: &impl FileFixer,
+    file_fixer: &mut impl FileFixer,
 ) -> Result<FixSummary, FixError> {
     let mut summary = FixSummary::default();
 
@@ -243,7 +243,12 @@ pub fn fix_files(
                 for term in after.into_iter().flatten() {
                     // We currently only support fixing the content of normal files, so we skip
                     // directories and symlinks, and we ignore the executable bit.
-                    if let TreeValue::File { id, executable: _ } = term {
+                    if let TreeValue::File {
+                        id,
+                        executable: _,
+                        copy_id: _,
+                    } = term
+                    {
                         // TODO: Skip the file if its content is larger than some configured size,
                         // preferably without actually reading it yet.
                         let file_to_fix = FileToFix {
@@ -286,7 +291,12 @@ pub fn fix_files(
         for repo_path in repo_paths {
             let old_value = old_tree.path_value(repo_path)?;
             let new_value = old_value.map(|old_term| {
-                if let Some(TreeValue::File { id, executable }) = old_term {
+                if let Some(TreeValue::File {
+                    id,
+                    executable,
+                    copy_id,
+                }) = old_term
+                {
                     let file_to_fix = FileToFix {
                         file_id: id.clone(),
                         repo_path: repo_path.clone(),
@@ -295,6 +305,7 @@ pub fn fix_files(
                         return Some(TreeValue::File {
                             id: new_id.clone(),
                             executable: *executable,
+                            copy_id: copy_id.clone(),
                         });
                     }
                 }
