@@ -60,8 +60,7 @@ impl FsmonitorSettings {
         let name = "core.fsmonitor";
         match settings.get_string(name)?.as_ref() {
             "watchman" => Ok(Self::Watchman(WatchmanConfig {
-                // TODO: rename to "register-snapshot-trigger" for consistency?
-                register_trigger: settings.get_bool("core.watchman.register_snapshot_trigger")?,
+                register_trigger: settings.get_bool("core.watchman.register-snapshot-trigger")?,
             })),
             "test" => Err(ConfigGetError::Type {
                 name: name.to_owned(),
@@ -86,7 +85,7 @@ pub mod watchman {
     use std::path::Path;
     use std::path::PathBuf;
 
-    use itertools::Itertools;
+    use itertools::Itertools as _;
     use thiserror::Error;
     use tracing::info;
     use tracing::instrument;
@@ -147,7 +146,7 @@ pub mod watchman {
         }
     }
 
-    #[allow(missing_docs)]
+    #[expect(missing_docs)]
     #[derive(Debug, Error)]
     pub enum Error {
         #[error("Could not connect to Watchman")]
