@@ -23,9 +23,9 @@ use std::fmt::Formatter;
 use std::fs;
 use std::io;
 use std::io::Cursor;
-use std::io::Read;
 use std::path::Path;
 use std::path::PathBuf;
+use std::pin::Pin;
 use std::process::Command;
 use std::process::ExitStatus;
 use std::str;
@@ -45,6 +45,8 @@ use pollster::FutureExt as _;
 use prost::Message as _;
 use smallvec::SmallVec;
 use thiserror::Error;
+use tokio::io::AsyncRead;
+use tokio::io::AsyncReadExt as _;
 
 use crate::backend::make_root_commit;
 use crate::backend::Backend;
@@ -58,6 +60,8 @@ use crate::backend::CommitId;
 use crate::backend::Conflict;
 use crate::backend::ConflictId;
 use crate::backend::ConflictTerm;
+use crate::backend::CopyHistory;
+use crate::backend::CopyId;
 use crate::backend::CopyRecord;
 use crate::backend::FileId;
 use crate::backend::MergedTreeId;
@@ -425,7 +429,7 @@ impl GitBackend {
         self.save_extra_metadata_table(mut_table, &table_lock)
     }
 
-    fn read_file_sync(&self, id: &FileId) -> BackendResult<Box<dyn Read>> {
+    fn read_file_sync(&self, id: &FileId) -> BackendResult<Vec<u8>> {
         let git_blob_id = validate_git_object_id(id)?;
         let locked_repo = self.lock_git_repo();
         let mut blob = locked_repo
@@ -433,7 +437,7 @@ impl GitBackend {
             .map_err(|err| map_not_found_err(err, id))?
             .try_into_blob()
             .map_err(|err| to_read_object_err(err, id))?;
-        Ok(Box::new(Cursor::new(blob.take_data())))
+        Ok(blob.take_data())
     }
 
     fn new_diff_platform(&self) -> BackendResult<gix::diff::blob::Platform> {
@@ -969,17 +973,22 @@ impl Backend for GitBackend {
         1
     }
 
-    async fn read_file(&self, _path: &RepoPath, id: &FileId) -> BackendResult<Box<dyn Read>> {
-        self.read_file_sync(id)
+    async fn read_file(
+        &self,
+        _path: &RepoPath,
+        id: &FileId,
+    ) -> BackendResult<Pin<Box<dyn AsyncRead>>> {
+        let data = self.read_file_sync(id)?;
+        Ok(Box::pin(Cursor::new(data)))
     }
 
     async fn write_file(
         &self,
         _path: &RepoPath,
-        contents: &mut (dyn Read + Send),
+        contents: &mut (dyn AsyncRead + Send + Unpin),
     ) -> BackendResult<FileId> {
         let mut bytes = Vec::new();
-        contents.read_to_end(&mut bytes).unwrap();
+        contents.read_to_end(&mut bytes).await.unwrap();
         let locked_repo = self.lock_git_repo();
         let oid = locked_repo
             .write_blob(bytes)
@@ -1015,6 +1024,24 @@ impl Backend for GitBackend {
         Ok(SymlinkId::new(oid.as_bytes().to_vec()))
     }
 
+    async fn read_copy(&self, _id: &CopyId) -> BackendResult<CopyHistory> {
+        Err(BackendError::Unsupported(
+            "The Git backend doesn't support tracked copies yet".to_string(),
+        ))
+    }
+
+    async fn write_copy(&self, _contents: &CopyHistory) -> BackendResult<CopyId> {
+        Err(BackendError::Unsupported(
+            "The Git backend doesn't support tracked copies yet".to_string(),
+        ))
+    }
+
+    async fn get_related_copies(&self, _copy_id: &CopyId) -> BackendResult<Vec<CopyHistory>> {
+        Err(BackendError::Unsupported(
+            "The Git backend doesn't support tracked copies yet".to_string(),
+        ))
+    }
+
     async fn read_tree(&self, _path: &RepoPath, id: &TreeId) -> BackendResult<Tree> {
         if id == &self.empty_tree_id {
             return Ok(Tree::default());
@@ -1050,6 +1077,7 @@ impl Backend for GitBackend {
                             TreeValue::File {
                                 id,
                                 executable: false,
+                                copy_id: CopyId::placeholder(),
                             },
                         )
                     }
@@ -1061,6 +1089,7 @@ impl Backend for GitBackend {
                         TreeValue::File {
                             id,
                             executable: true,
+                            copy_id: CopyId::placeholder(),
                         },
                     )
                 }
@@ -1089,6 +1118,7 @@ impl Backend for GitBackend {
                     TreeValue::File {
                         id,
                         executable: false,
+                        copy_id: _, // TODO: Use the value
                     } => gix::objs::tree::Entry {
                         mode: gix::object::tree::EntryKind::Blob.into(),
                         filename: name.into(),
@@ -1097,6 +1127,7 @@ impl Backend for GitBackend {
                     TreeValue::File {
                         id,
                         executable: true,
+                        copy_id: _, // TODO: Use the value
                     } => gix::objs::tree::Entry {
                         mode: gix::object::tree::EntryKind::BlobExecutable.into(),
                         filename: name.into(),
@@ -1137,15 +1168,8 @@ impl Backend for GitBackend {
     }
 
     fn read_conflict(&self, _path: &RepoPath, id: &ConflictId) -> BackendResult<Conflict> {
-        let mut file = self.read_file_sync(&FileId::new(id.to_bytes()))?;
-        let mut data = String::new();
-        file.read_to_string(&mut data)
-            .map_err(|err| BackendError::ReadObject {
-                object_type: "conflict".to_owned(),
-                hash: id.hex(),
-                source: err.into(),
-            })?;
-        let json: serde_json::Value = serde_json::from_str(&data).unwrap();
+        let data = self.read_file_sync(&FileId::new(id.to_bytes()))?;
+        let json: serde_json::Value = serde_json::from_slice(&data).unwrap();
         Ok(Conflict {
             removes: conflict_term_list_from_json(json.get("removes").unwrap()),
             adds: conflict_term_list_from_json(json.get("adds").unwrap()),
@@ -1523,7 +1547,11 @@ fn conflict_term_from_json(json: &serde_json::Value) -> ConflictTerm {
 
 fn tree_value_to_json(value: &TreeValue) -> serde_json::Value {
     match value {
-        TreeValue::File { id, executable } => serde_json::json!({
+        TreeValue::File {
+            id,
+            executable,
+            copy_id: _,
+        } => serde_json::json!({
              "file": {
                  "id": id.hex(),
                  "executable": executable,
@@ -1549,6 +1577,7 @@ fn tree_value_from_json(json: &serde_json::Value) -> TreeValue {
         TreeValue::File {
             id: FileId::new(bytes_vec_from_json(json_file.get("id").unwrap())),
             executable: json_file.get("executable").unwrap().as_bool().unwrap(),
+            copy_id: CopyId::placeholder(),
         }
     } else if let Some(json_id) = json.get("symlink_id") {
         TreeValue::Symlink(SymlinkId::new(bytes_vec_from_json(json_id)))
@@ -1574,8 +1603,6 @@ mod tests {
     use pollster::FutureExt as _;
 
     use super::*;
-    use crate::config::ConfigLayer;
-    use crate::config::ConfigSource;
     use crate::config::StackedConfig;
     use crate::content_hash::blake2b_hash;
     use crate::tests::new_temp_dir;
@@ -1759,7 +1786,8 @@ mod tests {
             file.value(),
             &TreeValue::File {
                 id: FileId::from_bytes(blob1.as_bytes()),
-                executable: false
+                executable: false,
+                copy_id: CopyId::placeholder(),
             }
         );
         assert_eq!(symlink.name().as_internal_str(), "symlink");
@@ -1877,7 +1905,7 @@ mod tests {
 
     #[test]
     fn round_trip_change_id_via_git_header() {
-        let settings = user_settings_with_change_id();
+        let settings = user_settings();
         let temp_dir = new_temp_dir();
 
         let store_path = temp_dir.path().join("store");
@@ -2168,7 +2196,7 @@ mod tests {
             parents: vec![backend.root_commit_id().clone()],
             predecessors: vec![],
             root_tree: MergedTreeId::Legacy(backend.empty_tree_id().clone()),
-            change_id: ChangeId::new(vec![]),
+            change_id: ChangeId::new(vec![42; 16]),
             description: "initial".to_string(),
             author: signature.clone(),
             committer: signature,
@@ -2288,7 +2316,7 @@ mod tests {
             parents: vec![backend.root_commit_id().clone()],
             predecessors: vec![],
             root_tree: MergedTreeId::Legacy(backend.empty_tree_id().clone()),
-            change_id: ChangeId::new(vec![]),
+            change_id: ChangeId::new(vec![42; 16]),
             description: "initial".to_string(),
             author: create_signature(),
             committer: create_signature(),
@@ -2313,8 +2341,9 @@ mod tests {
         tree 4b825dc642cb6eb9a060e54bf8d69288fbee4904
         author Someone <someone@example.com> 0 +0000
         committer Someone <someone@example.com> 0 +0000
+        change-id xpxpxpxpxpxpxpxpxpxpxpxpxpxpxpxp
         gpgsig test sig
-         hash=9ad9526c3b2103c41a229f2f3c82d107a0ecd902f476a855f0e1dd5f7bef1430663de12749b73e293a877113895a8a2a0f29da4bbc5a5f9a19c3523fb0e53518
+         hash=03feb0caccbacce2e7b7bca67f4c82292dd487e669ed8a813120c9f82d3fd0801420a1f5d05e1393abfe4e9fc662399ec4a9a1898c5f1e547e0044a52bd4bd29
 
         initial
         ");
@@ -2328,12 +2357,13 @@ mod tests {
 
         insta::assert_snapshot!(std::str::from_utf8(&sig.sig).unwrap(), @r"
         test sig
-        hash=9ad9526c3b2103c41a229f2f3c82d107a0ecd902f476a855f0e1dd5f7bef1430663de12749b73e293a877113895a8a2a0f29da4bbc5a5f9a19c3523fb0e53518
+        hash=03feb0caccbacce2e7b7bca67f4c82292dd487e669ed8a813120c9f82d3fd0801420a1f5d05e1393abfe4e9fc662399ec4a9a1898c5f1e547e0044a52bd4bd29
         ");
         insta::assert_snapshot!(std::str::from_utf8(&sig.data).unwrap(), @r"
         tree 4b825dc642cb6eb9a060e54bf8d69288fbee4904
         author Someone <someone@example.com> 0 +0000
         committer Someone <someone@example.com> 0 +0000
+        change-id xpxpxpxpxpxpxpxpxpxpxpxpxpxpxpxp
 
         initial
         ");
@@ -2362,12 +2392,4 @@ mod tests {
         let config = StackedConfig::with_defaults();
         UserSettings::from_config(config).unwrap()
     }
-
-    fn user_settings_with_change_id() -> UserSettings {
-        let mut config = StackedConfig::with_defaults();
-        let mut layer = ConfigLayer::empty(ConfigSource::Default);
-        layer.set_value("git.write-change-id-header", true).unwrap();
-        config.add_layer(layer);
-        UserSettings::from_config(config).unwrap()
-    }
 }
