@@ -42,6 +42,7 @@ pub mod default_index;
 pub mod default_submodule_store;
 pub mod diff;
 pub mod dsl_util;
+pub mod evolution;
 pub mod extensions_map;
 pub mod file_util;
 pub mod files;
@@ -112,6 +113,7 @@ pub mod submodule_store;
 #[cfg(feature = "testing")]
 pub mod test_signing_backend;
 pub mod time_util;
+pub mod trailer;
 pub mod transaction;
 pub mod tree;
 pub mod tree_builder;
