@@ -19,8 +19,6 @@ use std::collections::BTreeMap;
 use std::collections::HashMap;
 use std::collections::HashSet;
 use std::fmt::Debug;
-use std::fmt::Error;
-use std::fmt::Formatter;
 use std::iter;
 use std::time::SystemTime;
 
@@ -35,33 +33,15 @@ use crate::content_hash::ContentHash;
 use crate::merge::Merge;
 use crate::object_id::id_type;
 use crate::object_id::HexPrefix;
-use crate::object_id::ObjectId;
+use crate::object_id::ObjectId as _;
 use crate::object_id::PrefixResolution;
-
-#[derive(ContentHash, PartialEq, Eq, PartialOrd, Ord, Clone, Hash)]
-pub struct WorkspaceId(String);
-
-impl Debug for WorkspaceId {
-    fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), Error> {
-        f.debug_tuple("WorkspaceId").field(&self.0).finish()
-    }
-}
-
-impl Default for WorkspaceId {
-    fn default() -> Self {
-        Self("default".to_string())
-    }
-}
-
-impl WorkspaceId {
-    pub fn new(value: String) -> Self {
-        Self(value)
-    }
-
-    pub fn as_str(&self) -> &str {
-        &self.0
-    }
-}
+use crate::ref_name::GitRefNameBuf;
+use crate::ref_name::RefName;
+use crate::ref_name::RefNameBuf;
+use crate::ref_name::RemoteName;
+use crate::ref_name::RemoteNameBuf;
+use crate::ref_name::RemoteRefSymbol;
+use crate::ref_name::WorkspaceNameBuf;
 
 id_type!(pub ViewId { hex() });
 id_type!(pub OperationId { hex() });
@@ -187,16 +167,16 @@ impl RemoteRef {
     }
 
     /// Returns true if the ref is supposed to be merged in to the local ref.
-    pub fn is_tracking(&self) -> bool {
-        self.state == RemoteRefState::Tracking
+    pub fn is_tracked(&self) -> bool {
+        self.state == RemoteRefState::Tracked
     }
 
     /// Target that should have been merged in to the local ref.
     ///
     /// Use this as the base or known target when merging new remote ref in to
     /// local or pushing local ref to remote.
-    pub fn tracking_target(&self) -> &RefTarget {
-        if self.is_tracking() {
+    pub fn tracked_target(&self) -> &RefTarget {
+        if self.is_tracked() {
             &self.target
         } else {
             RefTarget::absent_ref()
@@ -211,7 +191,7 @@ pub enum RemoteRefState {
     New,
     /// Remote ref has been merged in to the local ref. Incoming ref will be
     /// merged, too.
-    Tracking,
+    Tracked,
 }
 
 /// Helper to strip redundant `Option<T>` from `RefTarget` lookup result.
@@ -259,7 +239,7 @@ pub struct BookmarkTarget<'a> {
     /// The commit the bookmark points to locally.
     pub local_target: &'a RefTarget,
     /// `(remote_name, remote_ref)` pairs in lexicographical order.
-    pub remote_refs: Vec<(&'a str, &'a RemoteRef)>,
+    pub remote_refs: Vec<(&'a RemoteName, &'a RemoteRef)>,
 }
 
 /// Represents the way the repo looks at a given time, just like how a Tree
@@ -268,10 +248,10 @@ pub struct BookmarkTarget<'a> {
 pub struct View {
     /// All head commits
     pub head_ids: HashSet<CommitId>,
-    pub local_bookmarks: BTreeMap<String, RefTarget>,
-    pub tags: BTreeMap<String, RefTarget>,
-    pub remote_views: BTreeMap<String, RemoteView>,
-    pub git_refs: BTreeMap<String, RefTarget>,
+    pub local_bookmarks: BTreeMap<RefNameBuf, RefTarget>,
+    pub tags: BTreeMap<RefNameBuf, RefTarget>,
+    pub remote_views: BTreeMap<RemoteNameBuf, RemoteView>,
+    pub git_refs: BTreeMap<GitRefNameBuf, RefTarget>,
     /// The commit the Git HEAD points to.
     // TODO: Support multiple Git worktrees?
     // TODO: Do we want to store the current bookmark name too?
@@ -279,7 +259,7 @@ pub struct View {
     // The commit that *should be* checked out in the workspace. Note that the working copy
     // (.jj/working_copy/) has the source of truth about which commit *is* checked out (to be
     // precise: the commit to which we most recently completed an update to).
-    pub wc_commit_ids: HashMap<WorkspaceId, CommitId>,
+    pub wc_commit_ids: BTreeMap<WorkspaceNameBuf, CommitId>,
 }
 
 impl View {
@@ -295,7 +275,7 @@ impl View {
             remote_views: BTreeMap::new(),
             git_refs: BTreeMap::new(),
             git_head: RefTarget::absent(),
-            wc_commit_ids: HashMap::new(),
+            wc_commit_ids: BTreeMap::new(),
         }
     }
 
@@ -308,7 +288,7 @@ impl View {
             remote_views: BTreeMap::new(),
             git_refs: BTreeMap::new(),
             git_head: RefTarget::absent(),
-            wc_commit_ids: HashMap::new(),
+            wc_commit_ids: BTreeMap::new(),
         }
     }
 }
@@ -320,37 +300,34 @@ pub struct RemoteView {
     // has been deleted locally and you pull from a remote, maybe it should make a difference
     // whether the bookmark is known to have existed on the remote. We may not want to resurrect
     // the bookmark if the bookmark's state on the remote was just not known.
-    pub bookmarks: BTreeMap<String, RemoteRef>,
-    // TODO: pub tags: BTreeMap<String, RemoteRef>,
+    pub bookmarks: BTreeMap<RefNameBuf, RemoteRef>,
+    // TODO: pub tags: BTreeMap<RefNameBuf, RemoteRef>,
 }
 
 /// Iterates pair of local and remote bookmarks by bookmark name.
 pub(crate) fn merge_join_bookmark_views<'a>(
-    local_bookmarks: &'a BTreeMap<String, RefTarget>,
-    remote_views: &'a BTreeMap<String, RemoteView>,
-) -> impl Iterator<Item = (&'a str, BookmarkTarget<'a>)> {
+    local_bookmarks: &'a BTreeMap<RefNameBuf, RefTarget>,
+    remote_views: &'a BTreeMap<RemoteNameBuf, RemoteView>,
+) -> impl Iterator<Item = (&'a RefName, BookmarkTarget<'a>)> {
     let mut local_bookmarks_iter = local_bookmarks
         .iter()
-        .map(|(bookmark_name, target)| (bookmark_name.as_str(), target))
+        .map(|(bookmark_name, target)| (&**bookmark_name, target))
         .peekable();
     let mut remote_bookmarks_iter = flatten_remote_bookmarks(remote_views).peekable();
 
     iter::from_fn(move || {
         // Pick earlier bookmark name
-        let (bookmark_name, local_target) = if let Some(&((remote_bookmark_name, _), _)) =
-            remote_bookmarks_iter.peek()
+        let (bookmark_name, local_target) = if let Some((symbol, _)) = remote_bookmarks_iter.peek()
         {
             local_bookmarks_iter
-                .next_if(|&(local_bookmark_name, _)| local_bookmark_name <= remote_bookmark_name)
-                .unwrap_or((remote_bookmark_name, RefTarget::absent_ref()))
+                .next_if(|&(local_bookmark_name, _)| local_bookmark_name <= symbol.name)
+                .unwrap_or((symbol.name, RefTarget::absent_ref()))
         } else {
             local_bookmarks_iter.next()?
         };
         let remote_refs = remote_bookmarks_iter
-            .peeking_take_while(|&((remote_bookmark_name, _), _)| {
-                remote_bookmark_name == bookmark_name
-            })
-            .map(|((_, remote_name), remote_ref)| (remote_name, remote_ref))
+            .peeking_take_while(|(symbol, _)| symbol.name == bookmark_name)
+            .map(|(symbol, remote_ref)| (symbol.remote, remote_ref))
             .collect();
         let bookmark_target = BookmarkTarget {
             local_target,
@@ -360,23 +337,19 @@ pub(crate) fn merge_join_bookmark_views<'a>(
     })
 }
 
-/// Iterates bookmark `((name, remote_name), remote_ref)`s in lexicographical
-/// order.
+/// Iterates bookmark `(symbol, remote_ref)`s in lexicographical order.
 pub(crate) fn flatten_remote_bookmarks(
-    remote_views: &BTreeMap<String, RemoteView>,
-) -> impl Iterator<Item = ((&str, &str), &RemoteRef)> {
+    remote_views: &BTreeMap<RemoteNameBuf, RemoteView>,
+) -> impl Iterator<Item = (RemoteRefSymbol<'_>, &RemoteRef)> {
     remote_views
         .iter()
-        .map(|(remote_name, remote_view)| {
+        .map(|(remote, remote_view)| {
             remote_view
                 .bookmarks
                 .iter()
-                .map(move |(bookmark_name, remote_ref)| {
-                    let full_name = (bookmark_name.as_str(), remote_name.as_str());
-                    (full_name, remote_ref)
-                })
+                .map(move |(name, remote_ref)| (name.to_remote_symbol(remote), remote_ref))
         })
-        .kmerge_by(|(full_name1, _), (full_name2, _)| full_name1 < full_name2)
+        .kmerge_by(|(symbol1, _), (symbol2, _)| symbol1 < symbol2)
 }
 
 /// Represents an operation (transaction) on the repo view, just like how a
@@ -396,6 +369,22 @@ pub struct Operation {
     pub view_id: ViewId,
     pub parents: Vec<OperationId>,
     pub metadata: OperationMetadata,
+    /// Mapping from new commit to its predecessors, or `None` if predecessors
+    /// weren't recorded when the operation was written.
+    ///
+    /// * `commit_id: []` if the commit was newly created.
+    /// * `commit_id: [predecessor_id, ..]` if the commit was rewritten.
+    ///
+    /// This mapping preserves all transitive predecessors if a commit was
+    /// rewritten multiple times within the same transaction. For example, if
+    /// `X` was rewritten as `Y`, then rebased as `Z`, these modifications are
+    /// recorded as `{Y: [X], Z: [Y]}`.
+    ///
+    /// Existing commits (including commits imported from Git) aren't tracked
+    /// even if they became visible at this operation.
+    // BTreeMap for ease of deterministic serialization. If the deserialization
+    // cost matters, maybe this can be changed to sorted Vec.
+    pub commit_predecessors: Option<BTreeMap<CommitId, Vec<CommitId>>>,
 }
 
 impl Operation {
@@ -417,6 +406,11 @@ impl Operation {
             view_id: root_view_id,
             parents: vec![],
             metadata,
+            // The root operation is guaranteed to have no new commits. The root
+            // commit could be considered born at the root operation, but there
+            // may be other commits created within the abandoned operations.
+            // They don't have any predecessors records as well.
+            commit_predecessors: Some(BTreeMap::new()),
         }
     }
 }
@@ -508,7 +502,7 @@ mod tests {
     fn test_merge_join_bookmark_views() {
         let remote_ref = |target: &RefTarget| RemoteRef {
             target: target.clone(),
-            state: RemoteRefState::Tracking, // doesn't matter
+            state: RemoteRefState::Tracked, // doesn't matter
         };
         let local_bookmark1_target = RefTarget::normal(CommitId::from_hex("111111"));
         let local_bookmark2_target = RefTarget::normal(CommitId::from_hex("222222"));
@@ -520,24 +514,24 @@ mod tests {
             remote_ref(&RefTarget::normal(CommitId::from_hex("666666")));
 
         let local_bookmarks = btreemap! {
-            "bookmark1".to_owned() => local_bookmark1_target.clone(),
-            "bookmark2".to_owned() => local_bookmark2_target.clone(),
+            "bookmark1".into() => local_bookmark1_target.clone(),
+            "bookmark2".into() => local_bookmark2_target.clone(),
         };
         let remote_views = btreemap! {
-            "git".to_owned() => RemoteView {
+            "git".into() => RemoteView {
                 bookmarks: btreemap! {
-                    "bookmark1".to_owned() => git_bookmark1_remote_ref.clone(),
-                    "bookmark2".to_owned() => git_bookmark2_remote_ref.clone(),
+                    "bookmark1".into() => git_bookmark1_remote_ref.clone(),
+                    "bookmark2".into() => git_bookmark2_remote_ref.clone(),
                 },
             },
-            "remote1".to_owned() => RemoteView {
+            "remote1".into() => RemoteView {
                 bookmarks: btreemap! {
-                    "bookmark1".to_owned() => remote1_bookmark1_remote_ref.clone(),
+                    "bookmark1".into() => remote1_bookmark1_remote_ref.clone(),
                 },
             },
-            "remote2".to_owned() => RemoteView {
+            "remote2".into() => RemoteView {
                 bookmarks: btreemap! {
-                    "bookmark2".to_owned() => remote2_bookmark2_remote_ref.clone(),
+                    "bookmark2".into() => remote2_bookmark2_remote_ref.clone(),
                 },
             },
         };
@@ -545,22 +539,22 @@ mod tests {
             merge_join_bookmark_views(&local_bookmarks, &remote_views).collect_vec(),
             vec![
                 (
-                    "bookmark1",
+                    "bookmark1".as_ref(),
                     BookmarkTarget {
                         local_target: &local_bookmark1_target,
                         remote_refs: vec![
-                            ("git", &git_bookmark1_remote_ref),
-                            ("remote1", &remote1_bookmark1_remote_ref),
+                            ("git".as_ref(), &git_bookmark1_remote_ref),
+                            ("remote1".as_ref(), &remote1_bookmark1_remote_ref),
                         ],
                     },
                 ),
                 (
-                    "bookmark2",
+                    "bookmark2".as_ref(),
                     BookmarkTarget {
                         local_target: &local_bookmark2_target.clone(),
                         remote_refs: vec![
-                            ("git", &git_bookmark2_remote_ref),
-                            ("remote2", &remote2_bookmark2_remote_ref),
+                            ("git".as_ref(), &git_bookmark2_remote_ref),
+                            ("remote2".as_ref(), &remote2_bookmark2_remote_ref),
                         ],
                     },
                 ),
@@ -569,13 +563,13 @@ mod tests {
 
         // Local only
         let local_bookmarks = btreemap! {
-            "bookmark1".to_owned() => local_bookmark1_target.clone(),
+            "bookmark1".into() => local_bookmark1_target.clone(),
         };
         let remote_views = btreemap! {};
         assert_eq!(
             merge_join_bookmark_views(&local_bookmarks, &remote_views).collect_vec(),
             vec![(
-                "bookmark1",
+                "bookmark1".as_ref(),
                 BookmarkTarget {
                     local_target: &local_bookmark1_target,
                     remote_refs: vec![],
@@ -586,19 +580,19 @@ mod tests {
         // Remote only
         let local_bookmarks = btreemap! {};
         let remote_views = btreemap! {
-            "remote1".to_owned() => RemoteView {
+            "remote1".into() => RemoteView {
                 bookmarks: btreemap! {
-                    "bookmark1".to_owned() => remote1_bookmark1_remote_ref.clone(),
+                    "bookmark1".into() => remote1_bookmark1_remote_ref.clone(),
                 },
             },
         };
         assert_eq!(
             merge_join_bookmark_views(&local_bookmarks, &remote_views).collect_vec(),
             vec![(
-                "bookmark1",
+                "bookmark1".as_ref(),
                 BookmarkTarget {
                     local_target: RefTarget::absent_ref(),
-                    remote_refs: vec![("remote1", &remote1_bookmark1_remote_ref)],
+                    remote_refs: vec![("remote1".as_ref(), &remote1_bookmark1_remote_ref)],
                 },
             )],
         );
