@@ -389,7 +389,6 @@ pub struct RewriteRefsOptions {
     pub delete_abandoned_bookmarks: bool,
 }
 
-#[derive(Default)]
 pub struct MoveCommitsStats {
     /// The number of commits in the target set which were rebased.
     pub num_rebased_targets: u32,
@@ -398,89 +397,139 @@ pub struct MoveCommitsStats {
     /// The number of commits for which rebase was skipped, due to the commit
     /// already being in place.
     pub num_skipped_rebases: u32,
-    /// The number of commits which were abandoned.
-    pub num_abandoned: u32,
+    /// The number of commits which were abandoned due to being empty.
+    pub num_abandoned_empty: u32,
+    /// The rebased commits
+    pub rebased_commits: HashMap<CommitId, RebasedCommit>,
+}
+
+/// Target and destination commits to be rebased by [`move_commits()`].
+#[derive(Clone, Debug)]
+pub struct MoveCommitsLocation {
+    pub new_parent_ids: Vec<CommitId>,
+    pub new_child_ids: Vec<CommitId>,
+    pub target: MoveCommitsTarget,
 }
 
 #[derive(Clone, Debug)]
 pub enum MoveCommitsTarget {
     /// The commits to be moved. Commits should be mutable and in reverse
     /// topological order.
-    Commits(Vec<Commit>),
+    Commits(Vec<CommitId>),
     /// The root commits to be moved, along with all their descendants.
-    Roots(Vec<Commit>),
+    Roots(Vec<CommitId>),
 }
 
-/// Moves `target_commits` from their current location to a new location in the
-/// graph.
+#[derive(Clone, Debug)]
+pub struct ComputedMoveCommits {
+    target_commit_ids: IndexSet<CommitId>,
+    descendants: Vec<Commit>,
+    commit_new_parents_map: HashMap<CommitId, Vec<CommitId>>,
+    to_abandon: HashSet<CommitId>,
+}
+
+impl ComputedMoveCommits {
+    fn empty() -> Self {
+        ComputedMoveCommits {
+            target_commit_ids: IndexSet::new(),
+            descendants: vec![],
+            commit_new_parents_map: HashMap::new(),
+            to_abandon: HashSet::new(),
+        }
+    }
+
+    /// Records a set of commits to abandon while rebasing.
+    ///
+    /// Abandoning these commits while rebasing ensures that their descendants
+    /// are still rebased properly. [`MutableRepo::record_abandoned_commit`] is
+    /// similar, but it can lead to issues when abandoning a target commit
+    /// before the rebase.
+    pub fn record_to_abandon(&mut self, commit_ids: impl IntoIterator<Item = CommitId>) {
+        self.to_abandon.extend(commit_ids);
+    }
+
+    pub fn apply(
+        self,
+        mut_repo: &mut MutableRepo,
+        options: &RebaseOptions,
+    ) -> BackendResult<MoveCommitsStats> {
+        apply_move_commits(mut_repo, self, options)
+    }
+}
+
+/// Moves `loc.target` commits from their current location to a new location in
+/// the graph.
 ///
 /// Commits in `target` are rebased onto the new parents given by
-/// `new_parent_ids`, while the `new_children` commits are rebased onto the
+/// `new_parent_ids`, while the `new_child_ids` commits are rebased onto the
 /// heads of the commits in `targets`. This assumes that commits in `target` and
-/// `new_children` can be rewritten, and there will be no cycles in the
+/// `new_child_ids` can be rewritten, and there will be no cycles in the
 /// resulting graph. Commits in `target` should be in reverse topological order.
 pub fn move_commits(
     mut_repo: &mut MutableRepo,
-    new_parent_ids: &[CommitId],
-    new_children: &[Commit],
-    target: &MoveCommitsTarget,
+    loc: &MoveCommitsLocation,
     options: &RebaseOptions,
 ) -> BackendResult<MoveCommitsStats> {
-    let target_commits: Vec<Commit>;
-    let target_commit_ids: HashSet<_>;
+    compute_move_commits(mut_repo, loc)?.apply(mut_repo, options)
+}
+
+pub fn compute_move_commits(
+    repo: &MutableRepo,
+    loc: &MoveCommitsLocation,
+) -> BackendResult<ComputedMoveCommits> {
+    let target_commit_ids: IndexSet<CommitId>;
     let connected_target_commits: Vec<Commit>;
     let connected_target_commits_internal_parents: HashMap<CommitId, Vec<CommitId>>;
     let target_roots: HashSet<CommitId>;
 
-    match target {
-        MoveCommitsTarget::Commits(commits) => {
-            if commits.is_empty() {
-                return Ok(MoveCommitsStats::default());
+    match &loc.target {
+        MoveCommitsTarget::Commits(commit_ids) => {
+            if commit_ids.is_empty() {
+                return Ok(ComputedMoveCommits::empty());
             }
 
-            target_commits = commits.clone();
-            target_commit_ids = target_commits.iter().ids().cloned().collect();
+            target_commit_ids = commit_ids.iter().cloned().collect();
 
-            connected_target_commits =
-                RevsetExpression::commits(target_commits.iter().ids().cloned().collect_vec())
-                    .connected()
-                    .evaluate(mut_repo)
-                    .map_err(|err| err.into_backend_error())?
-                    .iter()
-                    .commits(mut_repo.store())
-                    .try_collect()
-                    .map_err(|err| err.into_backend_error())?;
+            connected_target_commits = RevsetExpression::commits(commit_ids.clone())
+                .connected()
+                .evaluate(repo)
+                .map_err(|err| err.into_backend_error())?
+                .iter()
+                .commits(repo.store())
+                .try_collect()
+                .map_err(|err| err.into_backend_error())?;
             connected_target_commits_internal_parents =
                 compute_internal_parents_within(&target_commit_ids, &connected_target_commits);
 
             target_roots = connected_target_commits_internal_parents
                 .iter()
-                .filter(|(commit_id, parents)| {
+                .filter(|&(commit_id, parents)| {
                     target_commit_ids.contains(commit_id) && parents.is_empty()
                 })
                 .map(|(commit_id, _)| commit_id.clone())
                 .collect();
         }
-        MoveCommitsTarget::Roots(roots) => {
-            if roots.is_empty() {
-                return Ok(MoveCommitsStats::default());
+        MoveCommitsTarget::Roots(root_ids) => {
+            if root_ids.is_empty() {
+                return Ok(ComputedMoveCommits::empty());
             }
 
-            target_commits = RevsetExpression::commits(roots.iter().ids().cloned().collect_vec())
+            target_commit_ids = RevsetExpression::commits(root_ids.clone())
                 .descendants()
-                .evaluate(mut_repo)
+                .evaluate(repo)
                 .map_err(|err| err.into_backend_error())?
                 .iter()
-                .commits(mut_repo.store())
                 .try_collect()
                 .map_err(|err| err.into_backend_error())?;
-            target_commit_ids = target_commits.iter().ids().cloned().collect();
 
-            connected_target_commits = target_commits.iter().cloned().collect_vec();
+            connected_target_commits = target_commit_ids
+                .iter()
+                .map(|id| repo.store().get_commit(id))
+                .try_collect()?;
             // We don't have to compute the internal parents for the connected target set,
             // since the connected target set is the same as the target set.
             connected_target_commits_internal_parents = HashMap::new();
-            target_roots = roots.iter().ids().cloned().collect();
+            target_roots = root_ids.iter().cloned().collect();
         }
     }
 
@@ -488,7 +537,8 @@ pub fn move_commits(
     // parent, then - after the transformation - it should have that commit's
     // ancestors which are not in the target set as parents.
     let mut target_commits_external_parents: HashMap<CommitId, IndexSet<CommitId>> = HashMap::new();
-    for commit in target_commits.iter().rev() {
+    for id in target_commit_ids.iter().rev() {
+        let commit = repo.store().get_commit(id)?;
         let mut new_parents = IndexSet::new();
         for old_parent in commit.parent_ids() {
             if let Some(parents) = target_commits_external_parents.get(old_parent) {
@@ -503,7 +553,8 @@ pub fn move_commits(
     // If the new parents include a commit in the target set, replace it with the
     // commit's ancestors which are outside the set.
     // e.g. `jj rebase -r A --before A`
-    let new_parent_ids: Vec<_> = new_parent_ids
+    let new_parent_ids: Vec<_> = loc
+        .new_parent_ids
         .iter()
         .flat_map(|parent_id| {
             if let Some(parent_ids) = target_commits_external_parents.get(parent_id) {
@@ -517,9 +568,10 @@ pub fn move_commits(
     // If the new children include a commit in the target set, replace it with the
     // commit's descendants which are outside the set.
     // e.g. `jj rebase -r A --after A`
-    let new_children: Vec<_> = if new_children
+    let new_children: Vec<_> = if loc
+        .new_child_ids
         .iter()
-        .any(|child| target_commit_ids.contains(child.id()))
+        .any(|id| target_commit_ids.contains(id))
     {
         let target_commits_descendants: Vec<_> =
             RevsetExpression::commits(target_commit_ids.iter().cloned().collect_vec())
@@ -527,10 +579,10 @@ pub fn move_commits(
                     &RevsetExpression::commits(target_commit_ids.iter().cloned().collect_vec())
                         .children(),
                 )
-                .evaluate(mut_repo)
+                .evaluate(repo)
                 .map_err(|err| err.into_backend_error())?
                 .iter()
-                .commits(mut_repo.store())
+                .commits(repo.store())
                 .try_collect()
                 .map_err(|err| err.into_backend_error())?;
 
@@ -570,18 +622,20 @@ pub fn move_commits(
             }
         }
 
+        let mut new_children = Vec::new();
+        for id in &loc.new_child_ids {
+            if let Some(children) = target_commit_external_descendants.get(id) {
+                new_children.extend(children.iter().cloned());
+            } else {
+                new_children.push(repo.store().get_commit(id)?);
+            }
+        }
         new_children
-            .iter()
-            .flat_map(|child| {
-                if let Some(children) = target_commit_external_descendants.get(child.id()) {
-                    children.iter().cloned().collect_vec()
-                } else {
-                    vec![child.clone()]
-                }
-            })
-            .collect()
     } else {
-        new_children.to_vec()
+        loc.new_child_ids
+            .iter()
+            .map(|id| repo.store().get_commit(id))
+            .try_collect()?
     };
 
     // Compute the parents of the new children, which will include the heads of the
@@ -636,8 +690,8 @@ pub fn move_commits(
     let mut roots = target_roots.iter().cloned().collect_vec();
     roots.extend(new_children.iter().ids().cloned());
 
-    let descendants = mut_repo.find_descendants_for_rebase(roots.clone())?;
-    let commit_new_parents_map: HashMap<_, _> = descendants
+    let descendants = repo.find_descendants_for_rebase(roots.clone())?;
+    let commit_new_parents_map = descendants
         .iter()
         .map(|commit| {
             let commit_id = commit.id();
@@ -669,7 +723,7 @@ pub fn move_commits(
                                 connected_target_commits_internal_parents.get(parent_id) {
                             new_parents.extend(parents.iter().cloned());
                         } else if !new_children.iter().any(|new_child| {
-                                mut_repo.index().is_ancestor(new_child.id(), parent_id) }) {
+                                repo.index().is_ancestor(new_child.id(), parent_id) }) {
                             new_parents.push(parent_id.clone());
                         }
                     }
@@ -699,10 +753,23 @@ pub fn move_commits(
         })
         .collect();
 
+    Ok(ComputedMoveCommits {
+        target_commit_ids,
+        descendants,
+        commit_new_parents_map,
+        to_abandon: HashSet::new(),
+    })
+}
+
+fn apply_move_commits(
+    mut_repo: &mut MutableRepo,
+    commits: ComputedMoveCommits,
+    options: &RebaseOptions,
+) -> BackendResult<MoveCommitsStats> {
     let mut num_rebased_targets = 0;
     let mut num_rebased_descendants = 0;
     let mut num_skipped_rebases = 0;
-    let mut num_abandoned = 0;
+    let mut num_abandoned_empty = 0;
 
     // Always keep empty commits when rebasing descendants.
     let rebase_descendant_options = &RebaseOptions {
@@ -711,14 +778,17 @@ pub fn move_commits(
         simplify_ancestor_merge: options.simplify_ancestor_merge,
     };
 
+    let mut rebased_commits: HashMap<CommitId, RebasedCommit> = HashMap::new();
     mut_repo.transform_commits(
-        descendants,
-        &commit_new_parents_map,
+        commits.descendants,
+        &commits.commit_new_parents_map,
         &options.rewrite_refs,
         |rewriter| {
-            let old_commit_id = rewriter.old_commit().id();
-            if rewriter.parents_changed() {
-                let is_target_commit = target_commit_ids.contains(old_commit_id);
+            let old_commit_id = rewriter.old_commit().id().clone();
+            if commits.to_abandon.contains(&old_commit_id) {
+                rewriter.abandon();
+            } else if rewriter.parents_changed() {
+                let is_target_commit = commits.target_commit_ids.contains(&old_commit_id);
                 let rebased_commit = rebase_commit_with_options(
                     rewriter,
                     if is_target_commit {
@@ -728,12 +798,13 @@ pub fn move_commits(
                     },
                 )?;
                 if let RebasedCommit::Abandoned { .. } = rebased_commit {
-                    num_abandoned += 1;
+                    num_abandoned_empty += 1;
                 } else if is_target_commit {
                     num_rebased_targets += 1;
                 } else {
                     num_rebased_descendants += 1;
                 }
+                rebased_commits.insert(old_commit_id, rebased_commit);
             } else {
                 num_skipped_rebases += 1;
             }
@@ -746,7 +817,8 @@ pub fn move_commits(
         num_rebased_targets,
         num_rebased_descendants,
         num_skipped_rebases,
-        num_abandoned,
+        num_abandoned_empty,
+        rebased_commits,
     })
 }
 
@@ -759,37 +831,38 @@ pub struct DuplicateCommitsStats {
     pub num_rebased: u32,
 }
 
-/// Duplicates the given `target_commits` onto a new location in the graph.
+/// Duplicates the given `target_commit_ids` onto a new location in the graph.
 ///
-/// The roots of `target_commits` are duplicated on top of the new
-/// `parent_commit_ids`, whilst other commits in `target_commits` are duplicated
-/// on top of the newly duplicated commits in the target set. If
+/// The roots of `target_commit_ids` are duplicated on top of the new
+/// `parent_commit_ids`, whilst other commits in `target_commit_ids` are
+/// duplicated on top of the newly duplicated commits in the target set. If
 /// `children_commit_ids` is not empty, the `children_commit_ids` will be
 /// rebased onto the heads of the duplicated target commits.
 ///
 /// If `target_descriptions` is not empty, it will be consulted to retrieve the
-/// new descriptions of the target commits, falling back to the original if
-/// the map does not contain an entry for a given commit.
+/// new descriptions of the target commits, falling back to the original if the
+/// map does not contain an entry for a given commit.
 ///
 /// This assumes that commits in `children_commit_ids` can be rewritten. There
 /// should also be no cycles in the resulting graph, i.e. `children_commit_ids`
-/// should not be ancestors of `parent_commit_ids`. Commits in `target_commits`
-/// should be in reverse topological order (children before parents).
+/// should not be ancestors of `parent_commit_ids`. Commits in
+/// `target_commit_ids` should be in reverse topological order (children before
+/// parents).
 pub fn duplicate_commits(
     mut_repo: &mut MutableRepo,
-    target_commits: &[CommitId],
+    target_commit_ids: &[CommitId],
     target_descriptions: &HashMap<CommitId, String>,
     parent_commit_ids: &[CommitId],
     children_commit_ids: &[CommitId],
 ) -> BackendResult<DuplicateCommitsStats> {
-    if target_commits.is_empty() {
+    if target_commit_ids.is_empty() {
         return Ok(DuplicateCommitsStats::default());
     }
 
     let mut duplicated_old_to_new: IndexMap<CommitId, Commit> = IndexMap::new();
     let mut num_rebased = 0;
 
-    let target_commit_ids: HashSet<_> = target_commits.iter().cloned().collect();
+    let target_commit_ids: IndexSet<_> = target_commit_ids.iter().cloned().collect();
 
     let connected_target_commits: Vec<_> =
         RevsetExpression::commits(target_commit_ids.iter().cloned().collect_vec())
@@ -831,7 +904,7 @@ pub fn duplicate_commits(
 
     // Topological order ensures that any parents of the original commit are
     // either not in `target_commits` or were already duplicated.
-    for original_commit_id in target_commits.iter().rev() {
+    for original_commit_id in target_commit_ids.iter().rev() {
         let original_commit = mut_repo.store().get_commit(original_commit_id)?;
         let new_parent_ids = if target_root_ids.contains(original_commit_id) {
             parent_commit_ids.to_vec()
@@ -959,7 +1032,7 @@ pub fn duplicate_commits_onto_parents(
 /// traversing an edge if the parent is in the target set. `graph_commits`
 /// should be in reverse topological order.
 fn compute_internal_parents_within(
-    target_commit_ids: &HashSet<CommitId>,
+    target_commit_ids: &IndexSet<CommitId>,
     graph_commits: &[Commit],
 ) -> HashMap<CommitId, Vec<CommitId>> {
     let mut internal_parents: HashMap<CommitId, Vec<CommitId>> = HashMap::new();
@@ -985,7 +1058,7 @@ fn compute_internal_parents_within(
 /// `connected_target_commits` should be in reverse topological order (children
 /// before parents).
 fn compute_commits_heads(
-    target_commit_ids: &HashSet<CommitId>,
+    target_commit_ids: &IndexSet<CommitId>,
     connected_target_commits: &[Commit],
 ) -> Vec<CommitId> {
     let mut target_head_ids: HashSet<CommitId> = HashSet::new();
@@ -1129,3 +1202,82 @@ pub fn squash_commits<'repo>(
         abandoned_commits,
     }))
 }
+
+/// Find divergent commits from the target that are already present with
+/// identical contents in the destination. These commits should be able to be
+/// safely abandoned.
+pub fn find_duplicate_divergent_commits(
+    repo: &dyn Repo,
+    new_parent_ids: &[CommitId],
+    target: &MoveCommitsTarget,
+) -> BackendResult<Vec<Commit>> {
+    let target_commits: Vec<Commit> = match target {
+        MoveCommitsTarget::Commits(commit_ids) => commit_ids
+            .iter()
+            .map(|commit_id| repo.store().get_commit(commit_id))
+            .try_collect()?,
+        MoveCommitsTarget::Roots(root_ids) => RevsetExpression::commits(root_ids.clone())
+            .descendants()
+            .evaluate(repo)
+            .map_err(|err| err.into_backend_error())?
+            .iter()
+            .commits(repo.store())
+            .try_collect()
+            .map_err(|err| err.into_backend_error())?,
+    };
+    let target_commit_ids: HashSet<&CommitId> = target_commits.iter().map(Commit::id).collect();
+
+    // For each divergent change being rebased, we want to find all of the other
+    // commits with the same change ID which are not being rebased.
+    let divergent_changes: Vec<(&Commit, Vec<CommitId>)> = target_commits
+        .iter()
+        .map(|target_commit| {
+            let mut ancestor_candidates = repo
+                .resolve_change_id(target_commit.change_id())
+                .unwrap_or_default();
+            ancestor_candidates.retain(|commit_id| !target_commit_ids.contains(commit_id));
+            (target_commit, ancestor_candidates)
+        })
+        .filter(|(_, candidates)| !candidates.is_empty())
+        .collect();
+    if divergent_changes.is_empty() {
+        return Ok(Vec::new());
+    }
+
+    let target_root_ids = match target {
+        MoveCommitsTarget::Commits(commit_ids) => commit_ids,
+        MoveCommitsTarget::Roots(root_ids) => root_ids,
+    };
+
+    // We only care about divergent changes which are new ancestors of the rebased
+    // commits, not ones which were already ancestors of the rebased commits.
+    let is_new_ancestor = RevsetExpression::commits(target_root_ids.clone())
+        .range(&RevsetExpression::commits(new_parent_ids.to_owned()))
+        .evaluate(repo)
+        .map_err(|err| err.into_backend_error())?
+        .containing_fn();
+
+    let mut duplicate_divergent = Vec::new();
+    // Checking every pair of commits between these two sets could be expensive if
+    // there are several commits with the same change ID. However, it should be
+    // uncommon to have more than a couple commits with the same change ID being
+    // rebased at the same time, so it should be good enough in practice.
+    for (target_commit, ancestor_candidates) in divergent_changes {
+        for ancestor_candidate_id in ancestor_candidates {
+            if !is_new_ancestor(&ancestor_candidate_id).map_err(|err| err.into_backend_error())? {
+                continue;
+            }
+
+            let ancestor_candidate = repo.store().get_commit(&ancestor_candidate_id)?;
+            let new_tree =
+                rebase_to_dest_parent(repo, &[target_commit.clone()], &ancestor_candidate)?;
+            // Check whether the rebased commit would have the same tree as the existing
+            // commit if they had the same parents. If so, we can skip this rebased commit.
+            if new_tree.id() == *ancestor_candidate.tree_id() {
+                duplicate_divergent.push(target_commit.clone());
+                break;
+            }
+        }
+    }
+    Ok(duplicate_divergent)
+}
