@@ -15,12 +15,13 @@
 //! Provides a backend for testing ACLs
 
 use std::any::Any;
-use std::io::Read;
 use std::path::Path;
+use std::pin::Pin;
 use std::time::SystemTime;
 
 use async_trait::async_trait;
 use futures::stream::BoxStream;
+use tokio::io::AsyncRead;
 
 use crate::backend::Backend;
 use crate::backend::BackendError;
@@ -31,6 +32,8 @@ use crate::backend::Commit;
 use crate::backend::CommitId;
 use crate::backend::Conflict;
 use crate::backend::ConflictId;
+use crate::backend::CopyHistory;
+use crate::backend::CopyId;
 use crate::backend::CopyRecord;
 use crate::backend::FileId;
 use crate::backend::SigningFn;
@@ -39,7 +42,7 @@ use crate::backend::Tree;
 use crate::backend::TreeId;
 use crate::git_backend::GitBackend;
 use crate::index::Index;
-use crate::object_id::ObjectId;
+use crate::object_id::ObjectId as _;
 use crate::repo_path::RepoPath;
 use crate::repo_path::RepoPathBuf;
 use crate::settings::UserSettings;
@@ -118,7 +121,11 @@ impl Backend for SecretBackend {
         1
     }
 
-    async fn read_file(&self, path: &RepoPath, id: &FileId) -> BackendResult<Box<dyn Read>> {
+    async fn read_file(
+        &self,
+        path: &RepoPath,
+        id: &FileId,
+    ) -> BackendResult<Pin<Box<dyn AsyncRead>>> {
         if path.as_internal_file_string().contains("secret")
             || SECRET_CONTENTS_HEX.contains(&id.hex().as_ref())
         {
@@ -134,7 +141,7 @@ impl Backend for SecretBackend {
     async fn write_file(
         &self,
         path: &RepoPath,
-        contents: &mut (dyn Read + Send),
+        contents: &mut (dyn AsyncRead + Send + Unpin),
     ) -> BackendResult<FileId> {
         self.inner.write_file(path, contents).await
     }
@@ -156,6 +163,24 @@ impl Backend for SecretBackend {
         self.inner.write_symlink(path, target).await
     }
 
+    async fn read_copy(&self, _id: &CopyId) -> BackendResult<CopyHistory> {
+        Err(BackendError::Unsupported(
+            "The secret backend doesn't support copies".to_string(),
+        ))
+    }
+
+    async fn write_copy(&self, _contents: &CopyHistory) -> BackendResult<CopyId> {
+        Err(BackendError::Unsupported(
+            "The secret backend doesn't support copies".to_string(),
+        ))
+    }
+
+    async fn get_related_copies(&self, _copy_id: &CopyId) -> BackendResult<Vec<CopyHistory>> {
+        Err(BackendError::Unsupported(
+            "The secret backend doesn't support copies".to_string(),
+        ))
+    }
+
     async fn read_tree(&self, path: &RepoPath, id: &TreeId) -> BackendResult<Tree> {
         self.inner.read_tree(path, id).await
     }
