@@ -152,7 +152,9 @@ impl OpStore for SimpleOpStore {
         }
 
         let path = self.views_dir().join(id.hex());
-        let buf = fs::read(path).map_err(|err| io_to_read_error(err, id))?;
+        let buf = fs::read(&path)
+            .context(&path)
+            .map_err(|err| io_to_read_error(err, id))?;
 
         let proto = crate::protos::op_store::View::decode(&*buf)
             .map_err(|err| to_read_error(err.into(), id))?;
@@ -161,18 +163,22 @@ impl OpStore for SimpleOpStore {
 
     fn write_view(&self, view: &View) -> OpStoreResult<ViewId> {
         let dir = self.views_dir();
-        let temp_file =
-            NamedTempFile::new_in(&dir).map_err(|err| io_to_write_error(err, "view"))?;
+        let temp_file = NamedTempFile::new_in(&dir)
+            .context(&dir)
+            .map_err(|err| io_to_write_error(err, "view"))?;
 
         let proto = view_to_proto(view);
         temp_file
             .as_file()
             .write_all(&proto.encode_to_vec())
+            .context(temp_file.path())
             .map_err(|err| io_to_write_error(err, "view"))?;
 
         let id = ViewId::new(blake2b_hash(view).to_vec());
 
-        persist_content_addressed_temp_file(temp_file, dir.join(id.hex()))
+        let new_path = dir.join(id.hex());
+        persist_content_addressed_temp_file(temp_file, &new_path)
+            .context(&new_path)
             .map_err(|err| io_to_write_error(err, "view"))?;
         Ok(id)
     }
@@ -183,7 +189,9 @@ impl OpStore for SimpleOpStore {
         }
 
         let path = self.operations_dir().join(id.hex());
-        let buf = fs::read(path).map_err(|err| io_to_read_error(err, id))?;
+        let buf = fs::read(&path)
+            .context(&path)
+            .map_err(|err| io_to_read_error(err, id))?;
 
         let proto = crate::protos::op_store::Operation::decode(&*buf)
             .map_err(|err| to_read_error(err.into(), id))?;
@@ -200,18 +208,22 @@ impl OpStore for SimpleOpStore {
     fn write_operation(&self, operation: &Operation) -> OpStoreResult<OperationId> {
         assert!(!operation.parents.is_empty());
         let dir = self.operations_dir();
-        let temp_file =
-            NamedTempFile::new_in(&dir).map_err(|err| io_to_write_error(err, "operation"))?;
+        let temp_file = NamedTempFile::new_in(&dir)
+            .context(&dir)
+            .map_err(|err| io_to_write_error(err, "operation"))?;
 
         let proto = operation_to_proto(operation);
         temp_file
             .as_file()
             .write_all(&proto.encode_to_vec())
+            .context(temp_file.path())
             .map_err(|err| io_to_write_error(err, "operation"))?;
 
         let id = OperationId::new(blake2b_hash(operation).to_vec());
 
-        persist_content_addressed_temp_file(temp_file, dir.join(id.hex()))
+        let new_path = dir.join(id.hex());
+        persist_content_addressed_temp_file(temp_file, &new_path)
+            .context(&new_path)
             .map_err(|err| io_to_write_error(err, "operation"))?;
         Ok(id)
     }
@@ -345,8 +357,8 @@ impl OpStore for SimpleOpStore {
     }
 }
 
-fn io_to_read_error(err: std::io::Error, id: &impl ObjectId) -> OpStoreError {
-    if err.kind() == ErrorKind::NotFound {
+fn io_to_read_error(err: PathError, id: &impl ObjectId) -> OpStoreError {
+    if err.error.kind() == ErrorKind::NotFound {
         OpStoreError::ObjectNotFound {
             object_type: id.object_type(),
             hash: id.hex(),
@@ -368,7 +380,7 @@ fn to_read_error(
     }
 }
 
-fn io_to_write_error(err: std::io::Error, object_type: &'static str) -> OpStoreError {
+fn io_to_write_error(err: PathError, object_type: &'static str) -> OpStoreError {
     OpStoreError::WriteObject {
         object_type,
         source: Box::new(err),
@@ -447,11 +459,47 @@ fn operation_metadata_from_proto(
     }
 }
 
+fn commit_predecessors_map_to_proto(
+    map: &BTreeMap<CommitId, Vec<CommitId>>,
+) -> Vec<crate::protos::op_store::CommitPredecessors> {
+    map.iter()
+        .map(
+            |(commit_id, predecessor_ids)| crate::protos::op_store::CommitPredecessors {
+                commit_id: commit_id.to_bytes(),
+                predecessor_ids: predecessor_ids.iter().map(|id| id.to_bytes()).collect(),
+            },
+        )
+        .collect()
+}
+
+fn commit_predecessors_map_from_proto(
+    proto: Vec<crate::protos::op_store::CommitPredecessors>,
+) -> BTreeMap<CommitId, Vec<CommitId>> {
+    proto
+        .into_iter()
+        .map(|entry| {
+            let commit_id = CommitId::new(entry.commit_id);
+            let predecessor_ids = entry
+                .predecessor_ids
+                .into_iter()
+                .map(CommitId::new)
+                .collect();
+            (commit_id, predecessor_ids)
+        })
+        .collect()
+}
+
 fn operation_to_proto(operation: &Operation) -> crate::protos::op_store::Operation {
+    let (commit_predecessors, stores_commit_predecessors) = match &operation.commit_predecessors {
+        Some(map) => (commit_predecessors_map_to_proto(map), true),
+        None => (vec![], false),
+    };
     let mut proto = crate::protos::op_store::Operation {
         view_id: operation.view_id.as_bytes().to_vec(),
+        parents: Default::default(),
         metadata: Some(operation_metadata_to_proto(&operation.metadata)),
-        ..Default::default()
+        commit_predecessors,
+        stores_commit_predecessors,
     };
     for parent in &operation.parents {
         proto.parents.push(parent.to_bytes());
@@ -469,10 +517,14 @@ fn operation_from_proto(
         .try_collect()?;
     let view_id = view_id_from_proto(proto.view_id)?;
     let metadata = operation_metadata_from_proto(proto.metadata.unwrap_or_default());
+    let commit_predecessors = proto
+        .stores_commit_predecessors
+        .then(|| commit_predecessors_map_from_proto(proto.commit_predecessors));
     Ok(Operation {
         view_id,
         parents,
         metadata,
+        commit_predecessors,
     })
 }
 
@@ -820,6 +872,13 @@ mod tests {
                     "key2".to_string() => "value2".to_string(),
                 },
             },
+            commit_predecessors: Some(btreemap! {
+                CommitId::from_hex("111111") => vec![],
+                CommitId::from_hex("222222") => vec![
+                    CommitId::from_hex("333333"),
+                    CommitId::from_hex("444444"),
+                ],
+            }),
         }
     }
 
@@ -837,7 +896,7 @@ mod tests {
         // Test exact output so we detect regressions in compatibility
         assert_snapshot!(
             OperationId::new(blake2b_hash(&create_operation()).to_vec()).hex(),
-            @"a721c8bfe6d30b4279437722417743c2c5d9efe731942663e3e7d37320e0ab6b49a7c1452d101cc427ceb8927a4cab03d49dabe73c0677bb9edf5c8b2aa83585"
+            @"b544c80b5ededdd64d0f10468fa636a06b83c45d94dd9bdac95319f7fe11fee536506c5c110681dee6233e69db7647683e732939a3ec88e867250efd765fea18"
         );
     }
 
