@@ -18,6 +18,8 @@ use std::borrow::Borrow;
 use std::borrow::Cow;
 use std::collections::BTreeMap;
 use std::fmt;
+use std::fmt::Debug;
+use std::ops::Deref;
 
 use either::Either;
 use thiserror::Error;
@@ -36,8 +38,26 @@ pub enum StringPatternParseError {
     Regex(regex::Error),
 }
 
-fn parse_glob(src: &str) -> Result<glob::Pattern, StringPatternParseError> {
-    glob::Pattern::new(src).map_err(StringPatternParseError::GlobPattern)
+/// A wrapper for [`glob::Pattern`] with a more concise Debug impl
+#[derive(Clone)]
+pub struct GlobPattern(pub glob::Pattern);
+
+impl GlobPattern {
+    fn as_str(&self) -> &str {
+        self.0.as_str()
+    }
+}
+
+impl Debug for GlobPattern {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_tuple("GlobPattern").field(&self.as_str()).finish()
+    }
+}
+
+fn parse_glob(src: &str) -> Result<GlobPattern, StringPatternParseError> {
+    glob::Pattern::new(src)
+        .map(GlobPattern)
+        .map_err(StringPatternParseError::GlobPattern)
 }
 
 /// Pattern to be tested against string property like commit description or
@@ -53,12 +73,13 @@ pub enum StringPattern {
     /// Matches strings that case‐insensitively contain a substring.
     SubstringI(String),
     /// Matches with a Unix‐style shell wildcard pattern.
-    Glob(glob::Pattern),
+    Glob(GlobPattern),
     /// Matches with a case‐insensitive Unix‐style shell wildcard pattern.
-    GlobI(glob::Pattern),
+    GlobI(GlobPattern),
     /// Matches substrings with a regular expression.
     Regex(regex::Regex),
-    // TODO: Should we add RegexI and "regex-i" prefix?
+    /// Matches substrings with a case‐insensitive regular expression.
+    RegexI(regex::Regex),
 }
 
 impl StringPattern {
@@ -120,6 +141,15 @@ impl StringPattern {
         Ok(StringPattern::Regex(pattern))
     }
 
+    /// Parses the given string as a case-insensitive regular expression.
+    pub fn regex_i(src: &str) -> Result<Self, StringPatternParseError> {
+        let pattern = regex::RegexBuilder::new(src)
+            .case_insensitive(true)
+            .build()
+            .map_err(StringPatternParseError::Regex)?;
+        Ok(StringPattern::RegexI(pattern))
+    }
+
     /// Parses the given string as a pattern of the specified `kind`.
     pub fn from_str_kind(src: &str, kind: &str) -> Result<Self, StringPatternParseError> {
         match kind {
@@ -130,6 +160,7 @@ impl StringPattern {
             "glob" => StringPattern::glob(src),
             "glob-i" => StringPattern::glob_i(src),
             "regex" => StringPattern::regex(src),
+            "regex-i" => StringPattern::regex_i(src),
             _ => Err(StringPatternParseError::InvalidKind(kind.to_owned())),
         }
     }
@@ -162,6 +193,7 @@ impl StringPattern {
             StringPattern::Glob(pattern) => pattern.as_str(),
             StringPattern::GlobI(pattern) => pattern.as_str(),
             StringPattern::Regex(pattern) => pattern.as_str(),
+            StringPattern::RegexI(pattern) => pattern.as_str(),
         }
     }
 
@@ -185,6 +217,7 @@ impl StringPattern {
             StringPattern::SubstringI(_) => None,
             StringPattern::GlobI(_) => None,
             StringPattern::Regex(_) => None,
+            StringPattern::RegexI(_) => None,
         }
     }
 
@@ -193,10 +226,12 @@ impl StringPattern {
     /// When matching against a case‐insensitive pattern, only ASCII case
     /// differences are currently folded. This may change in the future.
     pub fn matches(&self, haystack: &str) -> bool {
-        // TODO: Unicode case folding is complicated and can be locale‐specific. The
-        // `glob` crate and Gitoxide only deal with ASCII case folding, so we do
-        // the same here; a more elaborate case folding system will require
-        // making sure those behave in a matching manner where relevant.
+        // TODO: Unicode case folding is complicated and can be
+        // locale‐specific. The `glob` crate and Gitoxide only deal with ASCII
+        // case folding, so we do the same here; a more elaborate case folding
+        // system will require making sure those behave in a matching manner
+        // where relevant. That said, regex patterns are unicode-aware by
+        // default, so we already have some inconsistencies.
         //
         // Care will need to be taken regarding normalization and the choice of an
         // appropriate case‐insensitive comparison scheme (`toNFKC_Casefold`?) to ensure
@@ -215,27 +250,69 @@ impl StringPattern {
             StringPattern::SubstringI(needle) => haystack
                 .to_ascii_lowercase()
                 .contains(&needle.to_ascii_lowercase()),
-            StringPattern::Glob(pattern) => pattern.matches(haystack),
-            StringPattern::GlobI(pattern) => pattern.matches_with(
+            StringPattern::Glob(pattern) => pattern.0.matches(haystack),
+            StringPattern::GlobI(pattern) => pattern.0.matches_with(
                 haystack,
                 glob::MatchOptions {
                     case_sensitive: false,
                     ..glob::MatchOptions::new()
                 },
             ),
+            // Regex and RegexI are identical here, but callers might want to
+            // translate these to backend-specific query differently.
             StringPattern::Regex(pattern) => pattern.is_match(haystack),
+            StringPattern::RegexI(pattern) => pattern.is_match(haystack),
         }
     }
 
-    /// Iterates entries of the given `map` whose keys matches this pattern.
-    pub fn filter_btree_map<'a: 'b, 'b, K: Borrow<str> + Ord, V>(
+    /// Iterates entries of the given `map` whose string keys match this
+    /// pattern.
+    pub fn filter_btree_map<'a, 'b, K: Borrow<str> + Ord, V>(
         &'b self,
         map: &'a BTreeMap<K, V>,
-    ) -> impl Iterator<Item = (&'a K, &'a V)> + 'b {
+    ) -> impl Iterator<Item = (&'a K, &'a V)> + use<'a, 'b, K, V> {
+        self.filter_btree_map_with(map, |key| key, |key| key)
+    }
+
+    /// Iterates entries of the given `map` whose string-like keys match this
+    /// pattern.
+    ///
+    /// The borrowed key type is constrained by the `Deref::Target`. It must be
+    /// convertible to/from `str`.
+    pub fn filter_btree_map_as_deref<'a, 'b, K, V>(
+        &'b self,
+        map: &'a BTreeMap<K, V>,
+    ) -> impl Iterator<Item = (&'a K, &'a V)> + use<'a, 'b, K, V>
+    where
+        K: Borrow<K::Target> + Deref + Ord,
+        K::Target: AsRef<str> + Ord,
+        str: AsRef<K::Target>,
+    {
+        self.filter_btree_map_with(map, AsRef::as_ref, AsRef::as_ref)
+    }
+
+    fn filter_btree_map_with<'a, 'b, K, Q, V, FromKey, ToKey>(
+        &'b self,
+        map: &'a BTreeMap<K, V>,
+        from_key: FromKey,
+        to_key: ToKey,
+        // TODO: Q, FromKey, and ToKey don't have to be captured, but
+        // "currently, all type parameters are required to be mentioned in the
+        // precise captures list" as of rustc 1.85.0.
+    ) -> impl Iterator<Item = (&'a K, &'a V)> + use<'a, 'b, K, Q, V, FromKey, ToKey>
+    where
+        K: Borrow<Q> + Ord,
+        Q: Ord + ?Sized,
+        FromKey: Fn(&Q) -> &str,
+        ToKey: Fn(&str) -> &Q,
+    {
         if let Some(key) = self.as_exact() {
-            Either::Left(map.get_key_value(key).into_iter())
+            Either::Left(map.get_key_value(to_key(key)).into_iter())
         } else {
-            Either::Right(map.iter().filter(|&(key, _)| self.matches(key.borrow())))
+            Either::Right(
+                map.iter()
+                    .filter(move |&(key, _)| self.matches(from_key(key.borrow()))),
+            )
         }
     }
 }
@@ -315,6 +392,14 @@ mod tests {
             StringPattern::from_str_kind("foo", "regex"),
             Ok(StringPattern::Regex(p)) if p.as_str() == "foo"
         );
+        assert_matches!(
+            StringPattern::parse("regex-i:foo"),
+            Ok(StringPattern::RegexI(p)) if p.as_str() == "foo"
+        );
+        assert_matches!(
+            StringPattern::from_str_kind("foo", "regex-i"),
+            Ok(StringPattern::RegexI(p)) if p.as_str() == "foo"
+        );
 
         // Parse a pattern that contains a : itself.
         assert_matches!(
