@@ -17,13 +17,16 @@
 use std::sync::Arc;
 
 use itertools::Itertools as _;
+use thiserror::Error;
 
 use crate::backend::Timestamp;
 use crate::dag_walk;
+use crate::index::IndexWriteError;
 use crate::index::ReadonlyIndex;
 use crate::op_heads_store::OpHeadsStore;
 use crate::op_heads_store::OpHeadsStoreError;
 use crate::op_store;
+use crate::op_store::OpStoreError;
 use crate::op_store::OperationMetadata;
 use crate::operation::Operation;
 use crate::repo::MutableRepo;
@@ -34,6 +37,15 @@ use crate::repo::RepoLoaderError;
 use crate::settings::UserSettings;
 use crate::view::View;
 
+/// Error from attempts to write and publish transaction.
+#[derive(Debug, Error)]
+#[error("Failed to commit new operation")]
+pub enum TransactionCommitError {
+    IndexWrite(#[from] IndexWriteError),
+    OpHeadsStore(#[from] OpHeadsStoreError),
+    OpStore(#[from] OpStoreError),
+}
+
 /// An in-memory representation of a repo and any changes being made to it.
 ///
 /// Within the scope of a transaction, changes to the repository are made
@@ -106,14 +118,17 @@ impl Transaction {
     pub fn commit(
         self,
         description: impl Into<String>,
-    ) -> Result<Arc<ReadonlyRepo>, OpHeadsStoreError> {
-        self.write(description).publish()
+    ) -> Result<Arc<ReadonlyRepo>, TransactionCommitError> {
+        self.write(description)?.publish()
     }
 
     /// Writes the transaction to the operation store, but does not publish it.
     /// That means that a repo can be loaded at the operation, but the
     /// operation will not be seen when loading the repo at head.
-    pub fn write(mut self, description: impl Into<String>) -> UnpublishedOperation {
+    pub fn write(
+        mut self,
+        description: impl Into<String>,
+    ) -> Result<UnpublishedOperation, TransactionCommitError> {
         let mut_repo = self.mut_repo;
         // TODO: Should we instead just do the rebasing here if necessary?
         assert!(
@@ -121,28 +136,26 @@ impl Transaction {
             "BUG: Descendants have not been rebased after the last rewrites."
         );
         let base_repo = mut_repo.base_repo().clone();
-        let (mut_index, view) = mut_repo.consume();
+        let (mut_index, view, predecessors) = mut_repo.consume();
 
-        let view_id = base_repo.op_store().write_view(view.store_view()).unwrap();
-        self.op_metadata.description = description.into();
-        self.op_metadata.end_time = self.end_time.unwrap_or_else(Timestamp::now);
-        let parents = self.parent_ops.iter().map(|op| op.id().clone()).collect();
-        let store_operation = op_store::Operation {
-            view_id,
-            parents,
-            metadata: self.op_metadata,
+        let operation = {
+            let view_id = base_repo.op_store().write_view(view.store_view())?;
+            self.op_metadata.description = description.into();
+            self.op_metadata.end_time = self.end_time.unwrap_or_else(Timestamp::now);
+            let parents = self.parent_ops.iter().map(|op| op.id().clone()).collect();
+            let store_operation = op_store::Operation {
+                view_id,
+                parents,
+                metadata: self.op_metadata,
+                commit_predecessors: Some(predecessors),
+            };
+            let new_op_id = base_repo.op_store().write_operation(&store_operation)?;
+            Operation::new(base_repo.op_store().clone(), new_op_id, store_operation)
         };
-        let new_op_id = base_repo
-            .op_store()
-            .write_operation(&store_operation)
-            .unwrap();
-        let operation = Operation::new(base_repo.op_store().clone(), new_op_id, store_operation);
 
-        let index = base_repo
-            .index_store()
-            .write_index(mut_index, &operation)
-            .unwrap();
-        UnpublishedOperation::new(base_repo.loader(), operation, view, index)
+        let index = base_repo.index_store().write_index(mut_index, &operation)?;
+        let unpublished = UnpublishedOperation::new(base_repo.loader(), operation, view, index);
+        Ok(unpublished)
     }
 }
 
@@ -199,7 +212,7 @@ impl UnpublishedOperation {
         self.repo.operation()
     }
 
-    pub fn publish(self) -> Result<Arc<ReadonlyRepo>, OpHeadsStoreError> {
+    pub fn publish(self) -> Result<Arc<ReadonlyRepo>, TransactionCommitError> {
         let _lock = self.op_heads_store.lock()?;
         self.op_heads_store
             .update_op_heads(self.operation().parent_ids(), self.operation().id())?;
