@@ -19,11 +19,10 @@ use std::fmt::Error;
 use std::fmt::Formatter;
 use std::hash::Hash;
 use std::hash::Hasher;
-use std::io::Read as _;
 use std::sync::Arc;
 
-use futures::future::try_join_all;
 use itertools::Itertools as _;
+use tokio::io::AsyncReadExt as _;
 use tracing::instrument;
 
 use crate::backend;
@@ -31,14 +30,11 @@ use crate::backend::BackendError;
 use crate::backend::BackendResult;
 use crate::backend::ConflictId;
 use crate::backend::TreeEntriesNonRecursiveIterator;
-use crate::backend::TreeEntry;
 use crate::backend::TreeId;
 use crate::backend::TreeValue;
 use crate::files;
-use crate::files::MergeResult;
 use crate::matchers::EverythingMatcher;
 use crate::matchers::Matcher;
-use crate::merge::Merge;
 use crate::merge::MergedTreeVal;
 use crate::object_id::ObjectId as _;
 use crate::repo_path::RepoPath;
@@ -118,10 +114,6 @@ impl Tree {
         self.data.entries()
     }
 
-    pub fn entries(&self) -> TreeEntriesIterator<'static> {
-        TreeEntriesIterator::new(self.clone(), &EverythingMatcher)
-    }
-
     pub fn entries_matching<'matcher>(
         &self,
         matcher: &'matcher dyn Matcher,
@@ -129,10 +121,6 @@ impl Tree {
         TreeEntriesIterator::new(self.clone(), matcher)
     }
 
-    pub fn entry(&self, basename: &RepoPathComponent) -> Option<TreeEntry> {
-        self.data.entry(basename)
-    }
-
     pub fn value(&self, basename: &RepoPathComponent) -> Option<&TreeValue> {
         self.data.value(basename)
     }
@@ -279,15 +267,33 @@ pub async fn try_resolve_file_conflict(
     // merge it. We check early so we don't waste time reading file contents if
     // we can't merge them anyway. At the same time we determine whether the
     // resulting file should be executable.
-    let Some(file_id_conflict) = conflict.maybe_map(|term| match term {
-        Some(TreeValue::File { id, executable: _ }) => Some(id),
-        _ => None,
+    let Ok(file_id_conflict) = conflict.try_map(|term| match term {
+        Some(TreeValue::File {
+            id,
+            executable: _,
+            copy_id: _,
+        }) => Ok(id),
+        _ => Err(()),
     }) else {
         return Ok(None);
     };
-    let Some(executable_conflict) = conflict.maybe_map(|term| match term {
-        Some(TreeValue::File { id: _, executable }) => Some(executable),
-        _ => None,
+    let Ok(executable_conflict) = conflict.try_map(|term| match term {
+        Some(TreeValue::File {
+            id: _,
+            executable,
+            copy_id: _,
+        }) => Ok(executable),
+        _ => Err(()),
+    }) else {
+        return Ok(None);
+    };
+    let Ok(copy_id_conflict) = conflict.try_map(|term| match term {
+        Some(TreeValue::File {
+            id: _,
+            executable: _,
+            copy_id,
+        }) => Ok(copy_id),
+        _ => Err(()),
     }) else {
         return Ok(None);
     };
@@ -295,12 +301,17 @@ pub async fn try_resolve_file_conflict(
         // We're unable to determine whether the result should be executable
         return Ok(None);
     };
+    let Some(&copy_id) = copy_id_conflict.resolve_trivial() else {
+        // We're unable to determine the file's copy ID
+        return Ok(None);
+    };
     if let Some(&resolved_file_id) = file_id_conflict.resolve_trivial() {
         // Don't bother reading the file contents if the conflict can be trivially
         // resolved.
         return Ok(Some(TreeValue::File {
             id: resolved_file_id.clone(),
             executable,
+            copy_id: copy_id.clone(),
         }));
     }
 
@@ -312,27 +323,31 @@ pub async fn try_resolve_file_conflict(
     //    cannot
     let file_id_conflict = file_id_conflict.simplify();
 
-    let content_futures = file_id_conflict.into_iter().map(|file_id| async {
-        let mut content = vec![];
-        let mut reader = store.read_file_async(filename, file_id).await?;
-        reader
-            .read_to_end(&mut content)
-            .map_err(|err| BackendError::ReadObject {
-                object_type: file_id.object_type(),
-                hash: file_id.hex(),
-                source: err.into(),
-            })?;
-        BackendResult::Ok(content)
-    });
-    let contents = Merge::from_vec(try_join_all(content_futures).await?);
-    let merge_result = files::merge(&contents);
-    match merge_result {
-        MergeResult::Resolved(merged_content) => {
-            let id = store
-                .write_file(filename, &mut merged_content.as_slice())
-                .await?;
-            Ok(Some(TreeValue::File { id, executable }))
-        }
-        MergeResult::Conflict(_) => Ok(None),
+    let contents = file_id_conflict
+        .try_map_async(|file_id| async {
+            let mut content = vec![];
+            let mut reader = store.read_file(filename, file_id).await?;
+            reader
+                .read_to_end(&mut content)
+                .await
+                .map_err(|err| BackendError::ReadObject {
+                    object_type: file_id.object_type(),
+                    hash: file_id.hex(),
+                    source: err.into(),
+                })?;
+            BackendResult::Ok(content)
+        })
+        .await?;
+    if let Some(merged_content) = files::try_merge(&contents) {
+        let id = store
+            .write_file(filename, &mut merged_content.as_slice())
+            .await?;
+        Ok(Some(TreeValue::File {
+            id,
+            executable,
+            copy_id: copy_id.clone(),
+        }))
+    } else {
+        Ok(None)
     }
 }
