@@ -12,6 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use assert_matches::assert_matches;
 use futures::StreamExt as _;
 use indoc::indoc;
 use itertools::Itertools as _;
@@ -34,6 +35,7 @@ use test_case::test_case;
 use testutils::assert_rebased_onto;
 use testutils::create_tree;
 use testutils::rebase_descendants_with_options_return_map;
+use testutils::repo_path;
 use testutils::CommitGraphBuilder;
 use testutils::TestRepo;
 use testutils::TestRepoBackend;
@@ -70,8 +72,8 @@ fn test_initial(backend: TestRepoBackend) {
     let repo = &test_repo.repo;
     let store = repo.store();
 
-    let root_file_path = RepoPath::from_internal_string("file");
-    let dir_file_path = RepoPath::from_internal_string("dir/file");
+    let root_file_path = repo_path("file");
+    let dir_file_path = repo_path("dir/file");
     let tree = create_tree(
         repo,
         &[
@@ -112,11 +114,11 @@ fn test_initial(backend: TestRepoBackend) {
     assert_eq!(builder.author(), &author_signature);
     assert_eq!(builder.committer(), &committer_signature);
     let commit = builder.write().unwrap();
-    tx.commit("test").unwrap();
+    let repo = tx.commit("test").unwrap();
 
     let parents: Vec<_> = commit.parents().try_collect().unwrap();
     assert_eq!(parents, vec![store.root_commit()]);
-    assert!(commit.predecessors().next().is_none());
+    assert!(commit.store_commit().predecessors.is_empty());
     assert_eq!(commit.description(), "description");
     assert_eq!(commit.author(), &author_signature);
     assert_eq!(commit.committer(), &committer_signature);
@@ -127,6 +129,10 @@ fn test_initial(backend: TestRepoBackend) {
         ),
         to_owned_path_vec(&[dir_file_path, root_file_path]),
     );
+    assert_matches!(
+        repo.operation().predecessors_for_commit(commit.id()),
+        Some([])
+    );
 }
 
 #[test_case(TestRepoBackend::Simple ; "simple backend")]
@@ -138,8 +144,8 @@ fn test_rewrite(backend: TestRepoBackend) {
     let repo = &test_repo.repo;
     let store = repo.store();
 
-    let root_file_path = RepoPath::from_internal_string("file");
-    let dir_file_path = RepoPath::from_internal_string("dir/file");
+    let root_file_path = repo_path("file");
+    let dir_file_path = repo_path("dir/file");
     let initial_tree = create_tree(
         repo,
         &[
@@ -187,11 +193,13 @@ fn test_rewrite(backend: TestRepoBackend) {
         .write()
         .unwrap();
     tx.repo_mut().rebase_descendants().unwrap();
-    tx.commit("test").unwrap();
+    let repo = tx.commit("test").unwrap();
     let parents: Vec<_> = rewritten_commit.parents().try_collect().unwrap();
     assert_eq!(parents, vec![store.root_commit()]);
-    let predecessors: Vec<_> = rewritten_commit.predecessors().try_collect().unwrap();
-    assert_eq!(predecessors, vec![initial_commit.clone()]);
+    assert_eq!(
+        rewritten_commit.store_commit().predecessors,
+        [initial_commit.id().clone()]
+    );
     assert_eq!(rewritten_commit.author().name, settings.user_name());
     assert_eq!(rewritten_commit.author().email, settings.user_email());
     assert_eq!(
@@ -216,6 +224,15 @@ fn test_rewrite(backend: TestRepoBackend) {
         ),
         to_owned_path_vec(&[dir_file_path]),
     );
+    assert_matches!(
+        repo.operation().predecessors_for_commit(rewritten_commit.id()),
+        Some([id]) if id == initial_commit.id()
+    );
+    assert_matches!(
+        repo.operation()
+            .predecessors_for_commit(initial_commit.id()),
+        None
+    );
 }
 
 // An author field with an empty name/email should get filled in on rewrite
