@@ -14,9 +14,7 @@
 
 use std::collections::HashMap;
 use std::collections::HashSet;
-use std::sync::Arc;
 
-use itertools::Itertools as _;
 use jj_lib::backend::CommitId;
 use jj_lib::backend::FileId;
 use jj_lib::backend::MergedTreeId;
@@ -26,14 +24,14 @@ use jj_lib::fix::FileToFix;
 use jj_lib::fix::FixError;
 use jj_lib::fix::ParallelFileFixer;
 use jj_lib::matchers::EverythingMatcher;
-use jj_lib::merged_tree::MergedTree;
-use jj_lib::repo::ReadonlyRepo;
 use jj_lib::repo::Repo as _;
-use jj_lib::repo_path::RepoPath;
 use jj_lib::store::Store;
 use jj_lib::transaction::Transaction;
 use pollster::FutureExt as _;
 use testutils::create_tree;
+use testutils::create_tree_with;
+use testutils::read_file;
+use testutils::repo_path;
 use testutils::TestRepo;
 use thiserror::Error;
 
@@ -50,7 +48,7 @@ impl TestFileFixer {
 // leaves files unchanged.
 impl FileFixer for TestFileFixer {
     fn fix_files<'a>(
-        &self,
+        &mut self,
         store: &Store,
         files_to_fix: &'a HashSet<FileToFix>,
     ) -> Result<HashMap<&'a FileToFix, FileId>, FixError> {
@@ -76,11 +74,7 @@ fn make_fix_content_error(message: &str) -> FixError {
 // changed to uppercase and the new file id is returned. If the file starts with
 // "error", an error is raised. Otherwise returns None.
 fn fix_file(store: &Store, file_to_fix: &FileToFix) -> Result<Option<FileId>, FixError> {
-    let mut old_content = vec![];
-    let mut read = store
-        .read_file(&file_to_fix.repo_path, &file_to_fix.file_id)
-        .unwrap();
-    read.read_to_end(&mut old_content).unwrap();
+    let old_content = read_file(store, &file_to_fix.repo_path, &file_to_fix.file_id);
 
     if let Some(rest) = old_content.strip_prefix(b"fixme:") {
         let new_content = rest.to_ascii_uppercase();
@@ -96,17 +90,6 @@ fn fix_file(store: &Store, file_to_fix: &FileToFix) -> Result<Option<FileId>, Fi
     }
 }
 
-fn create_tree_helper(
-    repo: &Arc<ReadonlyRepo>,
-    path_and_content: &[(String, String)],
-) -> MergedTree {
-    let content_map = path_and_content
-        .iter()
-        .map(|p| (RepoPath::from_internal_string(&p.0), p.1.as_str()))
-        .collect_vec();
-    create_tree(repo, &content_map)
-}
-
 fn create_commit(tx: &mut Transaction, parents: Vec<CommitId>, tree_id: MergedTreeId) -> CommitId {
     tx.repo_mut()
         .new_commit(parents, tree_id)
@@ -122,7 +105,7 @@ fn test_fix_one_file() {
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction();
-    let path1 = RepoPath::from_internal_string("file1");
+    let path1 = repo_path("file1");
     let tree1 = create_tree(repo, &[(path1, "fixme:content")]);
     let commit_a = create_commit(
         &mut tx,
@@ -131,7 +114,7 @@ fn test_fix_one_file() {
     );
 
     let root_commits = vec![commit_a.clone()];
-    let file_fixer = TestFileFixer::new();
+    let mut file_fixer = TestFileFixer::new();
     let include_unchanged_files = false;
 
     let summary = fix_files(
@@ -139,7 +122,7 @@ fn test_fix_one_file() {
         &EverythingMatcher,
         include_unchanged_files,
         tx.repo_mut(),
-        &file_fixer,
+        &mut file_fixer,
     )
     .unwrap();
 
@@ -162,7 +145,7 @@ fn test_fixer_does_not_change_content() {
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction();
-    let path1 = RepoPath::from_internal_string("file1");
+    let path1 = repo_path("file1");
     let tree1 = create_tree(repo, &[(path1, "content")]);
     let commit_a = create_commit(
         &mut tx,
@@ -171,7 +154,7 @@ fn test_fixer_does_not_change_content() {
     );
 
     let root_commits = vec![commit_a.clone()];
-    let file_fixer = TestFileFixer::new();
+    let mut file_fixer = TestFileFixer::new();
     let include_unchanged_files = false;
 
     let summary = fix_files(
@@ -179,7 +162,7 @@ fn test_fixer_does_not_change_content() {
         &EverythingMatcher,
         include_unchanged_files,
         tx.repo_mut(),
-        &file_fixer,
+        &mut file_fixer,
     )
     .unwrap();
 
@@ -202,7 +185,7 @@ fn test_empty_commit() {
     );
 
     let root_commits = vec![commit_a.clone()];
-    let file_fixer = TestFileFixer::new();
+    let mut file_fixer = TestFileFixer::new();
     let include_unchanged_files = false;
 
     let summary = fix_files(
@@ -210,7 +193,7 @@ fn test_empty_commit() {
         &EverythingMatcher,
         include_unchanged_files,
         tx.repo_mut(),
-        &file_fixer,
+        &mut file_fixer,
     )
     .unwrap();
 
@@ -225,7 +208,7 @@ fn test_fixer_fails() {
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction();
-    let path1 = RepoPath::from_internal_string("file1");
+    let path1 = repo_path("file1");
     let tree1 = create_tree(repo, &[(path1, "error:boo")]);
     let commit_a = create_commit(
         &mut tx,
@@ -234,7 +217,7 @@ fn test_fixer_fails() {
     );
 
     let root_commits = vec![commit_a.clone()];
-    let file_fixer = TestFileFixer::new();
+    let mut file_fixer = TestFileFixer::new();
     let include_unchanged_files = false;
 
     let result = fix_files(
@@ -242,7 +225,7 @@ fn test_fixer_fails() {
         &EverythingMatcher,
         include_unchanged_files,
         tx.repo_mut(),
-        &file_fixer,
+        &mut file_fixer,
     );
 
     let error = result.err().unwrap();
@@ -255,7 +238,7 @@ fn test_unchanged_file_is_not_fixed() {
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction();
-    let path1 = RepoPath::from_internal_string("file1");
+    let path1 = repo_path("file1");
     let tree1 = create_tree(repo, &[(path1, "fixme:content")]);
     let commit_a = create_commit(
         &mut tx,
@@ -267,7 +250,7 @@ fn test_unchanged_file_is_not_fixed() {
     let commit_b = create_commit(&mut tx, vec![commit_a.clone()], tree2.id());
 
     let root_commits = vec![commit_b.clone()];
-    let file_fixer = TestFileFixer::new();
+    let mut file_fixer = TestFileFixer::new();
     let include_unchanged_files = false;
 
     let summary = fix_files(
@@ -275,7 +258,7 @@ fn test_unchanged_file_is_not_fixed() {
         &EverythingMatcher,
         include_unchanged_files,
         tx.repo_mut(),
-        &file_fixer,
+        &mut file_fixer,
     )
     .unwrap();
 
@@ -290,7 +273,7 @@ fn test_unchanged_file_is_fixed() {
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction();
-    let path1 = RepoPath::from_internal_string("file1");
+    let path1 = repo_path("file1");
     let tree1 = create_tree(repo, &[(path1, "fixme:content")]);
     let commit_a = create_commit(
         &mut tx,
@@ -302,14 +285,14 @@ fn test_unchanged_file_is_fixed() {
     let commit_b = create_commit(&mut tx, vec![commit_a.clone()], tree2.id());
 
     let root_commits = vec![commit_b.clone()];
-    let file_fixer = TestFileFixer::new();
+    let mut file_fixer = TestFileFixer::new();
 
     let summary = fix_files(
         root_commits,
         &EverythingMatcher,
         true,
         tx.repo_mut(),
-        &file_fixer,
+        &mut file_fixer,
     )
     .unwrap();
 
@@ -334,7 +317,7 @@ fn test_already_fixed_descendant() {
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction();
-    let path1 = RepoPath::from_internal_string("file1");
+    let path1 = repo_path("file1");
     let tree1 = create_tree(repo, &[(path1, "fixme:content")]);
     let commit_a = create_commit(
         &mut tx,
@@ -346,14 +329,14 @@ fn test_already_fixed_descendant() {
     let commit_b = create_commit(&mut tx, vec![commit_a.clone()], tree2.id());
 
     let root_commits = vec![commit_a.clone()];
-    let file_fixer = TestFileFixer::new();
+    let mut file_fixer = TestFileFixer::new();
 
     let summary = fix_files(
         root_commits,
         &EverythingMatcher,
         true,
         tx.repo_mut(),
-        &file_fixer,
+        &mut file_fixer,
     )
     .unwrap();
 
@@ -381,7 +364,7 @@ fn test_parallel_fixer_basic() {
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction();
-    let path1 = RepoPath::from_internal_string("file1");
+    let path1 = repo_path("file1");
     let tree1 = create_tree(repo, &[(path1, "fixme:content")]);
     let commit_a = create_commit(
         &mut tx,
@@ -391,14 +374,14 @@ fn test_parallel_fixer_basic() {
 
     let root_commits = vec![commit_a.clone()];
     let include_unchanged_files = false;
-    let parallel_fixer = ParallelFileFixer::new(fix_file);
+    let mut parallel_fixer = ParallelFileFixer::new(fix_file);
 
     let summary = fix_files(
         root_commits,
         &EverythingMatcher,
         include_unchanged_files,
         tx.repo_mut(),
-        &parallel_fixer,
+        &mut parallel_fixer,
     )
     .unwrap();
 
@@ -421,13 +404,11 @@ fn test_parallel_fixer_fixes_files() {
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction();
-    let mut path_contents1 = vec![];
-    for i in 0..100 {
-        let path = format!("file{i}");
-        let content = format!("fixme:content{i}");
-        path_contents1.push((path, content));
-    }
-    let tree1 = create_tree_helper(repo, &path_contents1);
+    let tree1 = create_tree_with(repo, |builder| {
+        for i in 0..100 {
+            builder.file(repo_path(&format!("file{i}")), format!("fixme:content{i}"));
+        }
+    });
     let commit_a = create_commit(
         &mut tx,
         vec![repo.store().root_commit_id().clone()],
@@ -436,24 +417,22 @@ fn test_parallel_fixer_fixes_files() {
 
     let root_commits = vec![commit_a.clone()];
     let include_unchanged_files = false;
-    let parallel_fixer = ParallelFileFixer::new(fix_file);
+    let mut parallel_fixer = ParallelFileFixer::new(fix_file);
 
     let summary = fix_files(
         root_commits,
         &EverythingMatcher,
         include_unchanged_files,
         tx.repo_mut(),
-        &parallel_fixer,
+        &mut parallel_fixer,
     )
     .unwrap();
 
-    let mut expected_path_contents = vec![];
-    for i in 0..100 {
-        let path = format!("file{i}");
-        let content = format!("CONTENT{i}");
-        expected_path_contents.push((path, content));
-    }
-    let expected_tree_a = create_tree_helper(repo, &expected_path_contents);
+    let expected_tree_a = create_tree_with(repo, |builder| {
+        for i in 0..100 {
+            builder.file(repo_path(&format!("file{i}")), format!("CONTENT{i}"));
+        }
+    });
 
     assert_eq!(summary.rewrites.len(), 1);
     assert!(summary.rewrites.contains_key(&commit_a));
@@ -473,13 +452,11 @@ fn test_parallel_fixer_does_not_change_content() {
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction();
-    let mut path_contents1 = vec![];
-    for i in 0..100 {
-        let path = format!("file{i}");
-        let content = format!("content{i}");
-        path_contents1.push((path, content));
-    }
-    let tree1 = create_tree_helper(repo, &path_contents1);
+    let tree1 = create_tree_with(repo, |builder| {
+        for i in 0..100 {
+            builder.file(repo_path(&format!("file{i}")), format!("content{i}"));
+        }
+    });
     let commit_a = create_commit(
         &mut tx,
         vec![repo.store().root_commit_id().clone()],
@@ -488,14 +465,14 @@ fn test_parallel_fixer_does_not_change_content() {
 
     let root_commits = vec![commit_a.clone()];
     let include_unchanged_files = false;
-    let parallel_fixer = ParallelFileFixer::new(fix_file);
+    let mut parallel_fixer = ParallelFileFixer::new(fix_file);
 
     let summary = fix_files(
         root_commits,
         &EverythingMatcher,
         include_unchanged_files,
         tx.repo_mut(),
-        &parallel_fixer,
+        &mut parallel_fixer,
     )
     .unwrap();
 
@@ -510,19 +487,19 @@ fn test_parallel_fixer_no_changes_upon_partial_failure() {
     let repo = &test_repo.repo;
 
     let mut tx = repo.start_transaction();
-    let mut path_contents1 = vec![];
-    for i in 0..100 {
-        let path = format!("file{i}");
-        let content = if i == 7 {
-            format!("error:boo{i}")
-        } else if i % 3 == 0 {
-            format!("fixme:content{i}")
-        } else {
-            format!("foobar:{i}")
-        };
-        path_contents1.push((path, content));
-    }
-    let tree1 = create_tree_helper(repo, &path_contents1);
+    let tree1 = create_tree_with(repo, |builder| {
+        for i in 0..100 {
+            let contents = if i == 7 {
+                format!("error:boo{i}")
+            } else if i % 3 == 0 {
+                format!("fixme:content{i}")
+            } else {
+                format!("foobar:{i}")
+            };
+
+            builder.file(repo_path(&format!("file{i}")), &contents);
+        }
+    });
     let commit_a = create_commit(
         &mut tx,
         vec![repo.store().root_commit_id().clone()],
@@ -531,14 +508,14 @@ fn test_parallel_fixer_no_changes_upon_partial_failure() {
 
     let root_commits = vec![commit_a.clone()];
     let include_unchanged_files = false;
-    let parallel_fixer = ParallelFileFixer::new(fix_file);
+    let mut parallel_fixer = ParallelFileFixer::new(fix_file);
 
     let result = fix_files(
         root_commits,
         &EverythingMatcher,
         include_unchanged_files,
         tx.repo_mut(),
-        &parallel_fixer,
+        &mut parallel_fixer,
     );
     let error = result.err().unwrap();
     assert_eq!(error.to_string(), "Forced failure: boo7");
@@ -558,25 +535,25 @@ fn test_fix_multiple_revisions() {
     // |/
     // A
     let mut tx = repo.start_transaction();
-    let path1 = RepoPath::from_internal_string("file1");
+    let path1 = repo_path("file1");
     let tree1 = create_tree(repo, &[(path1, "fixme:xyz")]);
     let commit_a = create_commit(
         &mut tx,
         vec![repo.store().root_commit_id().clone()],
         tree1.id(),
     );
-    let path2 = RepoPath::from_internal_string("file2");
+    let path2 = repo_path("file2");
     let tree2 = create_tree(repo, &[(path2, "content")]);
     let commit_b = create_commit(&mut tx, vec![commit_a.clone()], tree2.id());
-    let path3 = RepoPath::from_internal_string("file3");
+    let path3 = repo_path("file3");
     let tree3 = create_tree(repo, &[(path3, "content")]);
     let _commit_c = create_commit(&mut tx, vec![commit_b.clone()], tree3.id());
-    let path4 = RepoPath::from_internal_string("file4");
+    let path4 = repo_path("file4");
     let tree4 = create_tree(repo, &[(path4, "content")]);
     let _commit_d = create_commit(&mut tx, vec![commit_a.clone()], tree4.id());
 
     let root_commits = vec![commit_a.clone()];
-    let file_fixer = TestFileFixer::new();
+    let mut file_fixer = TestFileFixer::new();
     let include_unchanged_files = false;
 
     let summary = fix_files(
@@ -584,7 +561,7 @@ fn test_fix_multiple_revisions() {
         &EverythingMatcher,
         include_unchanged_files,
         tx.repo_mut(),
-        &file_fixer,
+        &mut file_fixer,
     )
     .unwrap();
 
