@@ -13,6 +13,7 @@
 // limitations under the License.
 
 use itertools::Itertools as _;
+use jj_lib::backend::ChangeId;
 use jj_lib::commit::Commit;
 use jj_lib::matchers::EverythingMatcher;
 use jj_lib::matchers::FilesMatcher;
@@ -27,12 +28,13 @@ use jj_lib::ref_name::RemoteRefSymbol;
 use jj_lib::ref_name::WorkspaceName;
 use jj_lib::ref_name::WorkspaceNameBuf;
 use jj_lib::repo::Repo as _;
-use jj_lib::repo_path::RepoPath;
+use jj_lib::rewrite::find_duplicate_divergent_commits;
 use jj_lib::rewrite::rebase_commit_with_options;
 use jj_lib::rewrite::restore_tree;
 use jj_lib::rewrite::CommitRewriter;
 use jj_lib::rewrite::CommitWithSelection;
 use jj_lib::rewrite::EmptyBehaviour;
+use jj_lib::rewrite::MoveCommitsTarget;
 use jj_lib::rewrite::RebaseOptions;
 use jj_lib::rewrite::RewriteRefsOptions;
 use maplit::hashmap;
@@ -42,7 +44,9 @@ use testutils::assert_abandoned_with_parent;
 use testutils::assert_rebased_onto;
 use testutils::create_random_commit;
 use testutils::create_tree;
+use testutils::create_tree_with;
 use testutils::rebase_descendants_with_options_return_map;
+use testutils::repo_path;
 use testutils::write_random_commit;
 use testutils::CommitGraphBuilder;
 use testutils::TestRepo;
@@ -63,10 +67,10 @@ fn test_restore_tree() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let path1 = RepoPath::from_internal_string("file1");
-    let path2 = RepoPath::from_internal_string("dir1/file2");
-    let path3 = RepoPath::from_internal_string("dir1/file3");
-    let path4 = RepoPath::from_internal_string("dir2/file4");
+    let path1 = repo_path("file1");
+    let path2 = repo_path("dir1/file2");
+    let path3 = repo_path("dir1/file3");
+    let path4 = repo_path("dir2/file4");
     let left = create_tree(repo, &[(path2, "left"), (path3, "left"), (path4, "left")]);
     let right = create_tree(
         repo,
@@ -868,28 +872,28 @@ fn test_rebase_descendants_contents() {
     // |/
     // A
     let mut tx = repo.start_transaction();
-    let path1 = RepoPath::from_internal_string("file1");
+    let path1 = repo_path("file1");
     let tree1 = create_tree(repo, &[(path1, "content")]);
     let commit_a = tx
         .repo_mut()
         .new_commit(vec![repo.store().root_commit_id().clone()], tree1.id())
         .write()
         .unwrap();
-    let path2 = RepoPath::from_internal_string("file2");
+    let path2 = repo_path("file2");
     let tree2 = create_tree(repo, &[(path2, "content")]);
     let commit_b = tx
         .repo_mut()
         .new_commit(vec![commit_a.id().clone()], tree2.id())
         .write()
         .unwrap();
-    let path3 = RepoPath::from_internal_string("file3");
+    let path3 = repo_path("file3");
     let tree3 = create_tree(repo, &[(path3, "content")]);
     let commit_c = tx
         .repo_mut()
         .new_commit(vec![commit_b.id().clone()], tree3.id())
         .write()
         .unwrap();
-    let path4 = RepoPath::from_internal_string("file4");
+    let path4 = repo_path("file4");
     let tree4 = create_tree(repo, &[(path4, "content")]);
     let commit_d = tx
         .repo_mut()
@@ -1658,11 +1662,11 @@ fn test_empty_commit_option(empty_behavior: EmptyBehaviour) {
     let mut tx = repo.start_transaction();
     let mut_repo = tx.repo_mut();
     let create_fixed_tree = |paths: &[&str]| {
-        let content_map = paths
-            .iter()
-            .map(|&p| (RepoPath::from_internal_string(p), p))
-            .collect_vec();
-        create_tree(repo, &content_map)
+        create_tree_with(repo, |builder| {
+            for path in paths {
+                builder.file(repo_path(path), path);
+            }
+        })
     };
 
     // The commit_with_parents function generates non-empty merge commits, so it
@@ -1892,3 +1896,109 @@ fn test_commit_with_selection() {
     assert!(!full_selection.is_empty_selection());
     assert!(full_selection.is_full_selection());
 }
+
+#[test]
+fn test_find_duplicate_divergent_commits() {
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+
+    let mut tx = repo.start_transaction();
+    let store = repo.store();
+    // We want to create the following tree with divergent changes A, B, and C:
+    //     E
+    //     |
+    // A2  C2
+    // |   |
+    // C1  B2
+    // |   |
+    // B1  D
+    // |  /
+    // | /
+    // A1
+
+    let tree_a1 = create_tree(repo, &[(repo_path("file1"), "a\n")]);
+    // Commit b deletes file1 and adds file2
+    let tree_b1 = create_tree(repo, &[(repo_path("file2"), "b\n")]);
+    // Commit c appends to file2
+    let tree_c1 = create_tree(repo, &[(repo_path("file2"), "b\nc\n")]);
+    // Commit a2 re-adds file1, the same as commit a1. Since it already has a1
+    // as an ancestor, it shouldn't be abandoned even though it is divergent.
+    let tree_a2 = create_tree(
+        repo,
+        &[(repo_path("file1"), "a\n"), (repo_path("file2"), "b\nc\n")],
+    );
+    let tree_d = create_tree(
+        repo,
+        &[(repo_path("file1"), "a\n"), (repo_path("file3"), "d\n")],
+    );
+    let tree_b2 = create_tree(
+        repo,
+        &[(repo_path("file2"), "b\n"), (repo_path("file3"), "d\n")],
+    );
+    let tree_c2 = create_tree(
+        repo,
+        &[(repo_path("file2"), "b\nc\n"), (repo_path("file3"), "d\n")],
+    );
+    let tree_e = create_tree(
+        repo,
+        &[
+            (repo_path("file2"), "b\nc\n"),
+            (repo_path("file3"), "d\ne\n"),
+        ],
+    );
+
+    let mut make_commit = |change_id_byte, tree_id, parents| {
+        tx.repo_mut()
+            .new_commit(parents, tree_id)
+            .set_change_id(ChangeId::new(vec![
+                change_id_byte;
+                store.change_id_length()
+            ]))
+            .write()
+            .unwrap()
+    };
+
+    let commit_a1 = make_commit(0xAA, tree_a1.id(), vec![store.root_commit_id().clone()]);
+    let commit_b1 = make_commit(0xBB, tree_b1.id(), vec![commit_a1.id().clone()]);
+    let commit_c1 = make_commit(0xCC, tree_c1.id(), vec![commit_b1.id().clone()]);
+    let commit_a2 = make_commit(0xAA, tree_a2.id(), vec![commit_c1.id().clone()]);
+    let commit_d = make_commit(0xDD, tree_d.id(), vec![commit_a1.id().clone()]);
+    let commit_b2 = make_commit(0xBB, tree_b2.id(), vec![commit_d.id().clone()]);
+    let commit_c2 = make_commit(0xCC, tree_c2.id(), vec![commit_b2.id().clone()]);
+    let commit_e = make_commit(0xEE, tree_e.id(), vec![commit_c2.id().clone()]);
+
+    // Simulate rebase of "d::" onto "a2"
+    let duplicate_commits = find_duplicate_divergent_commits(
+        tx.repo(),
+        &[commit_a2.id().clone()],
+        &MoveCommitsTarget::Roots(vec![commit_d.id().clone()]),
+    )
+    .unwrap();
+    // Commits b2 and c2 are duplicates
+    assert_eq!(duplicate_commits, &[commit_c2.clone(), commit_b2.clone()]);
+
+    // Simulate rebase of "b1::" onto "e"
+    let duplicate_commits = find_duplicate_divergent_commits(
+        tx.repo(),
+        &[commit_e.id().clone()],
+        &MoveCommitsTarget::Roots(vec![commit_b1.id().clone()]),
+    )
+    .unwrap();
+    // Commits b1 and c1 are duplicates. Commit a2 is not a duplicate, because
+    // it already had a1 as an ancestor before the rebase.
+    assert_eq!(duplicate_commits, &[commit_c1.clone(), commit_b1.clone()]);
+
+    // Simulate rebase of "d | c2 | e" onto "a2"
+    let duplicate_commits = find_duplicate_divergent_commits(
+        tx.repo(),
+        &[commit_a2.id().clone()],
+        &MoveCommitsTarget::Commits(vec![
+            commit_d.id().clone(),
+            commit_c2.id().clone(),
+            commit_e.id().clone(),
+        ]),
+    )
+    .unwrap();
+    // Commit c2 is a duplicate
+    assert_eq!(duplicate_commits, &[commit_c2.clone()]);
+}
